!
! Program to find well defined molecular pairs (such that the closest neighbour of each of the
! two molecules of the pair is the other one)
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Writes:
!	1. File detailing ...
!
!  Last change: 10/2/2012
!

PROGRAM BUILD_CLUSTER	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES,   ONLY : frame_cell, average_cell_volume, pbc_jumps
   USE FORCEFIELD_VARIABLES,   ONLY : atomname
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,       ONLY : TRAJECT_UNWRAP, TRAJECT_JOIN_MOLECULES, TRAJECT_CART_2_FRACT
   USE ORIENTATION_UTILS,      ONLY : DEFINE_MOLECULAR_ORIENTATION_GEN
   USE CONFIG_UTILS,           ONLY : CONFIG_TRANSLATE, CONFIG_ROTATE, CONFIG_CENTEROFMASS
   USE LINEAR_ALGEBRA,         ONLY : NORM, NORMALIZE, CROSS_PRODUCT
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH)     :: traject_file, ff_file, output_file, pdb_file
   INTEGER                            :: I, J, K, I1, I2, ITEMP, J1, J2, JTEMP, UNIT, IOSTAT, index, ntemp, ICLUSTER, ICHECK, IFRAME
   INTEGER                            :: binth, nvector, maxcluster, maxbinth, nframes, com_species, IC1, JC1, IAUX
   REAL ( KIND = DP )                 :: rx, ry, rz, distance, total_weight, temp, closest_distance, closest_to_cluster, dot, sinth
   REAL ( KIND = DP )                 :: dtheta, theta
   REAL ( KIND = DP ), DIMENSION(3)   :: mol_center, sdist, cluster_center, cluster_vector, vector1, vector2
   REAL ( KIND = DP ), DIMENSION(3,3) :: rotation, rot_matrix
  
   INTEGER, ALLOCATABLE, DIMENSION(:)                    :: com_nummols
   INTEGER, ALLOCATABLE, DIMENSION(:,:)                  :: cluster
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)         :: weightmol
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)       :: weight, molecule, cluster_orientation
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:)   :: config_pos, config_com
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr, sss, traj_mol_center
						      

   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file to write histogram of orientations?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,'("Name of output file to write a PDB file with one cluster?")')
   READ (INPUT,'(A)') pdb_file
   WRITE (OUTPUT,'("Maximum number of molecules in the cluster?")')
   READ (INPUT,*) maxcluster
   WRITE (OUTPUT,'("Step in orientation histogram?")')
   READ (INPUT,*) dtheta
   WRITE (OUTPUT,*)
  
   maxbinth = 180.0 / dtheta
   ALLOCATE ( cluster_orientation (maxcluster-2, maxbinth ) ); cluster_orientation = 0.0
   ALLOCATE ( cluster (maxcluster, 2 ) ); cluster = 0

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) )  DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
      WRITE ( OUTPUT, '("System information:")' ) 
      WRITE ( OUTPUT, '("Number of different species = ",I3)') numspecies
      DO I = 1,numspecies
         WRITE ( OUTPUT, "('Molecule ',I3,' contains ',I3,' atoms')" ) I,numatoms(I)
      END DO
      WRITE ( OUTPUT, * )
   END IF 

      
   ! Give desired weights to calculate the COM of each molecule & 3 atoms to define the molecular orientation
   ! Axis 1: Along vector atom1-->atom2
   ! Axis 2: Perpendicular to 1 and in plane formed by the three atoms
   ! Axis 3: Axis 1 x Axis 2
   
   ALLOCATE ( weight ( maxval(numatoms), numspecies ) );   weight = 0.0
   ALLOCATE ( weightmol ( maxval(numatoms) ) );   weightmol = 0.0
   ALLOCATE ( molecule ( 3, maxval(numatoms) ) ); molecule = 0.0
   DO I = 1,numspecies
      WRITE (OUTPUT,'("Molecule type ",I3,": Number of atoms = ",I4)') I,numatoms(I)
      WRITE (OUTPUT,'("Give weights of all atoms ")')
      READ ( INPUT, * ) ( weight(J,I), J=1,numatoms(I))
   END DO
  

   ! Read trajectory and unwrap and re-join molecules

   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   PRINT *,'Finished reading trajectory!'
   IF (pbc_jumps) CALL TRAJECT_UNWRAP (nframes, rrr)  
   CALL TRAJECT_JOIN_MOLECULES (nframes, rrr)


   ! Now calculate trajectory of molecular centers and axes
     
   IF (ALLOCATED(traj_mol_center)) DEALLOCATE(traj_mol_center)
   ALLOCATE ( traj_mol_center ( 3, 1, maxval(nummols), numspecies, nframes ) ); traj_mol_center = 0.0 

   ! Allocate arrays containing positions for a single configuration
   
   IF (ALLOCATED(config_pos))   DEALLOCATE ( config_pos)
   IF (ALLOCATED(config_com)) DEALLOCATE ( config_com)
   ALLOCATE ( config_pos ( 3, maxval(numatoms), maxval(nummols), numspecies ) ); config_pos = 0.0
   ALLOCATE ( config_com ( 3, 1, maxval(nummols), numspecies ) ); config_com = 0.0
   
   DO IFRAME = 1, nframes
      DO I = 1, numspecies
         DO J = 1, nummols(I)   
            molecule(1:3,:) = rrr ( 1:3, :, J, I, IFRAME)
            weightmol = weight(:,I)
            mol_center = 0.0; total_weight = 0.0
            DO K = 1, numatoms(I)
               mol_center(1:3) = mol_center(1:3) + weight(K,I) * molecule(1:3,K)
               total_weight = total_weight + weight(K,I)
            END DO 
            mol_center = mol_center / total_weight
	    traj_mol_center(1:3,1,J,I,IFRAME) = mol_center(1:3)
         END DO	 
      END DO      
   END DO

   ! Calculate trajectory of centers in fractional units
   
   CALL TRAJECT_CART_2_FRACT (nframes, traj_mol_center, sss)


   frames_loop: DO IFRAME = 1,nframes
   
      !First find the closest pair
      
      closest_distance = 1.e6
      
      mol1_species_loop: DO I1 = 1, numspecies
      mol1_molecules_loop: DO J1 = 1,nummols(I1)

         mol2_species_loop: DO I2 = I1, numspecies
         mol2_molecules_loop: DO J2 = 1, nummols(I2)
		        
            !For each molecule find closest molecule, but avoid double counting so J2>J1
	    IF ( (I1 == I2) .AND. (J1 >= J2) ) CYCLE mol2_molecules_loop

	    ! Obtain distance between molecular centers
			
	    sdist= sss(1:3,1,J1,I1,IFRAME) - sss(1:3,1,J2,I2,IFRAME)
	    sdist = sdist - ANINT(sdist)
            rx = sdist(1)*frame_cell(1,1,IFRAME) + sdist(2)*frame_cell(2,1,IFRAME) + sdist(3)*frame_cell(3,1,IFRAME)
            ry = sdist(1)*frame_cell(1,2,IFRAME) + sdist(2)*frame_cell(2,2,IFRAME) + sdist(3)*frame_cell(3,2,IFRAME)
            rz = sdist(1)*frame_cell(1,3,IFRAME) + sdist(2)*frame_cell(2,3,IFRAME) + sdist(3)*frame_cell(3,3,IFRAME)
	    distance = SQRT( rx*rx + ry*ry + rz*rz )
	    IF ( distance < closest_distance ) THEN
	       closest_distance = distance
	       cluster(1,1) = I1
	       cluster(1,2) = J1
	       cluster(2,1) = I2
	       cluster(2,2) = J2
	    END IF 
            			
	 END DO mol2_molecules_loop	
	 END DO mol2_species_loop
      
      
         ! Compute center of mass of pair (cluster) and directing vector
       
         cluster_center = 0.5 * ( traj_mol_center(1:3,1,cluster(1,2),cluster(1,1),IFRAME) + &
                                traj_mol_center(1:3,1,cluster(2,2),cluster(2,1),IFRAME) ) 
				
         cluster_vector = traj_mol_center(1:3,1,cluster(1,2),cluster(1,1),IFRAME) - &
                          traj_mol_center(1:3,1,cluster(2,2),cluster(2,1),IFRAME)
         cluster_vector = NORMALIZE (cluster_vector)						
       
         ! Translate/rotate system so that the cluster is centered at (0,0,0) and oriented along X

         config_pos(:,:,:,:) = rrr(:,:,:,:,IFRAME)
         CALL CONFIG_TRANSLATE (config_pos, -cluster_center)
         rotation(1:3,1) = cluster_vector
         rotation(1:3,2) =  (/ 0.0d0, 1.0d0, 0.0d0 /)
         rotation(1:3,3) =  CROSS_PRODUCT ( rotation(1:3,1), rotation(1:3,2) )
         rotation(1:3,2) =  CROSS_PRODUCT ( rotation(1:3,1), rotation(1:3,3) )
         rot_matrix = TRANSPOSE(rotation)
         CALL CONFIG_ROTATE (config_pos, rot_matrix)
         CALL CONFIG_CENTEROFMASS (config_pos, config_com, com_species, com_nummols)
       
       
         ! Now build cluster up to specified number of molecules
       
         DO ICLUSTER = 3,maxcluster
          
	    closest_to_cluster = 1.e6
	  
            spec_cluster_loop: DO IC1 = 1, numspecies
            mol_cluster_loop:  DO JC1 = 1,nummols(IC1)
	  
	       DO ICHECK = 1, ICLUSTER-1
	          IF ( (IC1 == cluster(ICHECK,1)) .AND. (JC1 == cluster(ICHECK,2)) ) CYCLE mol_cluster_loop
	       END DO
	     
	       !Find closest molecule to cluster center
	    
	        distance = NORM(config_com(1:3,1,JC1,IC1)) 
	        IF ( distance < closest_to_cluster ) THEN
	           closest_to_cluster = distance
	           cluster(ICLUSTER,1) = IC1
	           cluster(ICLUSTER,2) = JC1
	        END IF 
	     
	    END DO mol_cluster_loop
	    END DO spec_cluster_loop
	    
	    ! Check
	    
	    index = 0
	    DO IAUX = 1, numatoms(cluster(1,1))
	       index = index + 1
	       WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', index, atomname(IAUX, cluster(1,1)), 'MOL', &
	       1, rrr(1:3, IAUX, cluster(1,2), cluster(1,1), IFRAME), 0.0, 0.0, 'XXX' 
	    END DO
	    DO IAUX = 1, numatoms(cluster(2,1))
	       index = index + 1
	       WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', index, atomname(IAUX, cluster(2,1)), 'MOL', &
	       1, rrr(1:3, IAUX, cluster(2,2), cluster(2,1), IFRAME), 0.0, 0.0, 'XXX' 
	    END DO
	    DO IAUX = 1, numatoms(cluster(3,1))
	       index = index + 1
	       WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', index, atomname(IAUX, cluster(3,1)), 'MOL', &
	       1, rrr(1:3, IAUX, cluster(3,2), cluster(3,1), IFRAME), 0.0, 0.0, 'XXX' 
	    END DO
	    STOP
      
            ! Determine relative orientation of the last molecules added to the cluster
	    ! As config is centered and the first molecule is oriented along (1,0,0) I compute
	    ! the angle of the vector between the center and the COM of the new molecule and
	    ! the vector between the center and the COM of the 1st molecule. 
            
            vector1 = NORMALIZE(config_com(1:3,1,cluster(ICLUSTER,2),cluster(ICLUSTER,1)))
            vector2 = NORMALIZE(config_com(1:3,1,cluster(1,2),cluster(1,1)))
	    dot = DOT_PRODUCT ( vector1, vector2 )
            theta = ACOS(dot)*180.0/pi
	    print *,'neighbour at ',closest_to_cluster,theta
            binth = INT(theta/dtheta) + 1
            !weight each point by 1/sin(theta) to take into account solid angle
            sinth = SIN(ACOS(dot))
            IF (sinth < 1.0e-3) sinth = 1.0e-3
            cluster_orientation(ICLUSTER-2,binth) = cluster_orientation(ICLUSTER-2,binth) + (1./sinth)
       
         END DO       
      
         !IF (IFRAME == nframes .AND. (J1 == 1) ) THEN
         !   index = 0
         !   UNIT = NEXT_UNIT ( )
         !   OPEN ( UNIT, FILE = pdb_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
         !   IF ( IOSTAT /= 0 ) STOP 'CONFIG_PDB_WRITE: Problem opening file!'
         !   DO ICLUSTER = 1,maxcluster
	 !      index = index + 1
         !      WRITE (UNIT,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', index, 'He', 'XXX', I, &
	 !       &config_com(1,1,cluster(ICLUSTER,2),cluster(ICLUSTER,1) ), &
	 !       &config_com(2,1,cluster(ICLUSTER,2),cluster(ICLUSTER,1) ), &
	 !       &config_com(3,1,cluster(ICLUSTER,2),cluster(ICLUSTER,1) ), 0.0, 0.0, 'XXX' 
         !      DO J = 1,numatoms(cluster(ICLUSTER,1))
	 !         index = index + 1
         !         WRITE (UNIT,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', index, &
	 !          &atomname(J,cluster(ICLUSTER,1)), 'XXX', I, &
	 !	   &config_pos(1,J,cluster(ICLUSTER,2),cluster(ICLUSTER,1) ), &
	 !          &config_pos(2,J,cluster(ICLUSTER,2),cluster(ICLUSTER,1) ), &
	 !	   &config_pos(3,J,cluster(ICLUSTER,2),cluster(ICLUSTER,1) ), 0.0, 0.0, 'XXX' 
         !      END DO
	 !   END DO
	 !   CLOSE (UNIT)
         !END IF
	 
      END DO mol1_molecules_loop	
      END DO mol1_species_loop	
      
   END DO frames_loop

   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'Output file for orientation histogram could not be created!'
   WRITE (UNIT,'("# Orientation distribution for each member of the cluster")')
   DO J = 1, maxbinth
       temp = float(J-1)*dtheta + 0.5*dtheta
       WRITE (UNIT,'(F5.1,5x,20(F8.3,1x))') temp, (cluster_orientation(I,J)/float(nframes),I=1,maxcluster-2)
   END DO
   CLOSE (UNIT)
      
END PROGRAM BUILD_CLUSTER
