!
! Program to find well defined molecular pairs (such that the closest neighbour of each of the
! two molecules of the pair is the other one)
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Writes:
!	1. File detailing ...
!
!  Last change: 14/12/2013
!

PROGRAM CLUSTER_ORIENTATIONAL_DISTRIBUTION	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES,   ONLY : frame_cell, average_cell_volume, pbc_jumps
   USE FORCEFIELD_VARIABLES,   ONLY : atomname
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,       ONLY : TRAJECT_UNWRAP, TRAJECT_JOIN_MOLECULES, TRAJECT_CART_2_FRACT
   USE ORIENTATION_UTILS,      ONLY : DEFINE_MOLECULAR_ORIENTATION_GEN
   USE CONFIG_UTILS,           ONLY : CONFIG_TRANSLATE, CONFIG_ROTATE, CONFIG_CENTEROFMASS
   USE LINEAR_ALGEBRA,         ONLY : NORM, NORMALIZE, CROSS_PRODUCT
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH)     :: traject_file, ff_file, output_file_1, output_file_2
   INTEGER                            :: IFRAME, I, J, K, I1, I2, J1, J2, UNIT, IOSTAT
   INTEGER                            :: nframes, selected_species, progress, maxneighbour
   INTEGER                            :: bin, binth, dist_index, ntemp, maxbin, maxbinth, nvector, pair_spec, pair_molec
   INTEGER, DIMENSION(3)              :: triademol
   REAL ( KIND = DP ), DIMENSION(3)   :: mol_center, sdist, pair_cm, pair_vector, newsss, newr
   REAL ( KIND = DP ), DIMENSION(3,3) :: mol_axes


   REAL ( KIND = DP ) :: dr, dtheta, rx, ry, rz, dot, theta, sinth, total_weight, temp, n1thmin, n1thmax, r, rmin
  
   INTEGER, ALLOCATABLE, DIMENSION(:,:)                  :: triade, neighbours, neighbour_population, neighbour_distance
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)         :: rneighbour, weightmol, stemp
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)       :: weight, molecule, neighbour_orientation
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr, sss, traj_mol_center, traj_mol_axes
						      

   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file to store the distance distribution for each neighbour?")')
   READ (INPUT,'(A)') output_file_1
   WRITE (OUTPUT,'("Name of output file to store the orientation distribution for each neighbour?")')
   READ (INPUT,'(A)') output_file_2
   WRITE (OUTPUT,'("Maximum number of neighbours?")')
   READ (INPUT,*) maxneighbour
   WRITE (OUTPUT,'("Step in distance histogram?")')
   READ (INPUT,*) dr
   WRITE (OUTPUT,'("Step in orientation histogram?")')
   READ (INPUT,*) dtheta
   WRITE (OUTPUT,'("Vector to use in orientation calculation (1, 2 or 3)?")')
   READ (INPUT,*) nvector
   IF (nvector < 1) THEN
      PRINT *, 'Using nvector = 1'
      nvector = 1
   END IF
   IF (nvector > 3) THEN
      PRINT *, 'Using nvector = 3'
      nvector = 3
   END IF
   WRITE (OUTPUT,*)
  
   maxbinth = 180.0 / dtheta

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) )  DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
      WRITE ( OUTPUT, '("System information:")' ) 
      WRITE ( OUTPUT, '("Number of different species = ",I3)') numspecies
      DO I = 1,numspecies
         WRITE ( OUTPUT, "('Molecule ',I3,' contains ',I3,' atoms')" ) I,numatoms(I)
      END DO
      WRITE ( OUTPUT, * )
   END IF 

      
   ! Give desired weights to calculate the COM of each molecule & 3 atoms to define the molecular orientation
   ! Axis 1: Along vector atom1-->atom2
   ! Axis 2: Perpendicular to 1 and in plane formed by the three atoms
   ! Axis 3: Axis 1 x Axis 2
   
   ALLOCATE ( weight ( maxval(numatoms), numspecies ) );   weight = 0.0
   ALLOCATE ( weightmol ( maxval(numatoms) ) );   weightmol = 0.0
   ALLOCATE ( triade ( 3, numspecies ) );   triade = 0
   ALLOCATE ( molecule ( 3, maxval(numatoms) ) ); molecule = 0.0
   DO I = 1,numspecies
      WRITE (OUTPUT,'("Molecule type ",I3,": Number of atoms = ",I4)') I,numatoms(I)
      WRITE (OUTPUT,'("Give weights of all atoms ")')
      READ ( INPUT, * ) ( weight(J,I), J=1,numatoms(I))
      IF (numatoms(I) >= 3) THEN
         WRITE (OUTPUT,'("Give numbers of 3 atoms to define the molecular orientation")')
	 READ ( INPUT, * ) (triade(J,I), J=1,3)
      END IF
   END DO
   WRITE (OUTPUT,'("Number of the selected species? ")')
   READ (INPUT,*) selected_species
   IF (selected_species > numspecies) STOP 'Error! Not so many species in trajectory'
  
   WRITE (OUTPUT,'("Angles to select first neighbour?")')
   READ (INPUT,*) n1thmin, n1thmax

   ! Read trajectory and unwrap and re-join molecules

   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   PRINT *,'Finished reading trajectory!'
   IF (pbc_jumps) CALL TRAJECT_UNWRAP (nframes, rrr)  
   CALL TRAJECT_JOIN_MOLECULES (nframes, rrr)


   ! Now calculate trajectory of molecular centers and axes
     
   IF (ALLOCATED(traj_mol_center)) DEALLOCATE(traj_mol_center)
   IF (ALLOCATED(traj_mol_axes))   DEALLOCATE(traj_mol_axes)
   ALLOCATE ( traj_mol_center ( 3, 1, maxval(nummols), numspecies, nframes ) ); traj_mol_center = 0.0 
   ALLOCATE ( traj_mol_axes ( 3, 3, maxval(nummols), numspecies, nframes ) );   traj_mol_axes = 0.0

   ! Allocate arrays containing positions for a single configuration
      
   DO IFRAME = 1, nframes

      DO I = 1, numspecies

         DO J = 1, nummols(I)
   
            molecule(1:3,:) = rrr ( 1:3, :, J, I, IFRAME)
            weightmol = weight(:,I)
            triademol = triade(:,I)

            IF (numatoms(I) >= 3) THEN

	       CALL DEFINE_MOLECULAR_ORIENTATION_GEN (molecule, weightmol, triademol, mol_center, mol_axes)

            ELSE

               mol_center = 0.0; total_weight = 0.0
               DO K = 1, numatoms(I)
                  mol_center(1:3) = mol_center(1:3) + weight(K,I) * molecule(1:3,K)
                  total_weight = total_weight + weight(K,I)
               END DO 
               mol_center = mol_center / total_weight
               IF (numatoms(I) == 2) THEN
                  ! Define first axis as bond between both atoms
                  mol_axes(1:3,1) = molecule(1:3,2) - molecule(1:3,1)	 
                  mol_axes(1:3,1) = NORMALIZE( mol_axes(1:3,1) )
               ELSE 
                  ! Use the cartesian axes
                  mol_axes(1:3,1) =  (/ 1.0d0, 0.0d0, 0.0d0 /)
               END IF
               mol_axes(1:3,2) =  (/ 0.0d0, 1.0d0, 0.0d0 /)
               mol_axes(1:3,3) =  (/ 0.0d0, 0.0d0, 1.0d0 /)

            END IF

	    traj_mol_center(1:3,1,J,I,IFRAME) = mol_center(1:3)
	    traj_mol_axes(1:3,1:3,J,I,IFRAME) = mol_axes(1:3,1:3)

         END DO	 

      END DO
      
   END DO

   ! Calculate trajectory of centers in fractional units
   
   CALL TRAJECT_CART_2_FRACT (nframes, traj_mol_center, sss)

   
   ! Allocate arrays to contain neighbour distances and list of neighbours & Compute histograms
   
   ALLOCATE ( rneighbour ( maxneighbour+1 ) ); rneighbour = 0.0 
   ALLOCATE ( neighbours (maxneighbour+1, 2 ) ); neighbours = 0
   ALLOCATE ( neighbour_population (maxneighbour, numspecies ) ); neighbour_population = 0
   ALLOCATE ( neighbour_distance (maxneighbour, 9999 ) ); neighbour_distance = 0
   ALLOCATE ( neighbour_orientation (maxneighbour, maxbinth ) ); neighbour_orientation = 0.0
   maxbin = 0
   maxbinth = 180.0 / dtheta

   frames_loop: DO IFRAME = 1,nframes

      progress = nframes / 20
      IF (progress < 1) progress = 1
      IF (MOD(IFRAME,progress) == 0) PRINT *,'Neighbour distribution. Completed ',(100*IFRAME/nframes),' %'

      mol1_species_loop: DO I1 = selected_species,selected_species
      mol1_molecules_loop: DO J1 = 1,nummols(I1)

         ! Set to zero neighbour distances and lists for the current molecule
         rmin = 9.999e9
         pair_spec = 0
	 pair_molec = 0

         mol2_species_loop: DO I2 = 1,numspecies
         mol2_molecules_loop: DO J2 = 1,nummols(I2)
		        
	    IF ( (I1 == I2) .AND. (J1 == J2) ) CYCLE mol2_molecules_loop
			
	    ! Obtain distance between molecular centers and find closer one
			
	    sdist= sss(1:3,1,J1,I1,IFRAME) - sss(1:3,1,J2,I2,IFRAME)
	    sdist = sdist - ANINT(sdist)
            rx = sdist(1)*frame_cell(1,1,IFRAME) + sdist(2)*frame_cell(2,1,IFRAME) + sdist(3)*frame_cell(3,1,IFRAME)
            ry = sdist(1)*frame_cell(1,2,IFRAME) + sdist(2)*frame_cell(2,2,IFRAME) + sdist(3)*frame_cell(3,2,IFRAME)
            rz = sdist(1)*frame_cell(1,3,IFRAME) + sdist(2)*frame_cell(2,3,IFRAME) + sdist(3)*frame_cell(3,3,IFRAME)
            r = SQRT( rx*rx + ry*ry + rz*rz )
	    IF (r < rmin) THEN
	       rmin = r
	       pair_spec = I2
	       pair_molec = J2
	    ENDIF
            			
	 END DO mol2_molecules_loop	
	 END DO mol2_species_loop
	 
         !Check orientation and select first neighbour
         dot = DOT_PRODUCT ( traj_mol_axes(1:3,nvector,J1,I1,IFRAME), traj_mol_axes(1:3,nvector,pair_molec,pair_spec,IFRAME) )
	 theta = ACOS(dot)*180.0/pi
	 
         !Check pair structure
	 
	IF (pair_molec == 955) THEN

	    PRINT *, 'Central pair:', I1, J1, pair_spec, pair_molec
	   PRINT *, 'r= ', rmin, ' angle = ', theta 
	   
	    PRINT *, 'Positions molecule 1:'
	    DO I = 1,10
	       WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', I, 'C ',  'MOL', &
		      1, rrr(1:3,I,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
	    END DO
	    PRINT *, 'Positions molecule 2:'
	    DO I = 1,10
	       WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', I, 'C ',  'MOL', &
		      2, rrr(1:3,I,pair_molec,pair_spec,IFRAME), 0.0, 0.0, 'XXX' 
	    END DO

	    PRINT *, 'CM and orientation molecule 1:'
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 1, 'HM',  'MOL', &
		   1, traj_mol_center(1:3,1,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2, 'H ',  'MOL', &
		      1, traj_mol_center(1:3,1,J1,I1,IFRAME)+1.6*traj_mol_axes(1:3,1,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 3, 'O ',  'MOL', &
		      1, traj_mol_center(1:3,1,J1,I1,IFRAME)+1.6*traj_mol_axes(1:3,2,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 4, 'N ',  'MOL', &
		      1, traj_mol_center(1:3,1,J1,I1,IFRAME)+1.6*traj_mol_axes(1:3,3,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
	    PRINT *, 'CM and orientation molecule 2:'
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 1, 'HM',  'MOL', &
		   2, traj_mol_center(1:3,1,pair_molec,pair_spec,IFRAME), 0.0, 0.0, 'XXX' 
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2, 'H ',  'MOL', &
		      2, traj_mol_center(1:3,1,pair_molec,pair_spec,IFRAME)+1.6*traj_mol_axes(1:3,1,pair_molec,pair_spec,IFRAME), &
		      0.0, 0.0, 'XXX' 
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 3, 'O ',  'MOL', &
		      2, traj_mol_center(1:3,1,pair_molec,pair_spec,IFRAME)+1.6*traj_mol_axes(1:3,2,pair_molec,pair_spec,IFRAME), &
		      0.0, 0.0, 'XXX' 
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 4, 'N ',  'MOL', &
		      2, traj_mol_center(1:3,1,pair_molec,pair_spec,IFRAME)+1.6*traj_mol_axes(1:3,3,pair_molec,pair_spec,IFRAME), &
		      0.0, 0.0, 'XXX' 

	    PRINT *, 'Axis molecule 1:'
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 1, 'CM',  'MOL', &
		   1, 0.0, 0.0, 0.0, 0.0, 0.0, 'XXX' 
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2, 'H ',  'MOL', &
		      1, 3.1*traj_mol_axes(1:3,1,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 3, 'O ',  'MOL', &
		      1, 3.1*traj_mol_axes(1:3,2,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 4, 'N ',  'MOL', &
		      1, 3.1*traj_mol_axes(1:3,3,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
	    PRINT *, 'Axis molecule 2:'
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 1, 'CM',  'MOL', &
		   2, 0.0, 0.0, 0.0, 0.0, 0.0, 'XXX' 
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2, 'H ',  'MOL', &
		      2, 5.1*traj_mol_axes(1:3,1,pair_molec,pair_spec,IFRAME), &
		      0.0, 0.0, 'XXX' 
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 3, 'O ',  'MOL', &
		      2, 5.1*traj_mol_axes(1:3,2,pair_molec,pair_spec,IFRAME), &
		      0.0, 0.0, 'XXX' 
	    WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 4, 'N ',  'MOL', &
		      2, 5.1*traj_mol_axes(1:3,3,pair_molec,pair_spec,IFRAME), &
		      0.0, 0.0, 'XXX' 

	    pair_cm = 0.5 * ( sss(1:3,1,J1,I1,IFRAME) + sss(1:3,1,pair_molec,pair_spec,IFRAME) )
	    pair_vector = sss(1:3,1,pair_molec,pair_spec,IFRAME) - sss(1:3,1,J1,I1,IFRAME)
	    print *, 'pair vector = ', pair_vector, NORM(pair_vector)
	    dot = DOT_PRODUCT ( pair_vector, traj_mol_axes(1:3,nvector,J1,I1,IFRAME) ) / NORM(pair_vector)
	    theta = ACOS(dot)*180.0/pi
	    print *, 'theta with molecule 1 = ',theta
	    dot = DOT_PRODUCT ( pair_vector, traj_mol_axes(1:3,nvector,pair_molec,pair_spec,IFRAME) ) / NORM(pair_vector)
	    theta = ACOS(dot)*180.0/pi
	    print *, 'theta with molecule 2 = ',theta
	    pair_vector = traj_mol_center(1:3,1,pair_molec,pair_spec,IFRAME) - traj_mol_center(1:3,1,J1,I1,IFRAME)
	    print *, 'pair vector = ', pair_vector, NORM(pair_vector)
	    dot = DOT_PRODUCT ( pair_vector, traj_mol_axes(1:3,nvector,J1,I1,IFRAME) ) / NORM(pair_vector)
	    theta = ACOS(dot)*180.0/pi
	    print *, 'theta with molecule 1 = ',theta
	    dot = DOT_PRODUCT ( pair_vector, traj_mol_axes(1:3,nvector,pair_molec,pair_spec,IFRAME) ) / NORM(pair_vector)
	    theta = ACOS(dot)*180.0/pi
	    print *, 'theta with molecule 2 = ',theta
	    STOP		      

	 END IF

	 IF ((theta >= n1thmin) .AND. (theta <= n1thmax)) THEN

	    !Reference point now is CM of pair of neighbours and reference vector is vector joining the CMs of both molecules

	    sdist= sss(1:3,1,J1,I1,IFRAME) - sss(1:3,1,pair_molec,pair_spec,IFRAME)
	    newsss = sss(1:3,1,pair_molec,pair_spec,IFRAME) + ANINT(sdist)
	    pair_cm = 0.5 * ( sss(1:3,1,J1,I1,IFRAME) + newsss )
	    pair_vector = newsss - sss(1:3,1,J1,I1,IFRAME)
	 
	    !Now look for closer molecules to pair_cm and get distribution between their axes and pair_vector

            ! Set to zero neighbour distances and lists for the current molecule
            rneighbour = 0.0
            neighbours = 0
            dist_index = 0
	 
            mol2_species_loop2: DO I2 = 1,numspecies
            mol2_molecules_loop2: DO J2 = 1,nummols(I2)

	       IF ( (I2 == I1) .AND. (J2 == J1 ) ) CYCLE mol2_molecules_loop2
	       IF ( (I2 == pair_spec) .AND. (J2 == pair_molec ) ) CYCLE mol2_molecules_loop2
		        
               dist_index = dist_index + 1
	    
	       sdist= pair_cm - sss(1:3,1,J2,I2,IFRAME)
	       sdist = sdist - ANINT(sdist)
               rx = sdist(1)*frame_cell(1,1,IFRAME) + sdist(2)*frame_cell(2,1,IFRAME) + sdist(3)*frame_cell(3,1,IFRAME)
               ry = sdist(1)*frame_cell(1,2,IFRAME) + sdist(2)*frame_cell(2,2,IFRAME) + sdist(3)*frame_cell(3,2,IFRAME)
               rz = sdist(1)*frame_cell(1,3,IFRAME) + sdist(2)*frame_cell(2,3,IFRAME) + sdist(3)*frame_cell(3,3,IFRAME)
               rneighbour(dist_index) = SQRT( rx*rx + ry*ry + rz*rz )
               neighbours(dist_index,1) = I2
               neighbours(dist_index,2) = J2

               ! Simple sorting
            
               DO I = 1,dist_index-1
                  DO J = I+1,dist_index
                     IF ( rneighbour(I) > rneighbour(J) ) THEN
                        temp = rneighbour(J); rneighbour(J)=rneighbour(I); rneighbour(I)=temp
                        ntemp = neighbours(J,1); neighbours(J,1)=neighbours(I,1); neighbours(I,1)=ntemp 
                        ntemp = neighbours(J,2); neighbours(J,2)=neighbours(I,2); neighbours(I,2)=ntemp 
                     END IF
                  END DO
               END DO		     
               IF (dist_index > maxneighbour) dist_index = maxneighbour
            			
	    END DO mol2_molecules_loop2	
	    END DO mol2_species_loop2
	    

            !Calculate distances and orientations for nearest neighbours

            DO I = 1, maxneighbour

               bin = INT(rneighbour(I)/dr) + 1
               IF (bin > 9999) STOP 'Too large distance!'
               IF (bin > maxbin) maxbin = bin 
               neighbour_distance(I,bin) = neighbour_distance(I,bin) + 1
	       dot = DOT_PRODUCT ( pair_vector, traj_mol_axes(1:3,nvector,neighbours(I,2),neighbours(I,1),IFRAME) ) / &
		    NORM(pair_vector)
	      theta = ACOS(dot)*180.0/pi
	      binth = INT(theta/dtheta) + 1
	      !weight each point by 1/sin(theta) to take into account solid angle
	      sinth = SIN(ACOS(dot))
	      IF (sinth < 1.0e-3) sinth = 1.0e-3
	       neighbour_orientation(I,binth) = neighbour_orientation(I,binth) + (1./sinth)

            END DO
	    
	    ! Special case: Very close 1st and 2nd neighbours
	    
	    IF ( rneighbour(2)< 6.0) THEN
	    
!		PRINT *, 'Central pair:', I1, J1, pair_spec, pair_molec
!		PRINT *, 'r= ', rmin, ' angle = ', theta 
!		PRINT *, '1st neighbour at ', rneighbour(1), ' is ',neighbours(1,1), neighbours(1,2)
!		PRINT *, '2nd neighbour at ', rneighbour(2), ' is ',neighbours(2,1), neighbours(2,2)
!		PRINT *, 'CM of pair  = ', pair_cm
!		PRINT *, 'CM of 1st n = ', sss(1:3,1,neighbours(1,2),neighbours(1,1),IFRAME)
!		PRINT *, 'CM of 2nd n = ', sss(1:3,1,neighbours(2,2),neighbours(2,1),IFRAME)
!		PRINT *, 'CM of 1st n = ', traj_mol_center(1:3,1,neighbours(1,2),neighbours(1,1),IFRAME)
!		PRINT *, 'CM of 2nd n = ', traj_mol_center(1:3,1,neighbours(2,2),neighbours(2,1),IFRAME)
!	    
!		PRINT *, 'Positions molecule 1:'
!		DO I = 1,10
!		   WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', I, 'C ',  'MOL', &
!			 1, rrr(1:3,I,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
!		END DO
!		PRINT *, 'Positions molecule 2:'
!		DO I = 1,10
!		   WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', I, 'C ',  'MOL', &
!			 2, rrr(1:3,I,pair_molec,pair_spec,IFRAME), 0.0, 0.0, 'XXX' 
!		END DO
!		PRINT *, 'Positions 1st neighbour to CM of pair:'
!		DO I = 1,10
!		   WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', I, 'C ',  'MOL', &
!			 3, rrr(1:3,I,neighbours(1,2),neighbours(1,1),IFRAME), 0.0, 0.0, 'XXX' 
!		END DO   
!		PRINT *, 'Positions 2nd neighbour to CM of pair:'
!		DO I = 1,10
!		   WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', I, 'C ',  'MOL', &
!			 4, rrr(1:3,I,neighbours(2,2),neighbours(2,1),IFRAME), 0.0, 0.0, 'XXX' 
!		END DO   
!
!		PRINT *, 'CM and orientation molecule 1:'
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 1, 'HM',  'MOL', &
!		      1, traj_mol_center(1:3,1,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2, 'H ',  'MOL', &
!		      1, traj_mol_center(1:3,1,J1,I1,IFRAME) + 1.2*traj_mol_axes(1:3,1,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 3, 'O ',  'MOL', &
!		      1, traj_mol_center(1:3,1,J1,I1,IFRAME) + 1.4*traj_mol_axes(1:3,2,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 4, 'N ',  'MOL', &
!		      1, traj_mol_center(1:3,1,J1,I1,IFRAME) + 1.4*traj_mol_axes(1:3,3,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
!
!		PRINT *, 'CM and orientation molecule 2:'
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 1, 'HM',  'MOL', &
!		      2, traj_mol_center(1:3,1,pair_molec,pair_spec,IFRAME), 0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2, 'H ',  'MOL', &
!		      2, traj_mol_center(1:3,1,pair_molec,pair_spec,IFRAME) + 1.2*traj_mol_axes(1:3,1,pair_molec,pair_spec,IFRAME), &
!		      0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 3, 'O ',  'MOL', &
!		      2, traj_mol_center(1:3,1,pair_molec,pair_spec,IFRAME) + 1.4*traj_mol_axes(1:3,2,pair_molec,pair_spec,IFRAME), &
!		      0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 4, 'N ',  'MOL', &
!		      2, traj_mol_center(1:3,1,pair_molec,pair_spec,IFRAME) + 1.4*traj_mol_axes(1:3,3,pair_molec,pair_spec,IFRAME), &
!		      0.0, 0.0, 'XXX' 
!
!		PRINT *, 'CM and orientation 1st neighbour:'
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 1, 'HM',  'MOL', &
!		      3, traj_mol_center(1:3,1,neighbours(1,2),neighbours(1,1),IFRAME), 0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2, 'H ',  'MOL', &
!		      3, traj_mol_center(1:3,1,neighbours(1,2),neighbours(1,1),IFRAME) + &
!		      1.2*traj_mol_axes(1:3,1,neighbours(1,2),neighbours(1,1),IFRAME), &
!		      0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 3, 'O ',  'MOL', &
!		      3, traj_mol_center(1:3,1,neighbours(1,2),neighbours(1,1),IFRAME) + &
!		      1.4*traj_mol_axes(1:3,2,neighbours(1,2),neighbours(1,1),IFRAME), &
!		      0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 4, 'N ',  'MOL', &
!		      3, traj_mol_center(1:3,1,neighbours(1,2),neighbours(1,1),IFRAME) + &
!		      1.4*traj_mol_axes(1:3,3,neighbours(1,2),neighbours(1,1),IFRAME), &
!		      0.0, 0.0, 'XXX' 
!
!		PRINT *, 'CM and orientation 2nd neighbour:'
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 1, 'HM',  'MOL', &
!		      4, traj_mol_center(1:3,1,neighbours(2,2),neighbours(2,1),IFRAME), 0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2, 'H ',  'MOL', &
!		      4, traj_mol_center(1:3,1,neighbours(2,2),neighbours(2,1),IFRAME) + &
!		      1.2*traj_mol_axes(1:3,1,neighbours(2,2),neighbours(2,1),IFRAME), &
!		      0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 3, 'O ',  'MOL', &
!		      4, traj_mol_center(1:3,1,neighbours(2,2),neighbours(2,1),IFRAME) + &
!		      1.4*traj_mol_axes(1:3,2,neighbours(2,2),neighbours(2,1),IFRAME), &
!		      0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 4, 'N ',  'MOL', &
!		      4, traj_mol_center(1:3,1,neighbours(2,2),neighbours(2,1),IFRAME) + &
!		      1.4*traj_mol_axes(1:3,3,neighbours(2,2),neighbours(2,1),IFRAME), &
!		      0.0, 0.0, 'XXX' 
!		      
!		PRINT *, 'Scaled CM and orientation molecule 1:'
!		rx = sss(1,1,J1,I1,IFRAME)*frame_cell(1,1,IFRAME) + &
!		     sss(2,1,J1,I1,IFRAME)*frame_cell(2,1,IFRAME) + &
!		     sss(3,1,J1,I1,IFRAME)*frame_cell(3,1,IFRAME)
!		ry = sss(1,1,J1,I1,IFRAME)*frame_cell(1,2,IFRAME) + &
!		     sss(2,1,J1,I1,IFRAME)*frame_cell(2,2,IFRAME) + &
!		     sss(3,1,J1,I1,IFRAME)*frame_cell(3,2,IFRAME)
!		rz = sss(1,1,J1,I1,IFRAME)*frame_cell(1,3,IFRAME) + &
!		     sss(2,1,J1,I1,IFRAME)*frame_cell(2,3,IFRAME) + &
!		     sss(3,1,J1,I1,IFRAME)*frame_cell(3,3,IFRAME)
!		newr(1) = rx
!		newr(2) = ry
!		newr(3) = rz	     
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 1, 'HM',  'MOL', &
!		      1, newr, 0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2, 'H ',  'MOL', &
!		      1, newr + 1.6*traj_mol_axes(1:3,1,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 3, 'O ',  'MOL', &
!		      1, newr + 1.6*traj_mol_axes(1:3,2,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 4, 'N ',  'MOL', &
!		      1, newr + 1.6*traj_mol_axes(1:3,3,J1,I1,IFRAME), 0.0, 0.0, 'XXX' 
!
!		PRINT *, 'Scaled CM and orientation molecule 2:'
!		sdist= sss(1:3,1,J1,I1,IFRAME) - sss(1:3,1,pair_molec,pair_spec,IFRAME)
!		newsss = sss(1:3,1,pair_molec,pair_spec,IFRAME) + ANINT(sdist)
!		rx = newsss(1)*frame_cell(1,1,IFRAME) + &
!		     newsss(2)*frame_cell(2,1,IFRAME) + &
!		     newsss(3)*frame_cell(3,1,IFRAME)
!		ry = newsss(1)*frame_cell(1,2,IFRAME) + &
!		     newsss(2)*frame_cell(2,2,IFRAME) + &
!		     newsss(3)*frame_cell(3,2,IFRAME)
!		rz = newsss(1)*frame_cell(1,3,IFRAME) + &
!		     newsss(2)*frame_cell(2,3,IFRAME) + &
!		     newsss(3)*frame_cell(3,3,IFRAME)
!		newr(1) = rx
!		newr(2) = ry
!		newr(3) = rz	     
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 1, 'HM',  'MOL', &
!		      2, newr, 0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2, 'H ',  'MOL', &
!		      2, newr + 1.6*traj_mol_axes(1:3,1,pair_molec,pair_spec,IFRAME), &
!		      0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 3, 'O ',  'MOL', &
!		      2, newr + 1.6*traj_mol_axes(1:3,2,pair_molec,pair_spec,IFRAME), &
!		       0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 4, 'N ',  'MOL', &
!		      2, newr + 1.6*traj_mol_axes(1:3,3,pair_molec,pair_spec,IFRAME), &
!		       0.0, 0.0, 'XXX' 
!
!		PRINT *, 'Scaled CM and orientation 1st neighbour:'
!		sdist= pair_cm - sss(1:3,1,neighbours(1,2),neighbours(1,1),IFRAME)
!		newsss = sss(1:3,1,neighbours(1,2),neighbours(1,1),IFRAME) - ANINT(sdist)
!		rx = newsss(1)*frame_cell(1,1,IFRAME) + &
!		     newsss(2)*frame_cell(2,1,IFRAME) + &
!		     newsss(3)*frame_cell(3,1,IFRAME)
!		ry = newsss(1)*frame_cell(1,2,IFRAME) + &
!		     newsss(2)*frame_cell(2,2,IFRAME) + &
!		     newsss(3)*frame_cell(3,2,IFRAME)
!		rz = newsss(1)*frame_cell(1,3,IFRAME) + &
!		     newsss(2)*frame_cell(2,3,IFRAME) + &
!		     newsss(3)*frame_cell(3,3,IFRAME)
!		newr(1) = rx
!		newr(2) = ry
!		newr(3) = rz	     
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 1, 'HM',  'MOL', &
!		      3, newr, 0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2, 'H ',  'MOL', &
!		      3, newr + 1.6*traj_mol_axes(1:3,1,neighbours(1,2),neighbours(1,1),IFRAME), &
!		      0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 3, 'O ',  'MOL', &
!		      3, newr + 1.6*traj_mol_axes(1:3,2,neighbours(1,2),neighbours(1,1),IFRAME), &
!		      0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 4, 'N ',  'MOL', &
!		      3, newr + 1.6*traj_mol_axes(1:3,3,neighbours(1,2),neighbours(1,1),IFRAME), &
!		      0.0, 0.0, 'XXX' 
!
!		PRINT *, 'Scaled CM and orientation 2nd neighbour:'
!		sdist= pair_cm - sss(1:3,1,neighbours(2,2),neighbours(2,1),IFRAME)
!		newsss = sss(1:3,1,neighbours(2,2),neighbours(2,1),IFRAME) + ANINT(sdist)
!		rx = newsss(1)*frame_cell(1,1,IFRAME) + &
!		     newsss(2)*frame_cell(2,1,IFRAME) + &
!		     newsss(3)*frame_cell(3,1,IFRAME)
!		ry = newsss(1)*frame_cell(1,2,IFRAME) + &
!		     newsss(2)*frame_cell(2,2,IFRAME) + &
!		     newsss(3)*frame_cell(3,2,IFRAME)
!		rz = newsss(1)*frame_cell(1,3,IFRAME) + &
!		     newsss(2)*frame_cell(2,3,IFRAME) + &
!		     newsss(3)*frame_cell(3,3,IFRAME)
!		newr(1) = rx
!		newr(2) = ry
!		newr(3) = rz	     
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 1, 'HM',  'MOL', &
!		      3, newr, 0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2, 'H ',  'MOL', &
!		      3, newr + 1.6*traj_mol_axes(1:3,1,neighbours(2,2),neighbours(2,1),IFRAME), &
!		      0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 3, 'O ',  'MOL', &
!		      3, newr + 1.6*traj_mol_axes(1:3,2,neighbours(2,2),neighbours(2,1),IFRAME), &
!		      0.0, 0.0, 'XXX' 
!		WRITE (*,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 4, 'N ',  'MOL', &
!		      3, newr + 1.6*traj_mol_axes(1:3,3,neighbours(2,2),neighbours(2,1),IFRAME), &
!		      0.0, 0.0, 'XXX' 
!
!		STOP	      

	    END IF
	    
	 END IF   

      END DO mol1_molecules_loop	
      END DO mol1_species_loop  

   END DO frames_loop

   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file_1, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'Output file for distance histogram could not be created!'
   WRITE (UNIT,'("# Distance distribution for each neighbour")')
   WRITE (UNIT,'("#",3x,"r - Neighbour order")')
   DO J = 1, maxbin
       temp = float(J-1)*dr + 0.5*dr
       WRITE (UNIT,'(F5.2,5x,20(I6,1x))') temp, (neighbour_distance(I,J),I=1,maxneighbour)
   END DO
   CLOSE (UNIT)
   
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file_2, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'Output file for orientation histogram could not be created!'
   WRITE (UNIT,'("# Orientation distribution for each neighbour")')
   WRITE (UNIT,'("# theta - Neighbour order")')
   DO J = 1, maxbinth
       temp = float(J-1)*dtheta + 0.5*dtheta
       WRITE (UNIT,'(F5.1,5x,20(F8.3,1x))') temp, (neighbour_orientation(I,J)/float(nframes),I=1,maxneighbour)
   END DO
   CLOSE (UNIT)

      
END PROGRAM CLUSTER_ORIENTATIONAL_DISTRIBUTION
