!
! Program to calculate intramolecular angles
!
! 	Reads:
!	1. CONFIG (+FIELD) from DL_POLY
!
!	Writes:
!	1. A file containing the average value of all intramolecular angles.
!
!  Last change: 5/3/2011
!

PROGRAM INTRAMOLECULAR_ANGLES	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
   USE GENERAL_VARIABLES
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE CONFIG_READWRITE,       ONLY : CONFIG_DLPOLY_READ
   USE CONFIG_UTILS,           ONLY : CONFIG_JOIN_MOLECULES
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH) :: config_file, ff_file, output_file

   INTEGER :: I, J, K1, K2, K3, IOSTAT, UNIT

   REAL ( KIND = DP )               :: d12, d13, costheta, theta
   REAL ( KIND = DP ), DIMENSION(3) :: r12, r13

   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:) :: rrr


   ! Get input
   
   WRITE (OUTPUT,'("Coordinates (CONFIG) file name?")')
   READ (INPUT,'(A)') config_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,*)
   

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 
       
 
   
   ! Read coordinates 
      
   CALL CONFIG_DLPOLY_READ (config_file, rrr)
   CALL CONFIG_JOIN_MOLECULES (rrr)
   
   ! Open output file
   
    UNIT = NEXT_UNIT ( )	
    OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )

   ! Compute average intramolecular angle
      
   species_loop: DO I = 1, numspecies
      WRITE (UNIT,*)
      WRITE (UNIT,'("Intramolecular angles for species ",I3)') I	
      atom1_loop:   DO K1 = 1, numatoms(I)        !K1 = central atom 
         atom2_loop:   DO K2 = 1, numatoms(I)-1
	    IF ( K2 == K1 ) CYCLE 
            atom3_loop:   DO K3 = K2+1, numatoms(I) 
	       IF ( K3 == K1 ) CYCLE
               theta = 0.0   
               molecules_loop: DO J = 1, nummols(I)
	          r12(1:3) = rrr(1:3, K2, J, I) - rrr(1:3, K1, J, I)		
	          r13(1:3) = rrr(1:3, K3, J, I) - rrr(1:3, K1, J, I)		
                  d12 = SQRT( DOT_PRODUCT ( r12, r12 ) )
                  d13 = SQRT( DOT_PRODUCT ( r13, r13 ) )
		  costheta = DOT_PRODUCT ( r12, r13 ) / d12 / d13
		  theta = theta + ACOS(costheta) * 180.0 / PI
              END DO molecules_loop
	      WRITE (UNIT,'("Angle ",I3," - ",I3," - ",I3," = ",F7.3)') K2, K1, K3, theta/real(nummols(I))	
	    END DO atom3_loop	
	 END DO atom2_loop	
      END DO atom1_loop	
   END DO species_loop
   
   CLOSE (UNIT)
   
END PROGRAM INTRAMOLECULAR_ANGLES
