!
! Program to calculate intramolecular dihedrals
!
! 	Reads:
!	1. CONFIG (+FIELD) from DL_POLY
!
!	Writes:
!	1. A file containing the average value of all intramolecular dihedrals.
!
!  Last change: 5/10/2013
!

PROGRAM INTRAMOLECULAR_DIHEDRALS	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE GENERAL_VARIABLES
   USE GENERAL_UTILS,          ONLY : DIHEDRAL_ANGLE
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE CONFIG_READWRITE,       ONLY : CONFIG_DLPOLY_READ
   USE CONFIG_UTILS,           ONLY : CONFIG_JOIN_MOLECULES
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH) :: config_file, ff_file, output_file

   INTEGER :: I, J, IOSTAT, UNIT, selected_species, numtors, ITORS, IMOL
   INTEGER, ALLOCATABLE, DIMENSION(:,:) :: dihatom

   REAL ( KIND = DP )               :: dihedral, dihedral_average
   REAL ( KIND = DP ), DIMENSION(3) :: r1, r2, r3, r4

   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:) :: rrr


   ! Get input
   
   WRITE (OUTPUT,'("Coordinates (CONFIG) file name?")')
   READ (INPUT,'(A)') config_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,*)
   

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 
       

   ! Select torsions to compute

   WRITE (OUTPUT,'("Number of the chosen species? ")')
   READ (INPUT,*) selected_species
   IF (selected_species > numspecies) STOP 'Error! Not so many species'
   WRITE (OUTPUT,'("Number of atoms in selected type = ",I4)') numatoms(selected_species)
   WRITE (OUTPUT,'("Number of torsions to compute? ")')
   READ (INPUT,*) numtors
   ALLOCATE ( dihatom ( numtors, 4 ) ); dihatom = 0
   WRITE (OUTPUT,'("For each torsion, give the numbers of the 4 atoms defining the dihedral angle ")')
   DO I = 1,numtors
      WRITE (OUTPUT, "('Torsion ',I3,'?')" ) I
      READ ( INPUT, * ) (dihatom(I,J), J = 1, 4)
    END DO
 
   
   ! Read coordinates 
      
   CALL CONFIG_DLPOLY_READ (config_file, rrr)
   CALL CONFIG_JOIN_MOLECULES (rrr)
   
   ! Open output file
   
    UNIT = NEXT_UNIT ( )	
    OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )

   ! Compute average intramolecular dihedral angle
      
   torsions_loop: DO ITORS = 1,numtors
      dihedral_average = 0.0
      molecules_loop: DO IMOL = 1, nummols(selected_species)
         r1 = rrr(1:3, dihatom(ITORS,1), IMOL, selected_species)
         r2 = rrr(1:3, dihatom(ITORS,2), IMOL, selected_species)
         r3 = rrr(1:3, dihatom(ITORS,3), IMOL, selected_species)
         r4 = rrr(1:3, dihatom(ITORS,4), IMOL, selected_species)
         CALL DIHEDRAL_ANGLE (r1, r2, r3, r4, dihedral)
         if (dihedral < -180.0) dihedral = -180.0
         if (dihedral > 180.0) dihedral = 180.0
	 dihedral_average = dihedral_average + dihedral
      END DO molecules_loop
      WRITE (UNIT,'("Dihedral ",I3," - ",I3," - ",I3," - ",I3," = ",F9.3)') dihatom(ITORS,:), &
             dihedral_average/real(nummols(selected_species))	
   END DO torsions_loop

   
   CLOSE (UNIT)
   
END PROGRAM INTRAMOLECULAR_DIHEDRALS
