!===============================================================================
! This file is part of MG_DLP_analysis - a set of programs to perform
! different analysis on Molecular Dynamics simulations, mainly intended
! to handle trajectories generated by DL_POLY (HISTORY format).
!
! Copyright (C) 2021 Miguel A. Gonzalez
!
!    MG_DLP_analysis is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    MG_DLP_analysis is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with MG_DLP_analysis.  If not, see <https://www.gnu.org/licenses/>.
!
!===============================================================================
!
! Module to read/write an atomic configurations. 
!
! Subroutines:
!
!      CONFIG_DLPOLY_READ (config_file, config_pos, [config_vel, config_forces])
!      CONFIG_DLPOLY_WRITE (config_file, config_pos)
!      CONFIG_PDB_WRITE (config_file, config_pos)
!      CONFIG_PDB_READ  (config_file, config_pos)
!      CONFIG_XYZ_WRITE (config_file, config_pos)
!
! Main variables:
!
!      config_pos (1:3=(X,Y,Z), 
!      numatoms, 
!      nummols, numspecies), 
!      config_vel, 
!      config_forces      
!
!  Last change: 5/2/2010
!
!===============================================================================

MODULE CONFIG_READWRITE

USE DEFINITIONS, ONLY : DP
USE IO_UNITS, ONLY : INPUT, OUTPUT
USE FILES, ONLY : NEXT_UNIT
USE PARSING, ONLY : GET_LINE, GET_WORD, GET_INTEGER, GET_REAL, WORD, WRDLEN, EMPTY, PUSH_UNIT
USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
USE GENERAL_VARIABLES 
USE FORCEFIELD_VARIABLES, ONLY : molname, atomname
USE CONFIG_VARIABLES

IMPLICIT NONE
PUBLIC

CONTAINS


   SUBROUTINE CONFIG_DLPOLY_READ (config_file, config_pos, config_vel, config_forces)

   CHARACTER ( LEN = * ), INTENT(IN) :: config_file
   REAL ( KIND = DP ), INTENT(OUT), ALLOCATABLE, DIMENSION(:,:,:,:) :: config_pos
   REAL ( KIND = DP ), INTENT(OUT), OPTIONAL, ALLOCATABLE, DIMENSION(:,:,:,:) :: config_vel, config_forces

   LOGICAL :: end
   CHARACTER ( LEN = 8) :: tmp_name
   INTEGER :: I, J, K, IOSTAT, UNIT
   REAL ( KIND = DP ) :: tmp

   
   ! Check if minimum needed information is available
   IF (numspecies <= 0) STOP 'CONFIG_DLPOLY_READ: Number of different species not set!'
   IF (maxval(nummols) <= 0) STOP 'CONFIG_DLPOLY_READ: Number of molecules of each type not set!'
   IF (maxval(numatoms) <= 0) STOP 'CONFIG_DLPOLY_READ: Number of atoms in each molecule not set!'
         
   ! Open CONFIG file
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = config_file, ACTION = "READ", STATUS = "OLD", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'CONFIG_DLPOLY_READ: CONFIG file not found  or could not be opened correctly!'

   ! Set the parsing unit.
   CALL PUSH_UNIT ( UNIT )

   ! Read in the title line of the file.
   READ ( UNIT, "(A)", IOSTAT = IOSTAT ) config_title
   
   ! Read in keys for atoms (coords/vel/forces) and cell (shape)
   CALL GET_LINE
   CALL GET_INTEGER ( dlpoly_atomkey )
   CALL GET_INTEGER ( dlpoly_cellkey )

   IF (dlpoly_cellkey > 0) THEN
   
      ! Read in cell vectors
      ! Vector a = cell(1,i)
      CALL GET_LINE
      CALL GET_REAL ( cell(1,1) )
      CALL GET_REAL ( cell(1,2) )
      CALL GET_REAL ( cell(1,3) )
      ! Vector b = cell(2,i)
      CALL GET_LINE
      CALL GET_REAL ( cell(2,1) )
      CALL GET_REAL ( cell(2,2) )
      CALL GET_REAL ( cell(2,3) )
      ! Vector c = cell(3,i)
      CALL GET_LINE
      CALL GET_REAL ( cell(3,1) )
      CALL GET_REAL ( cell(3,2) )
      CALL GET_REAL ( cell(3,3) )

   END IF
   
   ! Allocate arrays needed to store information read from CONFIG
   IF ( ALLOCATED(atomindex) )     DEALLOCATE(atomindex)
   IF ( ALLOCATED(atomicnumber) )  DEALLOCATE(atomicnumber)
   IF ( ALLOCATED(config_pos) )    DEALLOCATE(config_pos)
   ALLOCATE ( atomindex ( maxval(numatoms), maxval(nummols), numspecies ) ) 
   ALLOCATE ( atomicnumber ( maxval(numatoms), maxval(nummols), numspecies ) )  
   ALLOCATE ( config_pos ( 3, maxval(numatoms), maxval(nummols), numspecies ) ) 
   atomindex = 0; atomicnumber = 0; config_pos = 0.0
   IF ( PRESENT ( config_vel ) ) THEN
      IF (dlpoly_atomkey >= 1) THEN
         IF ( ALLOCATED(config_vel) )   DEALLOCATE(config_vel)
         ALLOCATE ( config_vel ( 3, maxval(numatoms), maxval(nummols), numspecies ) )
         config_vel = 0.0
      ELSE
         PRINT *,'CONFIG_DLPOLY_READ: WARNING !!! Array for velocities given in call, but no velocities present in file.'
      END IF
   END IF          
   IF ( PRESENT ( config_forces ) ) THEN
      IF (dlpoly_atomkey >= 2) THEN
         IF ( ALLOCATED(config_forces) )   DEALLOCATE(config_forces)
         ALLOCATE ( config_forces ( 3, maxval(numatoms), maxval(nummols), numspecies ) )
         config_forces = 0.0
      ELSE
         PRINT *,'CONFIG_DLPOLY_READ: WARNING !!! Array for forces given in call, but no velocities present in file.'
      END IF       
   END IF
   

   ! Read file
   
   species_loop: DO I = 1,numspecies
      molecules_loop: DO J = 1,nummols(I)
         atoms_loop:   DO K = 1,numatoms(I) 

            ! Get atom name 
            CALL GET_LINE
            CALL GET_WORD
            tmp_name = WORD(1:WRDLEN)
          
            ! Check if array of names exists and both agree
            IF (ALLOCATED(atomname)) THEN
               IF (tmp_name /= atomname(K,I)) THEN
                  PRINT *, 'CONFIG_DLPOLY_READ: WARNING !!!' 
                  PRINT *, '  Different atom name in FIELD and CONFIG files&
                           &  for type ',I,', molecule ',J,', atom ',K,'.'
                  PRINT *, '  Names: ',tmp_name,atomname(I,K)                
               ENDIF        
            ENDIF

            ! If present, get atom index
            end = EMPTY()
            IF (.NOT.end) THEN
               CALL GET_INTEGER ( atomindex(K,J,I) ) 
            ELSE
               atomindex(K,J,I) = 0
            ENDIF 
            
            ! If present, get atomic number
            end = EMPTY()
            IF (.NOT.end) THEN
               CALL GET_INTEGER ( atomicnumber(K,J,I) ) 
            ELSE
               atomicnumber(K,J,I) = 0.0
            ENDIF   
          
            ! Get positions
            CALL GET_LINE
            CALL GET_REAL    ( config_pos(1,K,J,I) )
            CALL GET_REAL    ( config_pos(2,K,J,I) )
            CALL GET_REAL    ( config_pos(3,K,J,I) )
          
            IF (dlpoly_atomkey >= 1) THEN     ! Get velocities
               CALL GET_LINE
               CALL GET_REAL    ( tmp )
               IF ( PRESENT (config_vel) ) config_vel(1,K,J,I) = tmp
               CALL GET_REAL    ( tmp )
               IF ( PRESENT (config_vel) ) config_vel(2,K,J,I) = tmp
               CALL GET_REAL    ( tmp )
               IF ( PRESENT (config_vel) ) config_vel(3,K,J,I) = tmp
            END IF
          
            IF (dlpoly_atomkey >= 2) THEN     ! Get forces
               CALL GET_LINE
               CALL GET_REAL    ( tmp )
               IF ( PRESENT (config_forces) ) config_forces(1,K,J,I) = tmp
               CALL GET_REAL    ( tmp )
               IF ( PRESENT (config_forces) ) config_forces(2,K,J,I) = tmp
               CALL GET_REAL    ( tmp )
               IF ( PRESENT (config_forces) ) config_forces(3,K,J,I) = tmp
            END IF
   
         END DO atoms_loop
      END DO molecules_loop
   END DO species_loop
   
   CLOSE ( UNIT )
   
   END SUBROUTINE CONFIG_DLPOLY_READ


   SUBROUTINE CONFIG_DLPOLY_WRITE (config_file, config_pos)

   CHARACTER ( LEN = * ), INTENT(IN) :: config_file
   REAL ( KIND = DP ), INTENT(IN), ALLOCATABLE, DIMENSION(:,:,:,:) :: config_pos

   INTEGER :: index, total_atoms, I, J, K, IOSTAT, UNIT
   CHARACTER ( LEN = 8) :: name

   
   ! Check if minimum needed information is available
   IF (numspecies <= 0) STOP 'CONFIG_DLPOLY_WRITE: Number of different species not set!'
   IF (maxval(nummols) <= 0) STOP 'CONFIG_DLPOLY_WRITE: Number of molecules of each type not set!'
   IF (maxval(numatoms) <= 0) STOP 'CONFIG_DLPOLY_WRITE: Number of atoms in each molecule not set!'
         
   ! Open CONFIG file
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = config_file, ACTION = "WRITE", STATUS = "NEW", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'CONFIG_DLPOLY_WRITE: CONFIG file could not be opened correctly!'


   ! Write file
   
   !WRITE (OUTPUT, *)
   !WRITE (OUTPUT, '("Title for CONFIG file?")')
   !READ (INPUT, '(A80)') config_title
   !WRITE ( UNIT, '(A80)' ) config_title
   WRITE ( UNIT, '(A80)' ) 'CONFIG file written by config_dlpoly_write'
   dlpoly_atomkey = 0  !only coordinates written
   dlpoly_cellkey = 3  !parallelepiped boundary conditions
   total_atoms = SUM (nummols*numatoms)
   WRITE ( UNIT, '(3I10)' ) dlpoly_atomkey, dlpoly_cellkey, total_atoms
   
   WRITE ( UNIT, '(3F20.10)' ) cell(1,:)
   WRITE ( UNIT, '(3F20.10)' ) cell(2,:)
   WRITE ( UNIT, '(3F20.10)' ) cell(3,:)
   
   index = 0
   species_loop: DO I = 1,numspecies
      molecules_loop: DO J = 1,nummols(I)
         atoms_loop:   DO K = 1,numatoms(I) 

            index = index + 1

            IF (ALLOCATED(atomname)) THEN
               name = atomname(K,I)(1:3)
            ELSE
               name = 'XXX'
            END IF
          
            ! Write atom name 
            WRITE ( UNIT, '(A8,I10)' ) name, index

            ! Write positions 
            WRITE ( UNIT, '(3F20.10)' ) config_pos ( :, K, J, I )
   
         END DO atoms_loop
      END DO molecules_loop
   END DO species_loop

   CLOSE ( UNIT )
   
   END SUBROUTINE CONFIG_DLPOLY_WRITE
   

   SUBROUTINE CONFIG_PDB_WRITE (config_file, config_pos)
 
   CHARACTER ( LEN = * ), INTENT(IN) :: config_file
   REAL ( KIND = DP ), INTENT(IN), ALLOCATABLE, DIMENSION(:,:,:,:) :: config_pos

   INTEGER :: I, J, K, UNIT, IOSTAT, index
   CHARACTER ( LEN = 3 ) :: name, molecule
   
   
   ! Check if minimum needed information is available
   IF (numspecies <= 0) STOP 'HISTORY_DLPOLY_WRITE: Number of different species not set!'
   IF (maxval(nummols) <= 0) STOP 'HISTORY_DLPOLY_WRITE: Number of molecules of each type not set!'
   IF (maxval(numatoms) <= 0) STOP 'HISTORY_DLPOLY_WRITE: Number of atoms in each molecule not set!'

   ! Check if config_pos compatible with available system information
   IF (SIZE(config_pos,4) /= numspecies) STOP 'CONFIG_PDB_WRITE: Number of species /= Size of config_pos'
      
   ! Open output file
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = config_file, ACTION = "WRITE", STATUS = "NEW", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'CONFIG_PDB_WRITE: Problem opening file!'

   IF (.NOT.ALLOCATED(atomname)) PRINT *,'CONFIG_PDB_WRITE: WARNING !!!Atom names not available. Writing XXX instead.'

   index = 0
   species_loop: DO I = 1,numspecies
      molecules_loop: DO J = 1,nummols(I)
         atoms_loop:   DO K = 1,numatoms(I) 

            index = index + 1
          
            IF (ALLOCATED(atomname)) THEN
               name = atomname(K,I)(1:3)
            ELSE
               name = 'XXX'
            END IF
          
            IF (ALLOCATED(molname)) THEN
               molecule = molname(I)(1:3)
            ELSE
               molecule = 'XXX'
            END IF
          
            WRITE (UNIT,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') &
             &'ATOM  ', index, name, molecule, J, config_pos(1,K,J,I), config_pos(2,K,J,I), &
             &config_pos(3,K,J,I), 0.0, 0.0, molecule 

         END DO atoms_loop
      END DO molecules_loop
   END DO species_loop

   CLOSE (UNIT)
  
   END SUBROUTINE CONFIG_PDB_WRITE
   

   SUBROUTINE CONFIG_PDB_READ (config_file, config_pos)
 
   CHARACTER ( LEN = * ), INTENT(IN) :: config_file
   REAL ( KIND = DP ), INTENT(OUT), ALLOCATABLE, DIMENSION(:,:,:,:) :: config_pos

   INTEGER, PARAMETER :: max_atoms = 99999
   
   LOGICAL                                            :: end = .FALSE., new_name
   CHARACTER ( LEN = 4 ), DIMENSION(max_atoms)        :: name, diffname
   CHARACTER ( LEN = 4 ), ALLOCATABLE, DIMENSION(:,:) :: tmp_name
   CHARACTER ( LEN = 1 )                              :: answer
   INTEGER                                            :: ndummy, atomindex, different_names, total_atoms, index
   INTEGER                                            :: I, J, K, UNIT, IOSTAT
   INTEGER, ALLOCATABLE, DIMENSION(:,:)               :: order
   REAL ( KIND = DP )                                 :: a, b, c, alpha, beta, gamma, calpha, cbeta, cgamma, sgamma, vcell
   REAL ( KIND = DP ), DIMENSION(3,max_atoms)         :: atompos
   
         
   ! Open input file
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = config_file, ACTION = "READ", STATUS = "OLD", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'CONFIG_PDB_READ: Problem opening file!'
   
   ! Set the parsing unit.
   CALL PUSH_UNIT ( UNIT )

   ! Read file line by line
   atomindex = 0
   different_names = 0
   DO WHILE (.NOT.end) 
      CALL GET_LINE (end)
      IF (.NOT.end) THEN
         CALL GET_WORD
         IF ( WORD(1:4) == 'CRYS' ) THEN
            CALL GET_REAL ( a )
            CALL GET_REAL ( b )
            CALL GET_REAL ( c )
            CALL GET_REAL ( alpha )
            CALL GET_REAL ( beta )
            CALL GET_REAL ( gamma )
            calpha = cos(alpha*PI/180.)
            cbeta = cos(beta*PI/180.)
            cgamma = cos(gamma*PI/180.)
            sgamma = sin(gamma*PI/180.)
            vcell = a * b * c * sqrt( 1.0 - calpha**2 - cbeta**2 - cgamma**2  + (2.0 * calpha * cbeta * cgamma ) )
            cell(1,1) = a
            cell(1,2) = 0.0
            cell(1,3) = 0.0
            cell(2,1) = b * cgamma
            cell(2,2) = b * sgamma
            cell(2,3) = 0.0
            cell(3,1) = c * cbeta
            cell(3,2) = c * ( calpha - cbeta*cgamma ) / sgamma
            cell(3,3) = vcell / ( a * b * sgamma )
         END IF
         IF ( WORD(1:4) == 'ATOM' ) THEN
            atomindex = atomindex + 1
            IF ( atomindex > max_atoms ) STOP 'Increase max_atoms' 
            CALL GET_INTEGER ( ndummy )
            CALL GET_WORD 
            name (atomindex) = TRIM(WORD)
            new_name = .TRUE.
            DO I = 1, different_names
               IF ( name (atomindex) == diffname(I) ) new_name = .FALSE.
            END DO
            IF ( new_name ) THEN
               different_names = different_names + 1
               diffname(different_names) = name(atomindex)
            END IF
            CALL GET_WORD
            CALL GET_INTEGER ( ndummy )
            CALL GET_REAL ( atompos ( 1, atomindex ) )
            CALL GET_REAL ( atompos ( 2, atomindex ) )
            CALL GET_REAL ( atompos ( 3, atomindex ) )
         END IF
      END IF
   END DO

   CLOSE (UNIT)
   
   WRITE ( OUTPUT, *)
   WRITE ( OUTPUT, "('Number of atoms found = ',I6)" ) atomindex
   WRITE ( OUTPUT, "('Number of different atom names = ',I3)" ) different_names
   WRITE ( OUTPUT, "('Names:',2X,20(A4,1X))" ) ( diffname(I), I = 1, different_names )
   WRITE ( OUTPUT, *)

   WRITE (OUTPUT,'("Provide system information or give number of species = 0 to write unmodified file as a single molecule.")')
   WRITE (OUTPUT,'("Number of different species?")')
   READ (INPUT,*) numspecies
   
   IF ( numspecies == 0 ) THEN
   
      numspecies = 1
      ALLOCATE ( nummols(1), numatoms(1) )
      nummols(1) = 1
      numatoms(1) = atomindex
      ALLOCATE ( atomname ( maxval(numatoms), numspecies ) ); atomname = ''
      ALLOCATE ( config_pos ( 3, maxval(numatoms), maxval(nummols), numspecies ) ); config_pos = 0.0
      index = 0
      DO I = 1, numspecies
         DO J = 1, nummols(I)
            DO K = 1, numatoms(I) 
               index = index+1
               config_pos ( :, K, J, I ) = atompos ( :, index )
               IF (J == 1) atomname(K,I) = name(index)
            END DO
         END DO
      END DO 
   
   ELSE
   
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
         WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
         READ (INPUT,*) nummols(I) 
         WRITE (OUTPUT,'("Number of atoms in this molecule?")')
         READ (INPUT,*) numatoms(I) 
      END DO

      total_atoms = SUM (nummols*numatoms)
      IF (total_atoms /= atomindex) STOP 'Number of atoms read /= nummols*numatoms'

      !Print & verify names
      ALLOCATE ( tmp_name ( maxval(numatoms), numspecies ) ); tmp_name = '' 
      index = 0
      DO I = 1, numspecies
         DO J = 1, nummols(I)
            DO K = 1, numatoms(I) 
               index = index+1
               IF (J == 1) THEN
                   tmp_name(K,I) = name(index)
               ELSE
                  IF (tmp_name(K,I) /= name(index)) THEN
                     WRITE (OUTPUT, '("Problem: Different names found!!!")')
                     WRITE (OUTPUT, '("Species ",I1,", Molecule ",I4,", Atom ",I3," Name:",A4)') I, J, K, tmp_name(K,I)
                     WRITE (OUTPUT, '("Atom index ",I6," Name:",A4)') index, name(index)
                     STOP
                  END IF   
               END IF
            END DO
         END DO
      END DO 

      WRITE (OUTPUT, *) 
      DO I = 1, numspecies
         WRITE (OUTPUT, '("Species ",I2,".")') I
         WRITE (OUTPUT, '("Atom names: ",20(A4,1X))') ( tmp_name(K,I), K = 1, numatoms(I) )
      END DO 
      WRITE (OUTPUT, '("Do you want to change the names?")') 
      READ (INPUT, '(A)') answer
      IF ( (answer == "Y") .OR. (answer == "y") ) THEN
         DO I = 1, numspecies
            WRITE (OUTPUT, '("Species ",I2,":")') I
            DO K =  1, numatoms(I)
               WRITE (OUTPUT, '("Atom ",I3,", old name: ",A4,".")', ADVANCE='NO') K, tmp_name(K,I)
               WRITE (OUTPUT, '(" New name? ")', ADVANCE='NO') 
               READ (INPUT, '(A4)') tmp_name(K,I)
            END DO
         END DO 
      END IF

      WRITE (OUTPUT, *) 
      WRITE (OUTPUT, '("For each atom give its ordering number in the molecule.")') 
      ALLOCATE ( order ( maxval(numatoms), numspecies ) ); order = 0 
      DO I = 1, numspecies
         WRITE (OUTPUT, '("Species ",I2,":")') I
         WRITE (OUTPUT, '("Atom names: ",20(A4,1X))') ( tmp_name(K,I), K = 1, numatoms(I) )
         READ (INPUT, *) ( order(K,I), K = 1, numatoms(I) )
         IF (maxval(order(:,I)) > numatoms(I)) STOP 'Order number > number of atoms'
      END DO 

      !Allocate final arrays for atom names and atom positions
      ALLOCATE ( atomname ( maxval(numatoms), numspecies ) ); atomname = ''
      ALLOCATE ( config_pos ( 3, maxval(numatoms), maxval(nummols), numspecies ) ); config_pos = 0.0

      index = 0
      DO I = 1, numspecies
         DO J = 1, nummols(I)
            DO K = 1, numatoms(I) 
               index = index+1
               config_pos ( :, order(K,I), J, I ) = atompos ( :, index )
               IF (J == 1) atomname(order(K,I),I) = tmp_name(K,I)
            END DO
         END DO
      END DO 
   
   END IF
   
   END SUBROUTINE CONFIG_PDB_READ
   

   SUBROUTINE CONFIG_XYZ_WRITE (config_file, config_pos)
 
   CHARACTER ( LEN = * ), INTENT(IN) :: config_file
   REAL ( KIND = DP ), INTENT(IN), ALLOCATABLE, DIMENSION(:,:,:,:) :: config_pos

   INTEGER :: I, J, K, UNIT, IOSTAT, index
   
   
   ! Check if minimum needed information is available
   IF (numspecies <= 0) STOP 'CONFIG_XYZ_WRITE: Number of different species not set!'
   IF (maxval(nummols) <= 0) STOP 'CONFIG_XYZ_WRITE: Number of molecules of each type not set!'
   IF (maxval(numatoms) <= 0) STOP 'CONFIG_XYZ_WRITE: Number of atoms in each molecule not set!'

   ! Check if config_pos compatible with available system information
   IF (SIZE(config_pos,4) /= numspecies) STOP 'CONFIG_XYZ_WRITE: Number of species /= Size of config_pos'
      
   ! Open output file
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = config_file, ACTION = "WRITE", STATUS = "NEW", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'CONFIG_XYZ_WRITE: Problem opening file!'

   index = 0
   species_loop: DO I = 1,numspecies
      molecules_loop: DO J = 1,nummols(I)
         atoms_loop:   DO K = 1,numatoms(I) 

            index = index + 1
          
            WRITE (UNIT,'(I5,5x,F12.6,3x,F12.6,3x,F12.6)') &
             & index, config_pos(1,K,J,I), config_pos(2,K,J,I), config_pos(3,K,J,I)

         END DO atoms_loop
      END DO molecules_loop
   END DO species_loop

   CLOSE (UNIT)
  
   END SUBROUTINE CONFIG_XYZ_WRITE


END MODULE CONFIG_READWRITE  

   
