!
! Reads a DL_POLY CONFIG file from DL_POLY and writes a reduced
! cell containing a smaller number of molecules.
!
! 	Reads:
!	1. A CONFIG file
!
! 	Writes:
!	1. A CONFIG file
!
!
!  Last change: 12/06/2012
!

PROGRAM CONFIG_REDUCE_CELL	

   USE DEFINITIONS,       ONLY : LINE_LENGTH, DP
   USE IO_UNITS,          ONLY : INPUT, OUTPUT
   USE GENERAL_VARIABLES
   USE FORCEFIELD_READ,   ONLY : FIELD_DLPOLY_READ
   USE CONFIG_READWRITE,  ONLY : CONFIG_DLPOLY_READ, CONFIG_DLPOLY_WRITE
   USE CONFIG_UTILS,      ONLY : CONFIG_JOIN_MOLECULES
   USE CONFIG_VARIABLES,  ONLY : cell
    
   IMPLICIT NONE

   INTEGER :: I, J, K, n_final, index
   INTEGER, DIMENSION(4) :: minident, maxident 
   REAL ( KIND = DP ) :: xmin, ymin, zmin, xmax, ymax, zmax, thickness 
   REAL ( KIND = DP ) :: x1, x2, x3, x4, y1, y2, y3, y4, z1, z2, z3, z4 
   CHARACTER ( LEN = LINE_LENGTH) :: file1, file2, ff_file
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:) :: config, config_new
   INTEGER, ALLOCATABLE, DIMENSION(:) :: mol_index

   ! Get input
   
   WRITE (OUTPUT,'("Name of input file?")')
   READ (INPUT,'(A)') file1
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') file2
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 
   
   IF (numspecies > 1) STOP 'Only adapted to work with a system containing 1 molecular type'
   WRITE (OUTPUT,'("Initial number of molecules = ", I6)') nummols(1)
   WRITE (OUTPUT,'("Final number of molecules?")') 
   READ (INPUT,*) n_final
   IF (n_final > nummols(1)) STOP 'N_final cannot be larger than N_ini'
   ALLOCATE ( mol_index (n_final) )
   WRITE (OUTPUT,'("Thickness of extra layer (in AA)?")') 
   READ (INPUT,*) thickness

   CALL CONFIG_DLPOLY_READ ( file1, config )

   CALL CONFIG_JOIN_MOLECULES ( config )
   
   ALLOCATE ( config_new ( 3, numatoms(1), n_final, 1 ) )

   
   ! Modify cell parameters
   
   cell(1,:) = cell(1,:) * (float(n_final)/float(nummols(1)))**(1./3.)
   cell(2,:) = cell(2,:) * (float(n_final)/float(nummols(1)))**(1./3.)
   cell(3,:) = cell(3,:) * (float(n_final)/float(nummols(1)))**(1./3.)
      
   print *, 'x = ', minval(config(1,1,:,1)), maxval(config(1,1,:,1))
   print *, 'y = ', minval(config(2,1,:,1)), maxval(config(2,1,:,1))
   print *, 'z = ', minval(config(3,1,:,1)), maxval(config(3,1,:,1))
   xmin = minval(config(1, 1, :, 1)) * (float(n_final)/float(nummols(1)))**(1./3.)
   ymin = minval(config(2, 1, :, 1)) * (float(n_final)/float(nummols(1)))**(1./3.)
   zmin = minval(config(3, 1, :, 1)) * (float(n_final)/float(nummols(1)))**(1./3.)
   xmax = maxval(config(1, 1, :, 1)) * (float(n_final)/float(nummols(1)))**(1./3.)
   ymax = maxval(config(2, 1, :, 1)) * (float(n_final)/float(nummols(1)))**(1./3.)
   zmax = maxval(config(3, 1, :, 1)) * (float(n_final)/float(nummols(1)))**(1./3.)
   print *, 'new x range = ', xmin, xmax
   print *, 'new yrange = ', ymin, ymax
   print *, 'new z range = ', zmin, zmax

   ! First selection of molecules
   index = 0
   DO I = 1, nummols(1)
      IF (config(1,1,I,1) >= xmin .AND. config(1,1,I,1) <= xmax .AND. &
          config(2,1,I,1) >= ymin .AND. config(2,1,I,1) <= ymax .AND. &
	  config(3,1,I,1) >= zmin .AND. config(3,1,I,1) <= zmax) THEN
	  index = index + 1
	  mol_index(index) = I
	  IF (index == n_final) EXIT
      END IF
   END DO
   PRINT *, '1st pass, index = ', index

   ! Second pass, adding thick AA on each side
   IF (index < n_final) THEN
     x1 = xmin-thickness
     x2 = xmin
     x3 = xmax
     x4 = xmax+thickness
     y1 = ymin-thickness
     y2 = ymin
     y3 = ymax
     y4 = ymax+thickness
     z1 = zmin-thickness
     z2 = zmin
     z3 = zmax
     z4 = zmax+thickness
     DO I = 1, nummols(1)
       IF ( ( ((config(1,1,I,1) >= x1   .AND. config(1,1,I,1) <= x2)    .OR.  &
               (config(1,1,I,1) >= x3   .AND. config(1,1,I,1) <= x4))   .AND. &
                config(2,1,I,1) >= ymin .AND. config(2,1,I,1) <= ymax   .AND. &
	        config(3,1,I,1) >= zmin .AND. config(3,1,I,1) <= zmax)  .OR.  &
            ( ((config(2,1,I,1) >= y1   .AND. config(2,1,I,1) <= y2)    .OR.  &
               (config(2,1,I,1) >= y3   .AND. config(2,1,I,1) <= y4))   .AND. &
                config(1,1,I,1) >= xmin .AND. config(1,1,I,1) <= xmax   .AND. &
	        config(3,1,I,1) >= zmin .AND. config(3,1,I,1) <= zmax)  .OR.  &
            ( ((config(3,1,I,1) >= z1   .AND. config(2,1,I,1) <= z2)    .OR.  &
               (config(3,1,I,1) >= z3   .AND. config(2,1,I,1) <= z4))   .AND. &
                config(1,1,I,1) >= xmin .AND. config(1,1,I,1) <= xmax   .AND. &
	        config(2,1,I,1) >= ymin .AND. config(3,1,I,1) <= ymax) ) THEN
	  index = index + 1
	  mol_index(index) = I
 	  IF (index == n_final) EXIT
        END IF
      END DO
    ENDIF
    print *, '2nd pass, index = ', index
   
   IF (index < n_final) STOP 'Did not work! Try increasing thickness of extra layer'
   
   !set config_new
   nummols(1) = n_final
   species_loop: DO I = 1,numspecies
      molecules_loop: DO J = 1,nummols(I)
         atoms_loop:   DO K = 1,numatoms(I) 
	    config_new(1:3,K,J,I) = config(1:3,K,mol_index(J),I) 
         END DO atoms_loop
      END DO molecules_loop
   END DO species_loop
   
   print *, 'check x = ', minval(config_new(1,1,:,1)), maxval(config_new(1,1,:,1))
   print *, 'check y = ', minval(config_new(2,1,:,1)), maxval(config_new(2,1,:,1))
   print *, 'check z = ', minval(config_new(3,1,:,1)), maxval(config_new(3,1,:,1))
   
   CALL CONFIG_DLPOLY_WRITE ( file2, config_new )
      

END PROGRAM CONFIG_REDUCE_CELL
