!===============================================================================
! This file is part of MG_DLP_analysis - a set of programs to perform
! different analysis on Molecular Dynamics simulations, mainly intended
! to handle trajectories generated by DL_POLY (HISTORY format).
!
! Copyright (C) 2021 Miguel A. Gonzalez
!
!    MG_DLP_analysis is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    MG_DLP_analysis is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with MG_DLP_analysis.  If not, see <https://www.gnu.org/licenses/>.
!
!===============================================================================
!
! Module containing different utils to manipulate a set of coordinates. 
!
! Subroutines:
!
!       CONFIG_TRANSLATE (config_pos, vector)
!       CONFIG_ROTATE (config_pos, rotation)
!       CONFIG_CART_2_FRACT (config_pos, config_fract)
!       CONFIG_FRACT_2_CART (config_fract, config_pos)
!       CONFIG_JOIN_MOLECULES (config_pos)
!       CONFIG_CENTEROFMASS (config_pos, config_com, com_species, com_nummols)
!
! Main variables:
!
!       config_pos (1:3=(X,Y,Z), numatoms, nummols, numspecies)   : configuration in cartesian coordinates     
!       config_fract (1:3=(X,Y,Z), numatoms, nummols, numspecies) : configuration in fractional coordinates     
!
!  Last change: 06/03/2011
!
!===============================================================================

MODULE CONFIG_UTILS

USE DEFINITIONS,      ONLY : DP
USE GENERAL_VARIABLES
USE CONFIG_VARIABLES, ONLY : cell
USE FORCEFIELD_VARIABLES, ONLY : atommass
USE GENERAL_UTILS,    ONLY : INVERT_CELL

IMPLICIT NONE
PUBLIC

CONTAINS

   SUBROUTINE CONFIG_TRANSLATE (config_pos, vector)
   
   REAL ( KIND = DP ), INTENT(INOUT), ALLOCATABLE, DIMENSION(:,:,:,:) :: config_pos
   REAL ( KIND = DP ), INTENT(IN), DIMENSION(3) :: vector
   
   INTEGER :: I, J, K
       
   species_loop: DO I = 1,numspecies
      molecules_loop: DO J = 1,nummols(I)
         atoms_loop:   DO K = 1,numatoms(I) 
            
            config_pos(1:3,K,J,I) = config_pos(1:3,K,J,I) + vector(1:3)
          
         END DO atoms_loop
      END DO molecules_loop
   END DO species_loop
     
   END SUBROUTINE CONFIG_TRANSLATE


   SUBROUTINE CONFIG_ROTATE (config_pos, rotation)
   
   REAL ( KIND = DP ), INTENT(INOUT), ALLOCATABLE, DIMENSION(:,:,:,:) :: config_pos
   REAL ( KIND = DP ), INTENT(IN), DIMENSION(3,3) :: rotation
   
   INTEGER            :: I, J, K
   REAL ( KIND = DP ) :: X, Y, Z   
   
   species_loop: DO I = 1,numspecies
      molecules_loop: DO J = 1,nummols(I)
         atoms_loop:   DO K = 1,numatoms(I) 
            
            X = config_pos(1,K,J,I) 
            Y = config_pos(2,K,J,I) 
            Z = config_pos(3,K,J,I) 
            config_pos(1,K,J,I) = ROTATION(1,1)*X + ROTATION(1,2)*Y + ROTATION(1,3)*Z
            config_pos(2,K,J,I) = ROTATION(2,1)*X + ROTATION(2,2)*Y + ROTATION(2,3)*Z
            config_pos(3,K,J,I) = ROTATION(3,1)*X + ROTATION(3,2)*Y + ROTATION(3,3)*Z
          
         END DO atoms_loop
      END DO molecules_loop
   END DO species_loop
     
   END SUBROUTINE CONFIG_ROTATE


   SUBROUTINE CONFIG_CART_2_FRACT (config_pos, config_fract)

   REAL ( KIND = DP ), INTENT(IN), ALLOCATABLE, DIMENSION(:,:,:,:)  :: config_pos
   REAL ( KIND = DP ), INTENT(OUT), ALLOCATABLE, DIMENSION(:,:,:,:) :: config_fract

   INTEGER                            :: I, J, K
   REAL ( KIND = DP )                 :: volume_cell
   REAL ( KIND = DP ), DIMENSION(3,3) :: inverse_cell

   IF (ALLOCATED(config_fract)) DEALLOCATE(config_fract)
   ALLOCATE ( config_fract ( 3, maxval(numatoms), maxval(nummols), numspecies) )
   config_fract = 0.0

   CALL INVERT_CELL (cell, inverse_cell, volume_cell)

   ! Scale atomic positions
   
   species_loop: DO I = 1,numspecies
      molecules_loop: DO J = 1,nummols(I)
         atoms_loop:   DO K = 1,numatoms(I) 
            
            config_fract(1,K,J,I) = config_pos(1,K,J,I)*inverse_cell(1,1) + &
                                    config_pos(2,K,J,I)*inverse_cell(2,1) + &
                                    config_pos(3,K,J,I)*inverse_cell(3,1)
            
            config_fract(2,K,J,I) = config_pos(1,K,J,I)*inverse_cell(1,2) + &
                                    config_pos(2,K,J,I)*inverse_cell(2,2) + &
                                    config_pos(3,K,J,I)*inverse_cell(3,2)

            config_fract(3,K,J,I) = config_pos(1,K,J,I)*inverse_cell(1,3) + &
                                    config_pos(2,K,J,I)*inverse_cell(2,3) + &
                                    config_pos(3,K,J,I)*inverse_cell(3,3)
          
         END DO atoms_loop
      END DO molecules_loop
   END DO species_loop
   
   END SUBROUTINE CONFIG_CART_2_FRACT


   SUBROUTINE CONFIG_FRACT_2_CART (config_fract, config_pos)

   REAL ( KIND = DP ), INTENT(IN), ALLOCATABLE, DIMENSION(:,:,:,:) :: config_fract
   REAL ( KIND = DP ), INTENT(OUT), ALLOCATABLE, DIMENSION(:,:,:,:) :: config_pos

   INTEGER                            :: I, J, K

   IF (ALLOCATED(config_pos)) DEALLOCATE(config_pos)
   ALLOCATE ( config_pos ( 3, maxval(numatoms), maxval(nummols), numspecies) )
   config_pos = 0.0

   ! Scale atomic positions
   
   species_loop: DO I = 1,numspecies
      molecules_loop: DO J = 1,nummols(I)
         atoms_loop:   DO K = 1,numatoms(I) 
            
            config_pos(1,K,J,I) = config_fract(1,K,J,I)*cell(1,1) + &
                                  config_fract(2,K,J,I)*cell(2,1) + &
                                  config_fract(3,K,J,I)*cell(3,1)
            
             config_pos(2,K,J,I) = config_fract(1,K,J,I)*cell(1,2) + &
                                   config_fract(2,K,J,I)*cell(2,2) + &
                                   config_fract(3,K,J,I)*cell(3,2)

             config_pos(3,K,J,I) = config_fract(1,K,J,I)*cell(1,3) + &
                                   config_fract(2,K,J,I)*cell(2,3) + &
                                   config_fract(3,K,J,I)*cell(3,3)
          
         END DO atoms_loop
      END DO molecules_loop
   END DO species_loop
   
   END SUBROUTINE CONFIG_FRACT_2_CART


   SUBROUTINE CONFIG_JOIN_MOLECULES (config_pos)

   REAL ( KIND = DP ), INTENT(INOUT), ALLOCATABLE, DIMENSION(:,:,:,:) :: config_pos  

   INTEGER :: I, J, K 
   REAL ( KIND = DP ), DIMENSION(3) :: sd
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:) :: tmp_s !Array for coordinates in fractional units


   ! Check if minimum needed information is available
   IF (numspecies <= 0) STOP 'CONFIG_JOIN_MOLECULES: Number of different species not set!'
   IF (maxval(nummols) <= 0) STOP 'CONFIG_JOIN_MOLECULES: Number of molecules of each type not set!'
   IF (maxval(numatoms) <= 0) STOP 'CONFIG_JOIN_MOLECULES: Number of atoms in each molecule not set!'

   ! Check if trajectory is compatible with available system information
   IF (SIZE(config_pos,4) /= numspecies) STOP 'CONFIG_JOIN_MOLECULES: Number of species /= Size of config_pos'

   ! Transform to fractional units
   CALL CONFIG_CART_2_FRACT (config_pos, tmp_s)

   
   ! Join molecule using previous atom as reference
   
   PRINT *,'CONFIG_JOIN_MOLECULES: Rejoining molecules broken due to PBC!'
   PRINT *,'Note that the output maybe wrong if the molecule is very large (e.g. a polymer) and'
   PRINT *,'consecutive atoms are not bonded, so they can really be separated by more than L/2'
   PRINT *
   
   species_loop: DO I = 1,numspecies
      molecules_loop: DO J = 1,nummols(I)
         atoms_loop:   DO K = 2,numatoms(I) 
             
            sd = tmp_s(1:3,K,J,I) - tmp_s(1:3,K-1,J,I)
            sd = sd - ANINT(sd)
            tmp_s(1:3,K,J,I) = tmp_s(1:3,K-1,J,I) + sd
             
         END DO atoms_loop
      END DO molecules_loop
   END DO species_loop
      
   ! Retransform joined trajectory to cartesian
   CALL CONFIG_FRACT_2_CART (tmp_s, config_pos)

   END SUBROUTINE CONFIG_JOIN_MOLECULES


   SUBROUTINE CONFIG_CENTEROFMASS (config_pos, config_com, com_species, com_nummols)

   REAL ( KIND = DP ), INTENT(IN), ALLOCATABLE, DIMENSION(:,:,:,:) ::  config_pos
   REAL ( KIND = DP ), INTENT(OUT), ALLOCATABLE, DIMENSION(:,:,:,:) :: config_com

   INTEGER, INTENT(OUT)                            :: com_species
   INTEGER, INTENT(OUT), ALLOCATABLE, DIMENSION(:) :: com_nummols
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)   :: molmass

   INTEGER :: I, J, K, index

   ! Check if minimum needed information is available
   IF (numspecies <= 0) STOP 'CONFIG_CENTEROFMASS: Number of different species not set!'
   IF (maxval(nummols) <= 0) STOP 'CONFIG_CENTEROFMASS: Number of molecules of each type not set!'
   IF (maxval(numatoms) <= 0) STOP 'CONFIG_CENTEROFMASS: Number of atoms in each molecule not set!'

   ! Check if atomic masses available
   IF (.NOT.ALLOCATED(atommass)) STOP 'CONFIG_CENTEROFMASS: Error!!! Atomic masses not available'
   IF (SIZE(atommass,2) /= numspecies) STOP 'CONFIG_CENTEROFMASS: Number of species /= Size of atommass'
   
   ! Compute mass of molecule   
   IF ( ALLOCATED(molmass) )     DEALLOCATE(molmass)
   IF ( ALLOCATED(com_nummols) ) DEALLOCATE(com_nummols)
   ALLOCATE ( molmass(numspecies) ); molmass = 0.0 
   ALLOCATE ( com_nummols(numspecies) ); com_nummols(:) = 0.0   
   
   com_species = 0
   DO I = 1,numspecies
      molmass(I) = 0.0
      DO J = 1,numatoms(I)
         molmass(I) = molmass(I) + atommass(J,I)
      END DO
      IF (molmass(I) > 0) THEN
         com_species = com_species + 1
         com_nummols(com_species) = nummols(I) 
      END IF       
   END DO
   IF (com_species <= 0) STOP 'CONFIG_CENTEROFMASS: Error!!! No species retained'
   
   ! Compute COM positions

   IF ( ALLOCATED(config_com) ) DEALLOCATE(config_com)
   ALLOCATE ( config_com ( 3, 1, maxval(com_nummols), com_species ) )
   config_com = 0.0  

   index = 0 !index of molecules with mass non null
   species_loop: DO I = 1,numspecies

      IF (molmass(I) > 0) THEN
         index = index + 1
         molecules_loop: DO J = 1,nummols(I)
            atoms_loop:   DO K = 1,numatoms(I) 
               config_com(1:3,1,J,index) = config_com(1:3,1,J,index) + &
                                           atommass(K,I) * config_pos(1:3,K,J,I)
            END DO atoms_loop
          config_com(1:3,1,J,index) = config_com(1:3,1,J,index) / molmass(I)
         END DO molecules_loop
      END IF

   END DO species_loop

   END SUBROUTINE CONFIG_CENTEROFMASS


END MODULE CONFIG_UTILS   

   
