!
! Program to find well defined molecular pairs (such that the closest neighbour of each of the
! two molecules of the pair is the other one)
!
! 	Reads:
!	1. CONFIG/REVCON (+FIELD) from DL_POLY
!
!	Writes:
!	1. File detailing
!
!  Last change: 7/2/2012
!

PROGRAM FIND_PAIRS	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE GENERAL_VARIABLES
   USE CONFIG_VARIABLES,       ONLY : cell
   USE FORCEFIELD_VARIABLES,   ONLY : atomname
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE CONFIG_READWRITE,       ONLY : CONFIG_DLPOLY_READ
   USE CONFIG_UTILS,           ONLY : CONFIG_JOIN_MOLECULES, CONFIG_CENTEROFMASS, CONFIG_CART_2_FRACT
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH)   :: config_file, ff_file, output_file, pdb_file, pdb_atom_file
   CHARACTER ( LEN = 3)             :: name
   INTEGER                          :: I, J, K, I1, I2, ITEMP, J1, J2, JTEMP, UNIT, IOSTAT, index, ntemp
   INTEGER                          :: select_species, number_pairs
   REAL ( KIND = DP )               :: rx, ry, rz, distance, temp
   REAL ( KIND = DP ), DIMENSION(3) :: sdist
  
   INTEGER, ALLOCATABLE, DIMENSION(:)                  :: select_nummols, mol_list, spec_list
   INTEGER, ALLOCATABLE, DIMENSION(:,:,:)              :: neighbour
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)       :: dist_list
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)     :: closest_distance, pair_distance
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:) :: rrr, com, comfract
						      

   ! Get input
   
   WRITE (OUTPUT,'("CONFIG file name?")')
   READ (INPUT,'(A)') config_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Name of output file to write pairs?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,'("Name of PDB file to write configuration of ordered CM?")')
   READ (INPUT,'(A)') pdb_file
   WRITE (OUTPUT,'("Name of PDB file to write configuration of ordered molecules?")')
   READ (INPUT,'(A)') pdb_atom_file
  

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) )  DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
      WRITE ( OUTPUT, '("System information:")' ) 
      WRITE ( OUTPUT, '("Number of different species = ",I3)') numspecies
      DO I = 1,numspecies
         WRITE ( OUTPUT, "('Molecule ',I3,' contains ',I3,' atoms')" ) I,numatoms(I)
      END DO
      WRITE ( OUTPUT, * )
   END IF 
      
   CALL CONFIG_DLPOLY_READ ( config_file, rrr )
   CALL CONFIG_JOIN_MOLECULES ( rrr )
   CALL CONFIG_CENTEROFMASS ( rrr, com, select_species, select_nummols )
   CALL CONFIG_CART_2_FRACT ( com, comfract)

   ALLOCATE ( closest_distance ( select_species, maxval(select_nummols) ) )
   ALLOCATE ( pair_distance ( select_species, maxval(select_nummols) ) )
   ALLOCATE ( neighbour ( select_species, maxval(select_nummols), 2 ) )
   closest_distance = 1.e6; pair_distance = 1.e6;neighbour = 0

   index = 0
   ALLOCATE ( dist_list ( select_species * maxval(select_nummols) ) )
   ALLOCATE ( mol_list ( select_species * maxval(select_nummols) ) )
   ALLOCATE ( spec_list ( select_species * maxval(select_nummols) ) )
   dist_list = 0.0; mol_list = 0; spec_list = 0

   mol1_species_loop: DO I1 = 1, select_species
      mol1_molecules_loop: DO J1 = 1,select_nummols(I1)

         mol2_species_loop: DO I2 = 1, select_species
         mol2_molecules_loop: DO J2 = 1, select_nummols(I2)
		        
	    IF ( (I1 == I2) .AND. (J1 == J2) ) CYCLE mol2_molecules_loop

	    ! Obtain distance between molecular centers
			
	    sdist= comfract(1:3,1,J1,I1) - comfract(1:3,1,J2,I2)
	    sdist = sdist - ANINT(sdist)
            rx = sdist(1)*cell(1,1) + sdist(2)*cell(2,1) + sdist(3)*cell(3,1)
            ry = sdist(1)*cell(1,2) + sdist(2)*cell(2,2) + sdist(3)*cell(3,2)
            rz = sdist(1)*cell(1,3) + sdist(2)*cell(2,3) + sdist(3)*cell(3,3)
	    distance = SQRT( rx*rx + ry*ry + rz*rz )
	    IF ( distance < closest_distance(I1,J1) ) THEN
	       closest_distance(I1, J1) = distance
	       pair_distance(I1, J1) = closest_distance(I1, J1)
	       neighbour(I1, J1, 1) = I2
	       neighbour(I1, J1, 2) = J2
	    END IF 
            			
	 END DO mol2_molecules_loop	
	 END DO mol2_species_loop
	 
	 !Need list of closest distances having a single index
	 index = index + 1
	 dist_list(index) = closest_distance(I1, J1)
	 spec_list(index) = I1
	 mol_list(index) = J1
	 
         !Check if pair is well defined, i.e. the second molecule does not have a closest neighbour
         DO ITEMP = 1, select_species
	    DO JTEMP = 1, select_nummols(ITEMP)
	       IF ( (ITEMP /= I1) .AND. (JTEMP /= J1) .AND. (pair_distance(I1,J1) < 1.e6) ) THEN
	          I2 = neighbour(I1,J1,1)
		  J2 = neighbour(I1,J1,2)
		  IF ( (ITEMP /= I2) .AND. (JTEMP /= J2) ) THEN
	             sdist= comfract(1:3,1,JTEMP,ITEMP) - comfract(1:3,1,J2,I2)
	             sdist = sdist - ANINT(sdist)
                     rx = sdist(1)*cell(1,1) + sdist(2)*cell(2,1) + sdist(3)*cell(3,1)
                     ry = sdist(1)*cell(1,2) + sdist(2)*cell(2,2) + sdist(3)*cell(3,2)
                     rz = sdist(1)*cell(1,3) + sdist(2)*cell(2,3) + sdist(3)*cell(3,3)
	             distance = SQRT( rx*rx + ry*ry + rz*rz )
	             IF (distance < pair_distance(I1,J1) ) THEN 
	                pair_distance(I1, J1) = 1.e6
	                neighbour(I1, J1, 1) = 0
	                neighbour(I1, J1, 2) = 0
	             END IF
		  END IF   
	       END IF	     
	    END DO
	 END DO

      END DO mol1_molecules_loop	
   END DO mol1_species_loop	
      
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT,'("# Pair distance - Atom 1 ident - Atom 2 ident")')
   number_pairs = 0
   DO I1 = 1, select_species
      DO J1 = 1, select_nummols(I1)
	 IF ( pair_distance(I1, J1) < 1.e6) THEN
            WRITE (UNIT,'(F8.4,5x,I3,1x,I3,5x,I3,1x,I3)') pair_distance(I1,J1), I1, J1, &
	                                                  neighbour(I1, J1, 1), neighbour(I1, J1, 2) 
            number_pairs = number_pairs + 1
	 END IF
      END DO
   END DO
   WRITE (UNIT,'("# Number of different pairs = ",I6)') number_pairs/2
   CLOSE (UNIT)
   
   !Now sort the molecules by their closest distance and write them as a PDB file
   DO I = 1,index-1
      DO J = I+1,index
         IF ( dist_list(I) > dist_list(J) ) THEN
                     temp = dist_list(J); dist_list(J)=dist_list(I); dist_list(I)=temp
                     ntemp = mol_list(J); mol_list(J)=mol_list(I); mol_list(I)=ntemp 
                     ntemp = spec_list(J); spec_list(J)=spec_list(I); spec_list(I)=ntemp 
         END IF
      END DO
   END DO

   ! Open output file
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = pdb_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'CONFIG_PDB_WRITE: Problem opening file!'
   DO I = 1,index
       IF (I < index/2) THEN 
          name = 'Si ' 
       ELSE 
          name = 'Ge '
       END IF
       WRITE (UNIT,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') &
	   &'ATOM  ', I, name, 'XXX', I, com(1,1,mol_list(I),spec_list(I)), &
	   &com(2,1,mol_list(I),spec_list(I)), com(3,1,mol_list(I),spec_list(I)), 0.0, 0.0, 'XXX' 
   END DO
   CLOSE (UNIT)

   ! Open output file
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = pdb_atom_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'CONFIG_PDB_WRITE: Problem opening file!'
   DO I = 1,index
      DO J = 1,numatoms(spec_list(I))
       WRITE (UNIT,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') &
	   &'ATOM  ', I, atomname(J,spec_list(I)), 'XXX', I, rrr(1,J,mol_list(I),spec_list(I)), &
	   &rrr(2,J,mol_list(I),spec_list(I)), rrr(3,J,mol_list(I),spec_list(I)), 0.0, 0.0, 'XXX' 
       END DO
   END DO
   CLOSE (UNIT)
   
  
END PROGRAM FIND_PAIRS
