!
! Program to calculate distance and orientational distributions for 1st, 2nd, 3rd, etc. neighbours
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Writes:
!	1. A file containing the fraction of species for each neighbour order.
!       2. A file containing the distance distribution for each neighbour order.
!       3. A file containing the orientation distribution for each neighbour order.
!
!  Last change: 21/6/2011
!

PROGRAM NEIGHBOUR_DISTRIBUTION	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES,   ONLY : frame_cell, average_cell_volume, pbc_jumps
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,       ONLY : TRAJECT_UNWRAP, TRAJECT_JOIN_MOLECULES, TRAJECT_CART_2_FRACT
   USE ORIENTATION_UTILS,      ONLY : DEFINE_MOLECULAR_ORIENTATION_GEN, DEFINE_ORIENTATION_IMIDAZOLIUM_1
   USE LINEAR_ALGEBRA,         ONLY : NORMALIZE
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH)     :: traject_file, ff_file, output_file_1, output_file_2, output_file_3
   INTEGER                            :: IFRAME, I, J, K, I1, I2, J1, J2, UNIT, IOSTAT
   INTEGER                            :: nframes, selected_species, progress, maxneighbour
   INTEGER                            :: bin, binth, dist_index, ntemp, maxbin, maxbinth, nvector
   INTEGER, DIMENSION(3)              :: triademol
   REAL ( KIND = DP ), DIMENSION(3)   :: mol_center, sdist
   REAL ( KIND = DP ), DIMENSION(3,3) :: mol_axes


   REAL ( KIND = DP ) :: dr, dtheta, rx, ry, rz, dot, theta, sinth, total_weight, temp
  
   INTEGER, ALLOCATABLE, DIMENSION(:,:)                  :: triade, neighbours, neighbour_population, neighbour_distance
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)         :: rneighbour, weightmol, stemp
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)       :: weight, molecule, neighbour_orientation
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr, sss, traj_mol_center, traj_mol_axes
						      

   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file to store the fraction of neighbours of each species?")')
   READ (INPUT,'(A)') output_file_1
   WRITE (OUTPUT,'("Name of output file to store the distance distribution for each neighbour?")')
   READ (INPUT,'(A)') output_file_2
   WRITE (OUTPUT,'("Name of output file to store the orientation distribution for each neighbour?")')
   READ (INPUT,'(A)') output_file_3
   WRITE (OUTPUT,'("Maximum number of neighbours?")')
   READ (INPUT,*) maxneighbour
   WRITE (OUTPUT,'("Step in distance histogram?")')
   READ (INPUT,*) dr
   WRITE (OUTPUT,'("Step in orientation histogram?")')
   READ (INPUT,*) dtheta
   WRITE (OUTPUT,'("Vector to use in orientation calculation (1, 2 or 3)?")')
   READ (INPUT,*) nvector
   IF (nvector < 1) THEN
      PRINT *, 'Using nvector = 1'
      nvector = 1
   END IF
   IF (nvector > 3) THEN
      PRINT *, 'Using nvector = 3'
      nvector = 3
   END IF
   WRITE (OUTPUT,*)
  

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) )  DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
      WRITE ( OUTPUT, '("System information:")' ) 
      WRITE ( OUTPUT, '("Number of different species = ",I3)') numspecies
      DO I = 1,numspecies
         WRITE ( OUTPUT, "('Molecule ',I3,' contains ',I3,' atoms')" ) I,numatoms(I)
      END DO
      WRITE ( OUTPUT, * )
   END IF 
      

   ! Give desired weights to calculate the COM of each molecule & 3 atoms to define the molecular orientation
   ! Axis 1: Along vector atom1-->atom2
   ! Axis 2: Perpendicular to 1 and in plane formed by the three atoms
   ! Axis 3: Axis 1 x Axis 2
   
   ALLOCATE ( weight ( maxval(numatoms), numspecies ) );   weight = 0.0
   ALLOCATE ( weightmol ( maxval(numatoms) ) );   weightmol = 0.0
   ALLOCATE ( triade ( 3, numspecies ) );   triade = 0
   ALLOCATE ( molecule ( 3, maxval(numatoms) ) ); molecule = 0.0
   DO I = 1,numspecies
      WRITE (OUTPUT,'("Molecule type ",I3,": Number of atoms = ",I4)') I,numatoms(I)
      WRITE (OUTPUT,'("Give weights of all atoms ")')
      READ ( INPUT, * ) ( weight(J,I), J=1,numatoms(I))
      IF (numatoms(I) >= 3) THEN
         WRITE (OUTPUT,'("Give numbers of 3 atoms to define the molecular orientation")')
	 READ ( INPUT, * ) (triade(J,I), J=1,3)
      END IF
   END DO
   WRITE (OUTPUT,'("Number of the selected species? ")')
   READ (INPUT,*) selected_species
   IF (selected_species > numspecies) STOP 'Error! Not so many species in trajectory'
  

   ! Read trajectory and unwrap and re-join molecules

   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   PRINT *,'Finished reading trajectory!'
   IF (pbc_jumps) CALL TRAJECT_UNWRAP (nframes, rrr)  
   CALL TRAJECT_JOIN_MOLECULES (nframes, rrr)


   ! Now calculate trajectory of molecular centers and axes
     
   IF (ALLOCATED(traj_mol_center)) DEALLOCATE(traj_mol_center)
   IF (ALLOCATED(traj_mol_axes))   DEALLOCATE(traj_mol_axes)
   ALLOCATE ( traj_mol_center ( 3, 1, maxval(nummols), numspecies, nframes ) ); traj_mol_center = 0.0 
   ALLOCATE ( traj_mol_axes ( 3, 3, maxval(nummols), numspecies, nframes ) );   traj_mol_axes = 0.0
   
   DO IFRAME = 1, nframes

      DO I = 1, numspecies

         DO J = 1, nummols(I)
   
            molecule(1:3,:) = rrr ( 1:3, :, J, I, IFRAME)
            weightmol = weight(:,I)
            triademol = triade(:,I)

            IF (numatoms(I) >= 3) THEN

	       CALL DEFINE_MOLECULAR_ORIENTATION_GEN (molecule, weightmol, triademol, mol_center, mol_axes)

            ELSE

               mol_center = 0.0; total_weight = 0.0
               DO K = 1, numatoms(I)
                  mol_center(1:3) = mol_center(1:3) + weight(K,I) * molecule(1:3,K)
                  total_weight = total_weight + weight(K,I)
               END DO 
               mol_center = mol_center / total_weight
               IF (numatoms(I) == 2) THEN
                  ! Define first axis as bond between both atoms
                  mol_axes(1:3,1) = molecule(1:3,2) - molecule(1:3,1)	 
                  mol_axes(1:3,1) = NORMALIZE( mol_axes(1:3,1) )
               ELSE 
                  ! Use the cartesian axes
                  mol_axes(1:3,1) =  (/ 1.0d0, 0.0d0, 0.0d0 /)
               END IF
               mol_axes(1:3,2) =  (/ 0.0d0, 1.0d0, 0.0d0 /)
               mol_axes(1:3,3) =  (/ 0.0d0, 0.0d0, 1.0d0 /)

            END IF
            
	    traj_mol_center(1:3,1,J,I,IFRAME) = mol_center(1:3)
	    traj_mol_axes(1:3,1:3,J,I,IFRAME) = mol_axes(1:3,1:3)

         END DO	 

      END DO
      
   END DO
   !print *,'traj_mol_center & traj_mol_axes computed'

   ! Calculate trajectory of centers in fractional units
   
   CALL TRAJECT_CART_2_FRACT (nframes, traj_mol_center, sss)
   
   ! Allocate arrays to contain neighbour distances and list of neighbours & Compute histograms
   
   ALLOCATE ( rneighbour ( maxneighbour+1 ) ); rneighbour = 0.0 
   ALLOCATE ( neighbours (maxneighbour+1, 4 ) ); neighbours = 0
   ALLOCATE ( neighbour_population (maxneighbour, numspecies ) ); neighbour_population = 0
   ALLOCATE ( neighbour_distance (maxneighbour, 9999 ) ); neighbour_distance = 0
   maxbin = 0
   maxbinth = 180.0 / dtheta
   ALLOCATE ( neighbour_orientation (maxneighbour, maxbinth ) ); neighbour_orientation = 0.0

   
   frames_loop: DO IFRAME = 1,nframes

      progress = nframes / 20
      IF (progress < 1) progress = 1
      IF (MOD(IFRAME,progress) == 0) PRINT *,'Neighbour distribution. Completed ',(100*IFRAME/nframes),' %'

      dist_index = 0   
      mol1_species_loop: DO I1 = selected_species,selected_species
      mol1_molecules_loop: DO J1 = 1,nummols(I1)

         ! Set to zero neighbour distances and lists for the current molecule
         dist_index = 0
         rneighbour = 0.0
         neighbours = 0

         mol2_species_loop: DO I2 = 1,numspecies
         mol2_molecules_loop: DO J2 = 1,nummols(I2)
		        
	    IF ( (I1 == I2) .AND. (J1 == J2) ) CYCLE mol2_molecules_loop
            dist_index = dist_index + 1
			
	    ! Obtain distance between molecular centers
			
	    sdist= sss(1:3,1,J1,I1,IFRAME) - sss(1:3,1,J2,I2,IFRAME)
	    sdist = sdist - ANINT(sdist)
            rx = sdist(1)*frame_cell(1,1,IFRAME) + sdist(2)*frame_cell(2,1,IFRAME) + sdist(3)*frame_cell(3,1,IFRAME)
            ry = sdist(1)*frame_cell(1,2,IFRAME) + sdist(2)*frame_cell(2,2,IFRAME) + sdist(3)*frame_cell(3,2,IFRAME)
            rz = sdist(1)*frame_cell(1,3,IFRAME) + sdist(2)*frame_cell(2,3,IFRAME) + sdist(3)*frame_cell(3,3,IFRAME)
            rneighbour(dist_index) = SQRT( rx*rx + ry*ry + rz*rz )
            neighbours(dist_index,1) = I1
            neighbours(dist_index,2) = J1
            neighbours(dist_index,3) = I2
            neighbours(dist_index,4) = J2

            ! Simple sorting
            
            DO I = 1,dist_index-1
               DO J = I+1,dist_index
                  IF ( rneighbour(I) > rneighbour(J) ) THEN
                     temp = rneighbour(J); rneighbour(J)=rneighbour(I); rneighbour(I)=temp
                     ntemp = neighbours(J,1); neighbours(J,1)=neighbours(I,1); neighbours(I,1)=ntemp 
                     ntemp = neighbours(J,2); neighbours(J,2)=neighbours(I,2); neighbours(I,2)=ntemp 
                     ntemp = neighbours(J,3); neighbours(J,3)=neighbours(I,3); neighbours(I,3)=ntemp 
                     ntemp = neighbours(J,4); neighbours(J,4)=neighbours(I,4); neighbours(I,4)=ntemp 
                  END IF
               END DO
            END DO		     
            IF (dist_index > maxneighbour) dist_index = maxneighbour
            			
	 END DO mol2_molecules_loop	
	 END DO mol2_species_loop
         !PRINT *,'Nearest neighbours for mol:',J1
         !DO I = 1,maxneighbour
         !   PRINT *,neighbours(I,1:4),rneighbour(I)
         !END DO	
         !STOP

         !Calculate distances and orientations for nearest neighbours

         DO I = 1, maxneighbour

            DO J = 1, numspecies
               IF (neighbours(I,3) == J) neighbour_population(I,J) = neighbour_population(I,J) + 1
            END DO

            bin = INT(rneighbour(I)/dr) + 1
            IF (bin > 9999) STOP 'Too large distance!'
            IF (bin > maxbin) maxbin = bin 
            neighbour_distance(I,bin) = neighbour_distance(I,bin) + 1
            dot = DOT_PRODUCT ( traj_mol_axes(1:3,nvector,neighbours(I,2),neighbours(I,1),IFRAME), &
                                traj_mol_axes(1:3,nvector,neighbours(I,4),neighbours(I,3),IFRAME) )
	    theta = ACOS(dot)*180.0/pi
	    binth = INT(theta/dtheta) + 1
	    !weight each point by 1/sin(theta) to take into account solid angle
	    sinth = SIN(ACOS(dot))
	    IF (sinth < 1.0e-3) sinth = 1.0e-3
            neighbour_orientation(I,binth) = neighbour_orientation(I,binth) + (1./sinth)

         END DO	

      END DO mol1_molecules_loop	
      END DO mol1_species_loop	

   END DO frames_loop

   ! Write results

   ! Neighbour population for different species
   
   ALLOCATE ( stemp (numspecies ) ); stemp=0.0
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file_1, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT,'("# Neighbour population for different species")')
   WRITE (UNIT,'("#",3x,"Neighbour_order - Fraction of neighbours of each species")')
   DO I = 1, maxneighbour
      ntemp = SUM(neighbour_population(I,:))
      print *,'Number of neighbours found ',I,ntemp,SUM(neighbour_distance(I,:))
      DO J = 1, numspecies
         stemp(J) = float(neighbour_population(I,J))/float(ntemp)
      END DO
      WRITE (UNIT,'(I2,5x,10(F5.3,1x))') I, (stemp(J), J = 1, numspecies)
   END DO
   CLOSE (UNIT)

   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file_2, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'Output file for distance histogram could not be created!'
   WRITE (UNIT,'("# Distance distribution for each neighbour")')
   WRITE (UNIT,'("#",3x,"r - Neighbour order")')
   DO J = 1, maxbin
       temp = float(J-1)*dr + 0.5*dr
       WRITE (UNIT,'(F5.2,5x,20(I6,1x))') temp, (neighbour_distance(I,J),I=1,maxneighbour)
   END DO
   CLOSE (UNIT)

   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file_3, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'Output file for orientation histogram could not be created!'
   WRITE (UNIT,'("# Orientation distribution for each neighbour")')
   WRITE (UNIT,'("#",3x,"r - Neighbour order")')
   DO J = 1, maxbinth
       temp = float(J-1)*dtheta + 0.5*dtheta
       WRITE (UNIT,'(F5.1,5x,20(F8.3,1x))') temp, (neighbour_orientation(I,J)/float(nframes),I=1,maxneighbour)
   END DO
   CLOSE (UNIT)

   
END PROGRAM NEIGHBOUR_DISTRIBUTION
