!===============================================================================
! This file is part of MG_DLP_analysis - a set of programs to perform
! different analysis on Molecular Dynamics simulations, mainly intended
! to handle trajectories generated by DL_POLY (HISTORY format).
!
! Copyright (C) 2021 Miguel A. Gonzalez
!
!    MG_DLP_analysis is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    MG_DLP_analysis is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with MG_DLP_analysis.  If not, see <https://www.gnu.org/licenses/>.
!
!===============================================================================
!
! Module to read a force field. 
!
! Subroutines:
!
!      FIELD_DLPOLY_READ (field_file)
!
! Main variables:
!
!       numspecies
!       molname(numspecies), 
!       nummols(numspecies), 
!       numatoms(numspecies)
!       atomname(numatoms,numspecies), 
!       atommass(numatoms,numspecies) 
!       atomcharge(numatoms,numspecies)    
!
!  Last change: 22/12/2009
!
!===============================================================================

MODULE FORCEFIELD_READ

USE FILES, ONLY : NEXT_UNIT
USE PARSING, ONLY : GET_LINE, GET_WORD, GET_INTEGER, GET_REAL, WORD, WRDLEN, EMPTY, PUSH_UNIT
USE GENERAL_VARIABLES
USE FORCEFIELD_VARIABLES

IMPLICIT NONE
PUBLIC

CONTAINS


   SUBROUTINE FIELD_DLPOLY_READ (field_file)

   CHARACTER ( LEN = * ), INTENT(IN) :: field_file

   LOGICAL :: end
   INTEGER :: I, IOSTAT, UNIT, index, nrept
      

   ! Open the file
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = field_file, ACTION = "READ", STATUS = "OLD", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'FIELD file not found!'
   
   ! Set the parsing unit.
   CALL PUSH_UNIT ( UNIT )
      
   ! Get forcefield file title
   READ ( UNIT, "(A)", IOSTAT = IOSTAT ) ff_header
     
   ! . Get energy units employed
   CALL GET_LINE  
   CALL GET_WORD   
   IF (WORD(1:WRDLEN) == "UNITS") THEN
      CALL GET_WORD
      energy_units = WORD(1:WRDLEN)
   ELSE
      PRINT *,'FIELD_DLPOLY_READ: WARNING!!! UNITS KEYWORD NOT FOUND' 
   END IF
      
   DO
      CALL GET_LINE
      CALL GET_WORD
      IF (WORD(1:WRDLEN) == "NEUT") neut = .true.
      IF (WORD(1:WRDLEN) == "MOLECULES") THEN
       CALL GET_INTEGER (numspecies)
       EXIT
      END IF
   END DO
   
   ! Allocate and initialize arrays
   IF ( ALLOCATED(molname) )  DEALLOCATE(molname)
   IF ( ALLOCATED(nummols) )  DEALLOCATE(nummols)
   IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
   ALLOCATE ( molname(numspecies), nummols(numspecies), numatoms(numspecies) )
   molname(:) = ''; nummols(:) = 0; numatoms(:) = 0
 
   species_loop_1: DO I = 1,numspecies

      ! Get name of molecule
      READ ( UNIT, "(A)", IOSTAT = IOSTAT ) molname(I)

      ! Get number of molecules of this type
      DO
         CALL GET_LINE  
         CALL GET_WORD
         IF (WORD(1:WRDLEN) == "NUMMOLS") THEN
            CALL GET_INTEGER (nummols(I))
          EXIT
         END IF
      END DO

      ! Get number of atoms in this molecule
      DO
         CALL GET_LINE  
       CALL GET_WORD
       IF (WORD(1:WRDLEN) == "ATOMS") THEN
          CALL GET_INTEGER (numatoms(I))
          EXIT
       END IF
      END DO

      ! Search finish keyword
      DO
       CALL GET_LINE
       CALL GET_WORD
       IF (WORD(1:WRDLEN) == "FINISH") EXIT
      END DO

   END DO species_loop_1

   ! Close the FIELD file 
   CLOSE ( UNIT )

   
   ! Allocate arrays for atomic information
   IF ( ALLOCATED(atomname) ) DEALLOCATE(atomname)
   IF ( ALLOCATED(atommass) ) DEALLOCATE(atommass)
   IF ( ALLOCATED(atomcharge) ) DEALLOCATE(atomcharge)
   ALLOCATE ( atomname ( maxval(numatoms), numspecies ) ) 
   ALLOCATE ( atommass ( maxval(numatoms), numspecies ) ) 
   ALLOCATE ( atomcharge(maxval(numatoms), numspecies) ) 
   atomname(:,:) = ''; atommass(:,:) = 0.0; atomcharge(:,:) = 0.0

   ! Reopen the file to read information about each atom
   OPEN ( UNIT, FILE = field_file, ACTION = "READ", STATUS = "OLD", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'FIELD file not found!'
   
   ! Set the parsing unit.
   CALL PUSH_UNIT ( UNIT )
      
   species_loop_2: DO I = 1,numspecies
   
      ! Get name, mass and charge of each atom
      index = 1 !index for atom number
      atom_loop: DO
         CALL GET_LINE  
         CALL GET_WORD
         IF (WORD(1:WRDLEN) == "ATOMS") THEN
            DO
               CALL GET_LINE  
               CALL GET_WORD
               atomname(index,I) = WORD(1:WRDLEN)
               CALL GET_REAL (atommass(index,I))
               CALL GET_REAL (atomcharge(index,I))
               end = EMPTY()
               IF (.NOT.end) THEN
                  CALL GET_INTEGER (nrept)
                  IF (nrept > 1) THEN
                     atomname(index+1:index+nrept-1,I) = atomname(index,I)
                     atommass(index+1:index+nrept-1,I) = atommass(index,I)
                     atomcharge(index+1:index+nrept-1,I) = atomcharge(index,I)
                  ENDIF
                  index = index+nrept
               ELSE
                  index = index+1
               END IF
               IF (index > numatoms(I)) EXIT
            END DO
         ELSE
            IF (WORD(1:WRDLEN) == "FINISH") EXIT atom_loop 
         ENDIF
      ENDDO atom_loop
      
   END DO species_loop_2

   ! Close the FIELD file 
   CLOSE ( UNIT )
             
   END SUBROUTINE FIELD_DLPOLY_READ

   
END MODULE FORCEFIELD_READ   

   
