!
! Program to calculate the coherent intermediate scattering function.
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Writes:
!	1. A series of ASCII files containing Q, F_ab(Q,t)
!
!  Last change: 20/07/2017
!

PROGRAM FQT_COH	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
   USE GENERAL_VARIABLES
   USE FORCEFIELD_VARIABLES,   ONLY : atomname
   USE TRAJECTORY_VARIABLES,   ONLY : frame_cell, frame_nstep, tstep
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,       ONLY : TRAJECT_CART_2_FRACT
   USE GENERAL_UTILS,          ONLY : HKL_VECTORS, INVERT_CELL
  
   IMPLICIT NONE

   INTEGER, PARAMETER :: max_number_types = 99

   LOGICAL                                            :: new_atom
   CHARACTER ( LEN = LINE_LENGTH)                     :: traject_file, ff_file, output_file
   CHARACTER ( LEN = 3 ), DIMENSION(max_number_types) :: type_name
   CHARACTER ( LEN = 4 )                              :: partial_name
   CHARACTER ( LEN = 2 )                              :: block_name
   INTEGER, DIMENSION(max_number_types)                :: number_atoms_type
   REAL ( KIND = DP ), DIMENSION(max_number_types)    :: b_type
   REAL ( KIND = DP ), DIMENSION(3)                   :: sdist
   REAL ( KIND = DP ), DIMENSION(3,3)                 :: rcell

   INTEGER :: IFRAME, I, J, K, L, I1, J1, K1, ITYPE, IQ, IVEC, IOSTAT, UNIT, nframes, number_types, &
              total_number_atoms, number_partials, index, T, T0, progress, itmp, QLOOP, NBLOCKS, &
              INB, IQ1, IQ2, maxvec, maxtotvec
              
   REAL ( KIND = DP ) :: qmin, qmax, qdel, qini, qend, weight_norm, rx, ry, rz, r, vcell
                         
   INTEGER, ALLOCATABLE, DIMENSION(:,:)                  :: type_atom
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr, sss
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)         :: weight_partial!, gr_intra, gr_inter, gr_total

   INTEGER                               :: qnlast, nqval
   
   INTEGER, ALLOCATABLE, DIMENSION(:,:)  :: hkl
   INTEGER, ALLOCATABLE, DIMENSION(:)    :: qnum
   INTEGER, ALLOCATABLE, DIMENSION(:)    :: nqave

   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)       :: qmodule
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)       :: qval, qave
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)     :: qvec
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:)   :: rhor, rhoi, mrhoi
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:) :: fqtcoh, fqtima
   
   REAL ( KIND = DP ) :: qr


   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,'("Minimum Q value?")')
   READ (INPUT,*) qmin
   WRITE (OUTPUT,'("Maximum Q value?")')
   READ (INPUT,*) qmax
   WRITE (OUTPUT,'("Delta Q (0 keeps all vectors separated)?")')
   READ (INPUT,*) qdel
   WRITE (OUTPUT,'("Max number of vectors per shell (0 for unlimited)?")')
   READ (INPUT,*) maxvec
   WRITE (OUTPUT,'("Max total number of vectors?")')
   READ (INPUT,*) maxtotvec
   WRITE (OUTPUT,*)
   
   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	     WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	     READ (INPUT,*) nummols(I) 
	     WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	     READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 
      

   ! Determine number of different types of atoms
   
   IF ( ALLOCATED(type_atom) ) DEALLOCATE(type_atom)
   ALLOCATE ( type_atom ( maxval(numatoms), numspecies ) )
   
   number_types = 0
   total_number_atoms = 0
   number_atoms_type(:) = 0
   
   IF ( ALLOCATED(atomname) ) THEN
      IF ( SIZE(atomname) >= 1 ) THEN
   
         species_loop: DO I = 1,numspecies
            atoms_loop:   DO K = 1,numatoms(I) 
	 
	       ! Check if atom type is new
	       new_atom = .TRUE.
	       DO L = 1,number_types
		  IF (atomname(K,I)(1:3) == type_name(L) ) THEN
		     new_atom = .FALSE.
		     number_atoms_type(L) = number_atoms_type(L) + nummols(I)
		     total_number_atoms = total_number_atoms + nummols(I)
		     type_atom(K,I) = L
		  END IF	 
	       END DO
	       IF (new_atom) THEN
		  number_types = number_types + 1
		  IF ( number_types > max_number_types ) STOP 'Too many different types of atoms!'
	          number_atoms_type(number_types) = nummols(I)
	          total_number_atoms = total_number_atoms + nummols(I)
		  type_name(number_types) = atomname(K,I)(1:3)
		  type_atom(K,I) = number_types
	       END IF
		   
            END DO atoms_loop
         END DO species_loop
      
      ENDIF
   END IF
 
 
   ! If no atom information is available, consider each atom as a different one
   
   IF ( number_types == 0 ) THEN
   
      species_loop_2: DO I = 1,numspecies
         atoms_loop_2:   DO K = 1,numatoms(I) 
	 
            number_types = number_types + 1
	    IF ( number_types > max_number_types ) STOP 'Too many different types of atoms!'
	    type_atom(K,I) = number_types
            number_atoms_type(number_types) = nummols(I)
	    total_number_atoms = total_number_atoms + nummols(I)
	    WRITE(type_name(number_types),'(i3.3)') number_types
		   
         END DO atoms_loop_2
      END DO species_loop_2
	 
   END IF
   
   number_partials = number_types**2
   ALLOCATE ( weight_partial(number_partials) )

   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("Total number of atoms = ",I5)') total_number_atoms
   WRITE (OUTPUT,'("Number of different types of atom = ",I3)') number_types
   WRITE (OUTPUT,'("Number of partial F(Q,t)s = ",I3)') number_partials
   WRITE (OUTPUT,*)

   DO I = 1, number_types
      IF ( type_name(I)(3:3) == ' ' ) type_name(I)(3:3) = '_'
      IF ( type_name(I)(2:2) == ' ' ) type_name(I)(2:2) = '_'
      WRITE (OUTPUT, "(A,I2,A,A3,A,I8)" ) '     Type:', I, ' - Name:', type_name(I), ' - Number of atoms = ',number_atoms_type(I)
      WRITE (OUTPUT, "(A)" ) '     Give scattering length for this type:'
      READ (INPUT,*) b_type(I)
      WRITE (OUTPUT,*)
   END DO
   
   
   ! Calculate weighting factors for each partial g(r) and normalization factor
   
   ! index = 0
   ! weight_norm = 0.0
   ! DO I = 1,number_types
   !   DO J = 1,number_types
   !      index = index + 1
   !      weight_partial(index) = b_type(I)*b_type(J)*REAL(number_atoms_type(I)*number_atoms_type(J))/ &
   !	                                 REAL(total_number_atoms**2)
   !	  weight_norm = weight_norm + weight_partial(index)
   !   END DO
   ! END DO
   
   
   ! Normalize weighting factors for each partial g(r) and write contribution of each partial to total
   
   !index = 0
   !DO I = 1,number_types
   !   DO J = I,number_types
   !      index = index + 1
   !	     weight_partial(index) = weight_partial(index) / weight_norm
   !     WRITE (OUTPUT,"(A,I4,A,A3,A,A3,A,A,G12.6)") ' Partial ', index, ' : ',type_name(i),'-',type_name(j),' . ', &
   !	                                               & 'Contribution to total r.d.f. = ',weight_partial(index)
   !   END DO
   !END DO
   WRITE (OUTPUT,*)
     
   ! Read trajectory and transform to fractional coordinates
      
   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   PRINT *,'Finished reading trajectory!'
   CALL TRAJECT_CART_2_FRACT (nframes, rrr, sss)

   ! Compute Q vectors to use in analysis
   ! Uses cell vectors of the first frame in trajectory, so strictly correct only for NVT simulations

   CALL HKL_VECTORS (frame_cell(:,:,1), qmin, qmax, qdel, maxvec, maxtotvec, hkl, qmodule, qnum, qnlast)
   
   PRINT *
   PRINT *, 'Number of vectors found = ', qnlast 
   
   IF (qdel > 0.0) THEN
      qini = qmin - 0.5*qdel
      qend = qmax + 0.5*qdel
      nqval = (qend-qini) / qdel + 1
      ALLOCATE ( qval(nqval), qave(nqval), nqave(nqval) ) 
      qval = 0.0; qave = 0.0; nqave = 0
      DO I = 1, nqval
         qval(I) = qini + 0.5*qdel + (i-1)*qdel
      END DO
      DO I = 1, qnlast
         itmp = NINT((qmodule(I) - qini)/qdel + 0.5)
         qave(itmp) = qave(itmp) + qmodule(I)
         nqave(itmp) = nqave(itmp) + 1
      END DO
   ELSE
      ALLOCATE ( qval(qnlast), qave(qnlast), nqave(qnlast) )
      qval = qmodule
      qave = qmodule
      nqave = 1
      nqval = qnlast
   END IF
   
   !To skip last |Q| if it does not contain any vector
   
   IF (nqave(nqval) == 0) THEN 
      nqval = nqval - 1
   END IF   

   PRINT *
   PRINT *, 'Individual vectors: index, H, K, L, |Q|, group'
   PRINT *
   DO I = 1, qnlast
      PRINT "(3X,I6,3X,3(I4,1X),3X,F8.3,3X,I6)", I, hkl(I,:), qmodule(I), qnum(I) 
   END DO
   PRINT *
   IF (qdel > 0.0) THEN
      PRINT *, 'Q groups: Index, Nominal value, Number of vectors, average'
      PRINT *
      DO I = 1, nqval
         IF (nqave(I) > 0) THEN
            PRINT "(3X,I6,3X,F8.3,3X,I6,3X,F8.3)", I, qval(I), nqave(I), qave(I)/float(nqave(I))
         ELSE
            PRINT "(3X,I6,3X,F8.3,3X,I6)", I, qval(I), nqave(I)
         END IF         
      END DO
   END IF
   PRINT *
   
   
   ! Allocate arrays
   ALLOCATE ( qvec(qnlast,3) ) 
   ALLOCATE ( rhor(number_types, qnlast, nframes) )
   ALLOCATE ( rhoi(number_types, qnlast, nframes) )
   ALLOCATE ( mrhoi(number_types, qnlast, nframes) )
   rhor = 0.0; rhoi = 0.0; mrhoi = 0.0  
   IF (qdel > 0.0) THEN
      ALLOCATE ( fqtcoh(number_types, number_types, nqval, 0:nframes-1) )
      ALLOCATE ( fqtima(number_types, number_types, nqval, 0:nframes-1) )
   ELSE
      ALLOCATE ( fqtcoh(number_types, number_types, qnlast, 0:nframes-1) )
      ALLOCATE ( fqtima(number_types, number_types, qnlast, 0:nframes-1) )
   END IF
   fqtcoh = 0.0; fqtima = 0.0

   ! Compute microscopic density for each species
   frames_loop: DO IFRAME = 1,nframes

      progress = nframes / 20
      IF (progress < 1) progress = 1
      IF (MOD(IFRAME,progress) == 0) PRINT *,'Computing rho. Completed ',(100*IFRAME/nframes),' %'

      ! Q vectors to use in computation
      CALL INVERT_CELL (frame_cell(:,:,IFRAME), rcell, vcell)

      qvec_loop: DO IVEC = 1,qnlast

         atom1_species_loop: DO I1 = 1,numspecies
            atom1_molecules_loop: DO J1 = 1,nummols(I1)
               atom1_atoms_loop:   DO K1 = 1,numatoms(I1) 
               
                  ITYPE = type_atom(K1,I1)
                  
                  ! exp(-i*k*r) = cos(k*r) + i*sin(k*r)
                  qr = hkl(IVEC,1) * sss(1, K1, J1, I1, IFRAME) + &
                       hkl(IVEC,2) * sss(2, K1, J1, I1, IFRAME) + &
                       hkl(IVEC,3) * sss(3, K1, J1, I1, IFRAME)
                       
		  qr = qr * 2.0 * PI
                       
                  rhor(ITYPE, IVEC, IFRAME) = rhor(ITYPE, IVEC, IFRAME) + cos(qr)
                  rhoi(ITYPE, IVEC, IFRAME) = rhor(ITYPE, IVEC, IFRAME) + sin(qr)
                  
                  ! exp(-i*-k*r) = cos(-k*r) + i*sin(-k*r) = cos(k*r) + i*sin(-k*r)
                  mrhoi(ITYPE, IVEC, IFRAME) = mrhoi(ITYPE, IVEC, IFRAME) + sin(-qr)
                                    
	           END DO atom1_atoms_loop	
	        END DO atom1_molecules_loop	
         END DO atom1_species_loop
	 	
      END DO qvec_loop

   END DO frames_loop
   
   ! Compute correlation function for each possible pair
    
    qvec_loop2: DO IVEC = 1,qnlast       
       atomtype1_loop: DO I = 1,number_types
          atomtype2_loop: DO J = 1,number_types

             DO T = 0, nframes-1      !time difference
                DO T0 = 1, nframes - T !loop over different time origins
                   IF (qdel > 0.0) THEN
                      IQ = qnum(IVEC)
                   ELSE
                      IQ = IVEC
                   END IF   
                   fqtcoh(I,J,IQ,T) = fqtcoh(I,J,IQ,T) + rhor(I,IVEC,T0+T)*rhor(J,IVEC,T0)
                   fqtima(I,J,IQ,T) = fqtima(I,J,IQ,T) + rhoi(I,IVEC,T0+T)*mrhoi(J,IVEC,T0)
                END DO
             END DO   
          
          END DO atomtype2_loop	
       END DO atomtype1_loop	
    END DO qvec_loop2

    IF (qdel > 0.0) THEN
       QLOOP = nqval
    ELSE
       QLOOP = qnlast
    END IF   
    
    ! Normalize by number origins and by number of Q vectors used in each group
    DO I = 1, number_types
    DO J = 1, number_types
       ! Need to count twice when I=J, possibly to take into account the contribution 
       ! atom1(0)*atom2(t) and atom2(0)*atom1(t), but did not find clear mathematical 
       ! expression to explain this
       IF (I == J) THEN
          fqtcoh(I,J,:,:) = 2.0 * fqtcoh(I,J,:,:)
          fqtima(I,J,:,:) = 2.0 * fqtima(I,J,:,:)
       END IF
       DO IQ = 1, QLOOP
          DO IFRAME = 0, nframes-1
             IF (nqave(IQ) > 0) THEN
                fqtcoh(I,J,IQ,IFRAME) = fqtcoh(I,J,IQ,IFRAME) / REAL(nframes-IFRAME) / nqave(IQ)
                fqtima(I,J,IQ,IFRAME) = fqtima(I,J,IQ,IFRAME) / REAL(nframes-IFRAME) / nqave(IQ)
             ELSE
                fqtcoh(I,J,IQ,IFRAME) = 0.0
                fqtima(I,J,IQ,IFRAME) = 0.0
             END IF              
          END DO  
       END DO
    END DO
    END DO
    
    ! Write partials F_ab(Q,t)
    
    IF (MOD(QLOOP,20) == 0) THEN
       NBLOCKS = QLOOP/20
    ELSE
       NBLOCKS = QLOOP/20 + 1
    END IF
    
    DO INB = 1, NBLOCKS
    
       IQ1 = (INB-1) * 20 + 1
       IQ2 = MIN(INB*20, QLOOP)

       index = 0
       WRITE (block_name,"(I2.2)") INB
       
       DO I = 1,number_types
          DO J = 1,number_types
             
             ! Open file to write partial F_ab(Q,t)
             l = 100*I + J
	         WRITE (partial_name,"(I4.4)") l

             UNIT = NEXT_UNIT ( )	
             OPEN ( UNIT, FILE = 'fqtcoh_'//partial_name//'_q'//block_name//'.dat', ACTION = "WRITE", &
                    STATUS = "UNKNOWN", IOSTAT = IOSTAT )
             IF ( IOSTAT /= 0 ) STOP 'Output file for partial fqtcoh could not be created!'    
	         WRITE (UNIT,'(''# F_coh(Q,t) for partial '',a3,''-'',a3)') type_name(I),type_name(J)
             WRITE (UNIT,'(''#'')')
             WRITE (UNIT,'(''#'',5x,''t/ps, Qave = '',20(2x,g13.4))') (qave(IQ)/nqave(IQ), IQ=IQ1,IQ2)
             WRITE (UNIT,'(''#'')')
             DO IFRAME = 0, nframes-1
                 WRITE (UNIT,"(1X,F9.3,9x,20(2x,e13.6))") tstep*( frame_nstep(IFRAME+1)-frame_nstep(1) ), &
                                                          (fqtcoh(I,J,IQ,IFRAME), IQ=IQ1,IQ2)
             END DO          
             CLOSE (UNIT)

             UNIT = NEXT_UNIT ( )	
             OPEN ( UNIT, FILE = 'fqtima_'//partial_name//'_q'//block_name//'.dat', ACTION = "WRITE", &
                    STATUS = "UNKNOWN", IOSTAT = IOSTAT )
             IF ( IOSTAT /= 0 ) STOP 'Output file for partial fqtima could not be created!'    
	         WRITE (UNIT,'(''# Im(F_coh(Q,t)) for partial '',a3,''-'',a3)') type_name(I),type_name(J)
             WRITE (UNIT,'(''#'')')
             WRITE (UNIT,'(''#'',5x,''t/ps, Qave = '',20(2x,g13.4))') (qave(IQ)/nqave(IQ), IQ=IQ1,IQ2)
             WRITE (UNIT,'(''#'')')
             DO IFRAME = 0, nframes-1
                 WRITE (UNIT,"(1X,F9.3,9x,20(2x,e13.6))") tstep*( frame_nstep(IFRAME+1)-frame_nstep(1) ), &
                                                          (fqtima(I,J,IQ,IFRAME), IQ=IQ1,IQ2)
             END DO          
             CLOSE (UNIT)
         
          END DO
       END DO
    END DO
   
END PROGRAM FQT_COH
