!===============================================================================
! This file is part of MG_DLP_analysis - a set of programs to perform
! different analysis on Molecular Dynamics simulations, mainly intended
! to handle trajectories generated by DL_POLY (HISTORY format).
!
! Copyright (C) 2021 Miguel A. Gonzalez
!
!    MG_DLP_analysis is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    MG_DLP_analysis is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with MG_DLP_analysis.  If not, see <https://www.gnu.org/licenses/>.
!
!===============================================================================
!
! Module to perform general operations. 
!
! Subroutines:
!
!   INVERT_CELL (direct_cell, inverse_cell, volume_cell)
!   ROTATE_CELL (cell, rotation)
!   DIHEDRAL_ANGLE (ri, rj, rk, rl, dihedral)
!   HKL_VECTORS (cell, qmin, qmax, qdel, nmax, mxq, hkl, qmodule, qnum, qnlast)
!
! Main variables:
!
!       inverse_cell (3,3)      
!
!  Last change: 7/6/2011
!
!===============================================================================

MODULE GENERAL_UTILS

USE DEFINITIONS,            ONLY : DP
USE LINEAR_ALGEBRA,         ONLY : NORM, NORMALIZE, CROSS_PRODUCT
USE CHEMPHYSMATH_CONSTANTS, ONLY : PI

IMPLICIT NONE
PUBLIC

CONTAINS

   SUBROUTINE INVERT_CELL (direct_cell, inverse_cell, volume_cell)

   REAL ( KIND = DP ), INTENT(IN), DIMENSION(3,3) :: direct_cell
   REAL ( KIND = DP ), INTENT(OUT), DIMENSION(3,3) :: inverse_cell
   REAL ( KIND = DP ), INTENT(OUT) :: volume_cell

   inverse_cell(1,1) = direct_cell(2,2) * direct_cell(3,3) - direct_cell(3,2)*direct_cell(2,3)
   inverse_cell(1,2) = direct_cell(1,3) * direct_cell(3,2) - direct_cell(1,2)*direct_cell(3,3)
   inverse_cell(1,3) = direct_cell(1,2) * direct_cell(2,3) - direct_cell(2,2)*direct_cell(1,3)
   inverse_cell(2,1) = direct_cell(3,1) * direct_cell(2,3) - direct_cell(2,1)*direct_cell(3,3)
   inverse_cell(2,2) = direct_cell(1,1) * direct_cell(3,3) - direct_cell(1,3)*direct_cell(3,1)
   inverse_cell(2,3) = direct_cell(1,3) * direct_cell(2,1) - direct_cell(1,1)*direct_cell(2,3)
   inverse_cell(3,1) = direct_cell(2,1) * direct_cell(3,2) - direct_cell(2,2)*direct_cell(3,1)
   inverse_cell(3,2) = direct_cell(1,2) * direct_cell(3,1) - direct_cell(1,1)*direct_cell(3,2)
   inverse_cell(3,3) = direct_cell(1,1) * direct_cell(2,2) - direct_cell(1,2)*direct_cell(2,1)

   volume_cell = direct_cell(1,1) * inverse_cell(1,1) + &
                 direct_cell(2,1) * inverse_cell(1,2) + &
                 direct_cell(3,1) * inverse_cell(1,3) 
 
   inverse_cell(:,:) = inverse_cell(:,:) / volume_cell
 
   END SUBROUTINE INVERT_CELL


   SUBROUTINE ROTATE_CELL (cell, rotation)

   REAL ( KIND = DP ), INTENT(INOUT), DIMENSION(3,3) :: cell
   REAL ( KIND = DP ), INTENT(IN), DIMENSION(3,3) :: rotation

   INTEGER            :: I
   REAL ( KIND = DP ) :: X, Y, Z   
   
   DO I = 1, 3    ! loop over cell vectors
            
      X = cell(I,1) 
      Y = cell(I,2) 
      Z = cell(I,3) 
      cell(I,1) = ROTATION(1,1)*X + ROTATION(1,2)*Y + ROTATION(1,3)*Z
      cell(I,2) = ROTATION(2,1)*X + ROTATION(2,2)*Y + ROTATION(2,3)*Z
      cell(I,3) = ROTATION(3,1)*X + ROTATION(3,2)*Y + ROTATION(3,3)*Z
          
   END DO 
 
   END SUBROUTINE ROTATE_CELL
   

   SUBROUTINE DIHEDRAL_ANGLE (ri, rj, rk, rl, dihedral)

   REAL ( KIND = DP ), INTENT(IN), DIMENSION(3) :: ri, rj, rk, rl
   REAL ( KIND = DP ), INTENT(OUT) :: dihedral

   REAL ( KIND = DP ), DIMENSION(3) :: rij, rjk, rkl, rijjk, rjkkl, vsign
   REAL ( KIND = DP ) :: phicos, fsign, phi
   
   rij = rj - ri
   rjk = rk - rj
   rkl = rl - rk

   rijjk = CROSS_PRODUCT (rij, rjk)
   rijjk = NORMALIZE (rijjk)

   rjkkl = CROSS_PRODUCT (rjk, rkl)
   rjkkl = NORMALIZE (rjkkl)

   phicos = DOT_PRODUCT (rijjk, rjkkl)
   if (phicos > 1.0) phicos = 1.0
   if (phicos < -1.0) phicos = -1.0
   phi = ACOS (phicos)

   vsign = CROSS_PRODUCT (rijjk, rjkkl)
   fsign = DOT_PRODUCT (vsign, rjk)

   if (fsign /= 0.) then
      fsign = fsign/ABS(fsign)
   else
      fsign = 1.0
   end if

   dihedral = fsign * phi * 180.0 / PI


   END SUBROUTINE DIHEDRAL_ANGLE

   
   SUBROUTINE HKL_VECTORS (cell, qmin, qmax, qdel, nmax, mxq, hkl, qmodule, qnum, qnlast)
   
   !nmax = Maximum number of vectors  per shell (0 = unlimited)
   !mxq  = Total maximum number of vectors
   !qdel = Shell width (0 = if hkl vector is treated as a different vector)
   
   INTEGER, INTENT(IN) :: nmax, mxq                               
   REAL ( KIND = DP ), INTENT(IN), DIMENSION(3,3) :: cell
   REAL ( KIND = DP ), INTENT(IN) :: qmin, qmax, qdel

   REAL ( KIND = DP ), ALLOCATABLE, INTENT(OUT), DIMENSION(:) :: qmodule
   
   INTEGER, INTENT(OUT), ALLOCATABLE, DIMENSION(:,:) :: hkl
   INTEGER, INTENT(OUT), ALLOCATABLE, DIMENSION(:)   :: qnum

   REAL ( KIND = DP ), DIMENSION(3,3) :: rcell
   REAL ( KIND = DP ), DIMENSION(3)   :: qtmp
   
   REAL ( KIND = DP ) :: vcell, qini, qend, qtmpmod
   INTEGER :: nqval, qnlast, hmin, hmax, kmin, kmax, lmin, lmax, idx, I, J, K, itmp = 0
   
   INTEGER, ALLOCATABLE, DIMENSION(:) :: vec_counter 
    
   IF (ALLOCATED(hkl)) DEALLOCATE(hkl); ALLOCATE(hkl(mxq,3)); hkl = 0
   IF (ALLOCATED(qnum)) DEALLOCATE(qnum); ALLOCATE(qnum(mxq)); qnum = 0
   IF (ALLOCATED(qmodule)) DEALLOCATE(qmodule); ALLOCATE(qmodule(mxq)); qmodule = 0.0
   IF (ALLOCATED(vec_counter)) DEALLOCATE(vec_counter); ALLOCATE(vec_counter(1)); vec_counter = 0
   
   ! vectors: 
   ! (a,0,0), (0,a,0), (0,0,a)
   ! (a,b,0), (a,-b,0), (a,0,b), (a,0,-b), (0,a,b),(0,a,-b)
   ! (a,b,c), (-a,b,c), (a,-b,c), (a,b,-c)

   CALL INVERT_CELL (cell, rcell, vcell)
   
   ! Determine min & max h, k, l
   
   qini = qmin - 0.5*qdel
   qend = qmax + 0.5*qdel
   IF (qdel > 0.0)  THEN
      nqval = INT((qend-qini) / qdel + 1)
      IF (nmax > 0) THEN
         ! To limit number of vector per shell
         IF (ALLOCATED(vec_counter)) DEALLOCATE(vec_counter)
         ALLOCATE(vec_counter(nqval))
         vec_counter = 0
      END IF       
   END IF  
   
   qtmp(1) = 2.0 * pi * rcell(1,1)
   qtmp(2) = 2.0 * pi * rcell(1,2)
   qtmp(3) = 2.0 * pi * rcell(1,3)
   qtmpmod = NORM(qtmp)
   IF (qtmpmod >= qini) THEN 
      hmin = 1
   ELSE 
      hmin = CEILING(qini/qtmpmod) 
   END IF   
   hmax = FLOOR(qend/qtmpmod) 
   
   qtmp(1) = 2.0 * pi * rcell(2,1)
   qtmp(2) = 2.0 * pi * rcell(2,2)
   qtmp(3) = 2.0 * pi * rcell(2,3)
   qtmpmod = NORM(qtmp)
   IF (qtmpmod >= qini) THEN 
      kmin = 1
   ELSE 
      kmin = CEILING(qini/qtmpmod) 
   END IF   
   kmax = FLOOR(qend/qtmpmod) 
   
   qtmp(1) = 2.0 * pi * rcell(3,1)
   qtmp(2) = 2.0 * pi * rcell(3,2)
   qtmp(3) = 2.0 * pi * rcell(3,3)
   qtmpmod = NORM(qtmp)
   IF (qtmpmod >= qini) THEN 
      lmin = 1
   ELSE 
      lmin = CEILING(qini/qtmpmod) 
   END IF   
   lmax = FLOOR(qend/qtmpmod) 
   
   idx = 0
   
   ! (a,0,0), (0,a,0), (0,0,a) vectors
   DO I = hmin, hmax 
      qtmp(1) = I * 2.0 * pi * rcell(1,1)
      qtmp(2) = I * 2.0 * pi * rcell(1,2)
      qtmp(3) = I * 2.0 * pi * rcell(1,3)
      qtmpmod = NORM(qtmp)
      IF ((qtmpmod < qini) .or. (qtmpmod > qend)) CYCLE
      idx = idx + 1
      IF (idx > mxq) EXIT
      IF (qdel > 0.0) THEN
         itmp = NINT((qtmpmod - qini)/qdel + 0.5)
         IF (nmax > 0) THEN
            vec_counter(itmp) = vec_counter(itmp) + 1
            IF (vec_counter(itmp) > nmax) THEN
               idx = idx -1 
               CYCLE
            END IF 
         END IF
      END IF
      hkl(idx,1) = I
      hkl(idx,2) = 0
      hkl(idx,3) = 0
      qmodule(idx) = qtmpmod
      IF (qdel > 0.0) THEN
         qnum(idx) = itmp
      ELSE       
         qnum(idx)  = idx 
      END IF
   END DO
   
   DO J = kmin, kmax 
      qtmp(1) = J * 2.0 * pi * rcell(2,1)
      qtmp(2) = J * 2.0 * pi * rcell(2,2)
      qtmp(3) = J * 2.0 * pi * rcell(2,3)
      qtmpmod = NORM(qtmp)
      IF ((qtmpmod < qini) .or. (qtmpmod > qend)) CYCLE
      idx = idx + 1
      IF (idx > mxq) EXIT
      IF (qdel > 0.0) THEN
         itmp = NINT((qtmpmod - qini)/qdel + 0.5)
         IF (nmax > 0) THEN
            vec_counter(itmp) = vec_counter(itmp) + 1
            IF (vec_counter(itmp) > nmax) THEN
               idx = idx -1 
               CYCLE
            END IF 
         END IF
      END IF
      hkl(idx,1) = 0
      hkl(idx,2) = J
      hkl(idx,3) = 0
      qmodule(idx) = qtmpmod
      IF (qdel > 0.0) THEN
         qnum(idx) = itmp
      ELSE       
         qnum(idx)  = idx 
      END IF
   END DO
    
   DO K = lmin, lmax 
      qtmp(1) = K * 2.0 * pi * rcell(3,1)
      qtmp(2) = K * 2.0 * pi * rcell(3,2)
      qtmp(3) = K * 2.0 * pi * rcell(3,3)
      qtmpmod = NORM(qtmp)
      IF ((qtmpmod < qini) .or. (qtmpmod > qend)) CYCLE
      idx = idx + 1
      IF (idx > mxq) EXIT
      IF (qdel > 0.0) THEN
         itmp = NINT((qtmpmod - qini)/qdel + 0.5)
         IF (nmax > 0) THEN
            vec_counter(itmp) = vec_counter(itmp) + 1
            IF (vec_counter(itmp) > nmax) THEN
               idx = idx -1 
               CYCLE
            END IF 
         END IF
      END IF
      hkl(idx,1) = 0
      hkl(idx,2) = 0
      hkl(idx,3) = K
      qmodule(idx) = qtmpmod
      IF (qdel > 0.0) THEN
         qnum(idx) = itmp
      ELSE       
         qnum(idx)  = idx 
      END IF
   END DO
  
   ! (a,b,0)+(a,-b,0), (a,0,b)+(a,0,-b), (0,a,b)+(0,a,-b) vectors
      
   DO I = 1, hmax 
      DO J = 1, kmax
         qtmp(1) = I * 2.0 * pi * rcell(1,1) + J * 2.0 * pi * rcell(2,1)
         qtmp(2) = I * 2.0 * pi * rcell(1,2) + J * 2.0 * pi * rcell(2,2)
         qtmp(3) = I * 2.0 * pi * rcell(1,3) + J * 2.0 * pi * rcell(2,3)
         qtmpmod = NORM(qtmp)
         IF ((qtmpmod < qini) .or. (qtmpmod > qend)) CYCLE
         idx = idx + 1
         IF (idx > mxq) EXIT
         IF (qdel > 0.0) THEN
            itmp = NINT((qtmpmod - qini)/qdel + 0.5)
            IF (nmax > 0) THEN
               vec_counter(itmp) = vec_counter(itmp) + 1
               IF (vec_counter(itmp) > nmax) THEN
                  idx = idx -1 
                  CYCLE
               END IF 
            END IF
         END IF
         hkl(idx,1) = I
         hkl(idx,2) = J
         hkl(idx,3) = 0
         qmodule(idx) = qtmpmod
         IF (qdel > 0.0) THEN
            qnum(idx) = itmp
         ELSE       
            qnum(idx)  = idx 
         END IF
         idx = idx + 1
         IF (idx > mxq) EXIT
         IF (qdel > 0.0) THEN
            itmp = NINT((qtmpmod - qini)/qdel + 0.5)
            IF (nmax > 0) THEN
               vec_counter(itmp) = vec_counter(itmp) + 1
               IF (vec_counter(itmp) > nmax) THEN
                  idx = idx -1 
                  CYCLE
               END IF 
            END IF
         END IF
         hkl(idx,1) = I
         hkl(idx,2) = -J
         hkl(idx,3) = 0
         qmodule(idx) = qtmpmod
         IF (qdel > 0.0) THEN
            qnum(idx) = itmp
         ELSE       
            qnum(idx)  = idx 
         END IF
      END DO
   END DO
      
   DO I = 1, hmax 
      DO K = 1, lmax
         qtmp(1) = I * 2.0 * pi * rcell(1,1) + K * 2.0 * pi * rcell(3,1)
         qtmp(2) = I * 2.0 * pi * rcell(1,2) + K * 2.0 * pi * rcell(3,2)
         qtmp(3) = I * 2.0 * pi * rcell(1,3) + K * 2.0 * pi * rcell(3,3)
         qtmpmod = NORM(qtmp)
         IF ((qtmpmod < qini) .or. (qtmpmod > qend)) CYCLE
         idx = idx + 1
         IF (idx > mxq) EXIT
         IF (qdel > 0.0) THEN
            itmp = NINT((qtmpmod - qini)/qdel + 0.5)
            IF (nmax > 0) THEN
               vec_counter(itmp) = vec_counter(itmp) + 1
               IF (vec_counter(itmp) > nmax) THEN
                  idx = idx -1 
                  CYCLE
               END IF 
            END IF
         END IF
         hkl(idx,1) = I
         hkl(idx,2) = 0
         hkl(idx,3) = K
         qmodule(idx) = qtmpmod
         IF (qdel > 0.0) THEN
            qnum(idx) = itmp
         ELSE       
            qnum(idx)  = idx 
         END IF
         idx = idx + 1
         IF (idx > mxq) EXIT
         IF (qdel > 0.0) THEN
            itmp = NINT((qtmpmod - qini)/qdel + 0.5)
            IF (nmax > 0) THEN
               vec_counter(itmp) = vec_counter(itmp) + 1
               IF (vec_counter(itmp) > nmax) THEN
                  idx = idx -1 
                  CYCLE
               END IF 
            END IF
         END IF
         hkl(idx,1) = I
         hkl(idx,2) = 0
         hkl(idx,3) = -K
         qmodule(idx) = qtmpmod
         IF (qdel > 0.0) THEN
            qnum(idx) = itmp
         ELSE       
            qnum(idx)  = idx 
         END IF
      END DO
   END DO
     
   DO J = 1, kmax 
      DO K = 1, lmax
         qtmp(1) = J * 2.0 * pi * rcell(2,1) + K * 2.0 * pi * rcell(3,1)
         qtmp(2) = J * 2.0 * pi * rcell(2,2) + K * 2.0 * pi * rcell(3,2)
         qtmp(3) = J * 2.0 * pi * rcell(2,3) + K * 2.0 * pi * rcell(3,3)
         qtmpmod = NORM(qtmp)
         IF ((qtmpmod < qini) .or. (qtmpmod > qend)) CYCLE
         idx = idx + 1
         IF (idx > mxq) EXIT
         IF (qdel > 0.0) THEN
            itmp = NINT((qtmpmod - qini)/qdel + 0.5)
            IF (nmax > 0) THEN
               vec_counter(itmp) = vec_counter(itmp) + 1
               IF (vec_counter(itmp) > nmax) THEN
                  idx = idx -1 
                  CYCLE
               END IF 
            END IF
         END IF
         hkl(idx,1) = 0
         hkl(idx,2) = J
         hkl(idx,3) = K
         qmodule(idx) = qtmpmod
         IF (qdel > 0.0) THEN
            qnum(idx) = itmp
         ELSE       
            qnum(idx)  = idx 
         END IF
         idx = idx + 1
         IF (idx > mxq) EXIT
         IF (qdel > 0.0) THEN
            itmp = NINT((qtmpmod - qini)/qdel + 0.5)
            IF (nmax > 0) THEN
               vec_counter(itmp) = vec_counter(itmp) + 1
               IF (vec_counter(itmp) > nmax) THEN
                  idx = idx -1 
                  CYCLE
               END IF 
            END IF
         END IF
         hkl(idx,1) = 0
         hkl(idx,2) = J
         hkl(idx,3) = -K
         qmodule(idx) = qtmpmod
         IF (qdel > 0.0) THEN
            qnum(idx) = itmp
         ELSE       
            qnum(idx)  = idx 
         END IF
      END DO
   END DO
   
   ! (a,b,c)+(-a,b,c)+(a,-b,c)+(a,b,-c) vectors
     
   DO I = 1, hmax
      DO J = 1, kmax 
         DO K = 1, lmax
            qtmp(1) = I * 2.0 * pi * rcell(1,1) + J * 2.0 * pi * rcell(2,1) + K * 2.0 * pi * rcell(3,1)
            qtmp(2) = I * 2.0 * pi * rcell(1,2) + J * 2.0 * pi * rcell(2,2) + K * 2.0 * pi * rcell(3,2)
            qtmp(3) = I * 2.0 * pi * rcell(1,3) + J * 2.0 * pi * rcell(2,3) + K * 2.0 * pi * rcell(3,3)
            qtmpmod = NORM(qtmp)
            IF ((qtmpmod < qini) .or. (qtmpmod > qend)) CYCLE
            idx = idx + 1
            IF (idx > mxq) EXIT
            IF (qdel > 0.0) THEN
               itmp = NINT((qtmpmod - qini)/qdel + 0.5)
               IF (nmax > 0) THEN
                  vec_counter(itmp) = vec_counter(itmp) + 1
                  IF (vec_counter(itmp) > nmax) THEN
                     idx = idx -1 
                     CYCLE
                  END IF 
               END IF
            END IF
            hkl(idx,1) = I
            hkl(idx,2) = J
            hkl(idx,3) = K
            qmodule(idx) = qtmpmod
            IF (qdel > 0.0) THEN
               qnum(idx) = itmp
            ELSE       
               qnum(idx)  = idx 
            END IF
            idx = idx + 1
            IF (idx > mxq) EXIT
            IF (qdel > 0.0) THEN
               itmp = NINT((qtmpmod - qini)/qdel + 0.5)
               IF (nmax > 0) THEN
                  vec_counter(itmp) = vec_counter(itmp) + 1
                  IF (vec_counter(itmp) > nmax) THEN
                     idx = idx -1 
                     CYCLE
                  END IF 
               END IF
            END IF
            hkl(idx,1) = -I
            hkl(idx,2) = J
            hkl(idx,3) = K
            qmodule(idx) = qtmpmod
            IF (qdel > 0.0) THEN
               qnum(idx) = itmp
            ELSE       
               qnum(idx)  = idx 
            END IF
            idx = idx + 1
            IF (idx > mxq) EXIT
            IF (qdel > 0.0) THEN
               itmp = NINT((qtmpmod - qini)/qdel + 0.5)
               IF (nmax > 0) THEN
                  vec_counter(itmp) = vec_counter(itmp) + 1
                  IF (vec_counter(itmp) > nmax) THEN
                     idx = idx -1 
                     CYCLE
                  END IF 
               END IF
            END IF
            hkl(idx,1) = I
            hkl(idx,2) = -J
            hkl(idx,3) = K
            qmodule(idx) = qtmpmod
            IF (qdel > 0.0) THEN
               qnum(idx) = itmp
            ELSE       
               qnum(idx)  = idx 
            END IF
            idx = idx + 1
            IF (idx > mxq) EXIT
            IF (qdel > 0.0) THEN
               itmp = NINT((qtmpmod - qini)/qdel + 0.5)
               IF (nmax > 0) THEN
                  vec_counter(itmp) = vec_counter(itmp) + 1
                  IF (vec_counter(itmp) > nmax) THEN
                     idx = idx -1 
                     CYCLE
                  END IF 
               END IF
            END IF
            hkl(idx,1) = I
            hkl(idx,2) = J
            hkl(idx,3) = -K
            qmodule(idx) = qtmpmod
            IF (qdel > 0.0) THEN
               qnum(idx) = itmp
            ELSE       
               qnum(idx)  = idx 
            END IF
         END DO
      END DO
   END DO

   IF (idx < mxq) THEN
      qnlast = idx
   ELSE
      qnlast = mxq
      PRINT *, 'Maximum number of vectors in subroutine HKL_VECTORS reached!'
      PRINT *, 'Last vector is no. ', qnlast, ' --> HKL = ', hkl(qnlast,:), ' Module = ', qmodule(qnlast)
      PRINT *, 'If needed, increase the parameter mxq in general_utils.f90!'
   END IF
      
   END SUBROUTINE HKL_VECTORS
      
END MODULE GENERAL_UTILS   

   
