!
! Program to calculate total radial distribution function and their partial contributions.
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Writes:
!	1. A series of ASCII files containing r, g(r)_intra, g(r)_inter, g(r)_total 
!          and for the partial r.d.f. a fith column with the total g(r) weighted by the
!          corresponding contribution of this partial to the total one.
!
!  Last change: 25/12/2009
!

PROGRAM RADIAL_DISTRIBUTION_FUNCTION	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
   USE GENERAL_VARIABLES
   USE FORCEFIELD_VARIABLES,   ONLY : atomname
   USE TRAJECTORY_VARIABLES,   ONLY : frame_cell, average_cell_volume
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ_SUBTRAJ
   USE TRAJECTORY_UTILS,       ONLY : TRAJECT_CART_2_FRACT
   
   IMPLICIT NONE

   INTEGER, PARAMETER :: max_number_types = 99

   LOGICAL                                            :: new_atom
   CHARACTER ( LEN = LINE_LENGTH)                     :: traject_file, ff_file, output_file
   CHARACTER ( LEN = 3 ), DIMENSION(max_number_types) :: type_name
   CHARACTER ( LEN = 4 )                              :: partial_name
   INTEGER, DIMENSION(max_number_types)               :: number_atoms_type
   REAL ( KIND = DP ), DIMENSION(max_number_types)    :: b_type
   REAL ( KIND = DP ), DIMENSION(3)                   :: sdist

   INTEGER :: IFRAME, I, J, K, L, I1, J1, K1, I2, J2, K2, II, JJ, IOSTAT, UNIT, IBLOCK, nframes, nblocks, nsingle, &
              number_types, total_number_atoms, number_partials, index, number_points, bin, progress, ifirst, ilast, &
	      total_number_frames

   REAL ( KIND = DP ) :: rmax, dr, weight_norm, rx, ry, rz, r, xa, xb, xx, shell_volume, n_ideal, n_intra, n_inter, n_total

   INTEGER, ALLOCATABLE, DIMENSION(:,:,:)                :: hist_intra, hist_inter
   INTEGER, ALLOCATABLE, DIMENSION(:,:)                  :: type_atom
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr, sss
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)         :: weight_partial, gr_intra, gr_inter, gr_total


   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) total_number_frames
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,'("Maximum distance to compute the r.d.f.?")')
   READ (INPUT,*) rmax
   WRITE (OUTPUT,'("Delta r value to use in r.d.f. grid?")')
   READ (INPUT,*) dr
   WRITE (OUTPUT,*)
   

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 
      

   ! . Determine number of different types of atoms
   
   IF ( ALLOCATED(type_atom) ) DEALLOCATE(type_atom)
   ALLOCATE ( type_atom ( maxval(numatoms), numspecies ) )
   
   number_types = 0
   total_number_atoms = 0
   number_atoms_type(:) = 0
   
   IF ( ALLOCATED(atomname) ) THEN
      IF ( SIZE(atomname) >= 1 ) THEN
   
         species_loop: DO I = 1,numspecies
               atoms_loop:   DO K = 1,numatoms(I) 
	 
	           ! . Check if atom type is new
	           new_atom = .TRUE.
	           DO L = 1,number_types
		      IF (atomname(K,I)(1:3) == type_name(L) ) THEN
		         new_atom = .FALSE.
			 number_atoms_type(L) = number_atoms_type(L) + nummols(I)
			 total_number_atoms = total_number_atoms + nummols(I)
			 type_atom(K,I) = L
		      END IF	 
		   END DO
	           IF (new_atom) THEN
		      number_types = number_types + 1
		      IF ( number_types > max_number_types ) STOP 'Too many different types of atoms!'
	              number_atoms_type(number_types) = nummols(I)
	              total_number_atoms = total_number_atoms + nummols(I)
		      type_name(number_types) = atomname(K,I)(1:3)
		      type_atom(K,I) = number_types
		   END IF
		   
               END DO atoms_loop
         END DO species_loop
      
      ENDIF
   END IF
 
 
   ! If no atom information is available, consider each atom as a different one
   
   IF ( number_types == 0 ) THEN
   
         species_loop_2: DO i = 1,numspecies
               atoms_loop_2:   DO k = 1,numatoms(i) 
	 
		      number_types = number_types + 1
		      IF ( number_types > max_number_types ) STOP 'Too many different types of atoms!'
	              type_atom(K,I) = number_types
		      number_atoms_type(number_types) = nummols(I)
	              total_number_atoms = total_number_atoms + nummols(I)
		      WRITE(type_name(number_types),'(i3.3)') number_types
		   
               END DO atoms_loop_2
         END DO species_loop_2
	 
   END IF
   
   number_partials = number_types * (1+number_types) / 2
   ALLOCATE ( weight_partial(number_partials) )

   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("Total number of atoms = ",I8)') total_number_atoms
   WRITE (OUTPUT,'("Number of different types of atom = ",I3)') number_types
   WRITE (OUTPUT,'("Number of partial r.d.f.s = ",I3)') number_partials
   WRITE (OUTPUT,*)

   DO I = 1, number_types
      IF ( type_name(I)(3:3) == ' ' ) type_name(I)(3:3) = '_'
      IF ( type_name(I)(2:2) == ' ' ) type_name(I)(2:2) = '_'
      WRITE (OUTPUT, "(A,I2,A,A3,A,I8)" ) '     Type:', I, ' - Name:', type_name(I), ' - Number of atoms = ',number_atoms_type(I)
      WRITE (OUTPUT, "(A)" ) '     Give scattering length for this type:'
      READ (INPUT,*) b_type(I)
      WRITE (OUTPUT,*)
   END DO
   
   
   ! Calculate weighting factors for each partial g(r) and normalization factor
   
   index = 0
   weight_norm = 0.0
   DO I = 1,number_types
      DO J = I,number_types
         index = index + 1
	 IF (I == J) THEN
	    weight_partial(index) = b_type(I)*b_type(J)*REAL(number_atoms_type(I)*(number_atoms_type(J)-1))/ &
	                            REAL(total_number_atoms**2)
	    weight_norm = weight_norm + weight_partial(index)
	 ELSE
	    weight_partial(index) = 2*b_type(I)*b_type(J)*REAL(number_atoms_type(I)*number_atoms_type(J))/ &
	                            REAL(total_number_atoms**2)
	    weight_norm = weight_norm + weight_partial(index)
	 END IF
     END DO
   END DO
   
   
   ! Normalize weighting factors for each partial g(r) and write contribution of each partial to total
   
   index = 0
   DO I = 1,number_types
      DO J = I,number_types
         index = index + 1
	 weight_partial(index) = weight_partial(index) / weight_norm
	 WRITE (OUTPUT,"(A,I4,A,A3,A,A3,A,A,G12.6)") ' Partial ', index, ' : ',type_name(i),'-',type_name(j),' . ', &
	                                           & 'Contribution to total r.d.f. = ',weight_partial(index)
      END DO
   END DO
   WRITE (OUTPUT,*)
  
   
   ! Initialize histograms
   
   number_points = INT(rmax/dr) 
   IF ( ALLOCATED(hist_intra) ) DEALLOCATE(hist_intra)
   IF ( ALLOCATED(hist_inter) ) DEALLOCATE(hist_inter)
   IF ( ALLOCATED(gr_intra) )   DEALLOCATE(gr_intra)
   IF ( ALLOCATED(gr_inter) )   DEALLOCATE(gr_inter)
   IF ( ALLOCATED(gr_total) )   DEALLOCATE(gr_total)
   ALLOCATE ( hist_intra(number_types,number_types,number_points), hist_inter(number_types,number_types,number_points) )
   ALLOCATE ( gr_intra(number_points), gr_inter(number_points), gr_total(number_points) )
   hist_intra = 0; hist_inter = 0; gr_intra = 0.0; gr_inter = 0.0; gr_total = 0.0
   
   
   ! Set here number of frames to read at a single time
   nsingle = 10
   PRINT *, 'Reading ', nsingle, 'frames at a time! This is OK for 120000 atoms.'
   PRINT *, 'Modify nsingle if needed!'
   
   nblocks = total_number_frames / nsingle
   IF (MOD(total_number_frames, nsingle) > 0) nblocks = nblocks + 1
   PRINT *, 'Starting loop over ', nblocks, 'blocks'
   
   ! Blocks loop to read frames block by block
   
   blocks_loop: DO IBLOCK = 1, nblocks

      ! Read trajectory and transform to fractional coordinates
      ! Needed in order to determine minimum image distance for a non-orthorombic system
      
      ifirst = (IBLOCK-1) * nsingle + 1
      ilast = MIN( IBLOCK * nsingle, total_number_frames)
      PRINT *
      PRINT *,'Reading trajectory: Block, 1st frame, last frame: ', IBLOCK, ifirst, ilast
      CALL HISTORY_DLPOLY_READ_SUBTRAJ (traject_file, ifirst, ilast, .FALSE., nframes, rrr)
      PRINT *, 'Read ', nframes, 'frames.'
      CALL TRAJECT_CART_2_FRACT (nframes, rrr, sss)
   
   ! Compute histograms
   
   frames_loop: DO IFRAME = 1,nframes

      progress = nframes / 20
      IF (progress < 1) progress = 1
      IF (MOD(IFRAME,progress) == 0) PRINT *,'RDF calculation. Completed ',(100*IFRAME/nframes),' %'

      atom1_species_loop: DO I1 = 1,numspecies
         atom1_molecules_loop: DO J1 = 1,nummols(I1)
            atom1_atoms_loop:   DO K1 = 1,numatoms(I1) 
               atom2_species_loop: DO I2 = 1,numspecies
                  atom2_molecules_loop: DO J2 = 1,nummols(I2)
                     atom2_atoms_loop:   DO K2 = 1,numatoms(I2) 
		        
		        IF ( (I1 == I2) .AND. (J1 == J2) .AND. (K1 == K2) ) CYCLE atom2_atoms_loop
			
			! Obtain distance between atoms
			
			sdist = sss(1:3, K1, J1, I1, IFRAME) - sss(1:3, K2, J2, I2, IFRAME)
			sdist = sdist - ANINT(sdist)
                        rx = sdist(1)*frame_cell(1,1,IFRAME) + sdist(2)*frame_cell(2,1,IFRAME) + sdist(3)*frame_cell(3,1,IFRAME)
                        ry = sdist(1)*frame_cell(1,2,IFRAME) + sdist(2)*frame_cell(2,2,IFRAME) + sdist(3)*frame_cell(3,2,IFRAME)
                        rz = sdist(1)*frame_cell(1,3,IFRAME) + sdist(2)*frame_cell(2,3,IFRAME) + sdist(3)*frame_cell(3,3,IFRAME)
                        r = SQRT( rx*rx + ry*ry + rz*rz )
                    
		        ! Add to histogram
		     
		        IF (r.le.rmax) THEN
                           bin = INT(r/dr) + 1
			   IF (type_atom(K1,I1) <= type_atom(K2,I2) ) THEN
			      II = type_atom(K1,I1)
			      JJ = type_atom(K2,I2)
			   ELSE
			      II = type_atom(K2,I2)
			      JJ = type_atom(K1,I1)
			   END IF
			   IF ( (I1 == I2) .AND. (J1 == J2) ) THEN
                              hist_intra(II,JJ,bin) = hist_intra(II,JJ,bin) + 1
			   ELSE
                              hist_inter(II,JJ,bin) = hist_inter(II,JJ,bin) + 1
			   END IF   
                        ENDIF
			
			
		     END DO atom2_atoms_loop	
		  END DO atom2_molecules_loop	
	       END DO atom2_species_loop	
	    END DO atom1_atoms_loop	
	 END DO atom1_molecules_loop	
      END DO atom1_species_loop	

   END DO frames_loop

   END DO blocks_loop
   
   ! Write partial r.d.f.'s and compute total g(r)
   
    index = 0
    
    atomtype1_loop: DO I = 1,number_types
       atomtype2_loop: DO J = i,number_types

          index = index + 1
       
          ! Open file to write partial r.d.f. 
          l = 100*I + J
	  WRITE (partial_name,"(I4.4)") l
          UNIT = NEXT_UNIT ( )	
          OPEN ( UNIT, FILE = 'gr.'//partial_name, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
          IF ( IOSTAT /= 0 ) STOP 'Output file for partial r.d.f. could not be created!'
              
	  WRITE (UNIT,'(''# g(r) '',a3,''-'',a3)') type_name(I),type_name(J)
          WRITE (UNIT,'(''#'',3x,''r/A'',4x,''g(r) intra'',2x,''g(r) inter'',2x,''g(r) total'',2x,''weight*g(r)'')')
    
          bin_loop: DO K = 1,number_points
	  
               xa = REAL(K-1)*dr
               xb = xa + dr
               xx = 0.5 * (xa + xb)
               shell_volume = 4.0 * pi * (xb**3 - xa**3) / 3.0
	       if (I == J) then
                  n_ideal = REAL( number_atoms_type(I) * (number_atoms_type(J)-1.0) ) / average_cell_volume
	       else
                  n_ideal = 2.0 * REAL( number_atoms_type(I) * number_atoms_type(J) ) / average_cell_volume
	       endif	
               n_intra = REAL(hist_intra(I,J,K))/ REAL(total_number_frames) / shell_volume / n_ideal
               n_inter = REAL(hist_inter(I,J,K))/ REAL(total_number_frames) / shell_volume / n_ideal
               n_total = REAL(hist_intra(I,J,K)+hist_inter(I,J,K))/ REAL(total_number_frames) / shell_volume / n_ideal
               
	       WRITE (UNIT,"(1X,F7.3,4(2X,F10.6))") xx,n_intra,n_inter,n_total,weight_partial(index)*n_total
	  
	       gr_intra(K) = gr_intra(K) + weight_partial(index)*n_intra
	       gr_inter(K) = gr_inter(K) + weight_partial(index)*n_inter
	       gr_total(K) = gr_total(K) + weight_partial(index)*n_total
	  
	  END DO bin_loop
       
          CLOSE (UNIT)
       
       END DO atomtype2_loop	
    END DO atomtype1_loop	

    
    ! Write total g(r)

    UNIT = NEXT_UNIT ( )	
    OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
    WRITE (UNIT,'(''# g(r) total'')')
    WRITE (UNIT,'(''#'',3x,''r/A'',4x,''g(r) intra'',2x,''g(r) inter'',2x,''g(r) total'')')
    
    DO K = 1,number_points
         xx = real(K-1)*dr + 0.5*dr
         WRITE (UNIT,"(1x,f7.3,3(2x,f10.6))") xx,gr_intra(K),gr_inter(K),gr_total(K)
    END DO
    CLOSE (UNIT)
   
END PROGRAM RADIAL_DISTRIBUTION_FUNCTION
