!
! Program to calculate HBs statistics.
!
! The user has to declare which are the donor (R-XH) species and the corresponding indexes for
! the X and H atoms in a donor molecule, and which are the acceptor species (R-Z) and the index
! for the Z atoms in the molecule. 
!
! An HB is defined by the geometric criterion 
! (definition 2 in Prada-Garcia et al., J. Chem. Phys. vol. 139, 084501 (2013)),
! i.e. distance X...Z < rmax_XZ and angle H-X...Z < theta_max.
! Typical values for water are rmax_XZ = 3.5 AA and thetha_max = 30 deg (see e.g. Prada-Garcia (2013)
! or A. Luzar, J. Chem. Phys. vol 113, 10663-10675 (2000)).
! Following Luzar (2000), it is possible to use an additional condition such that the
! distance H...Z < rmax_HZ (for water, Luzar (2000) proposes rmax_HZ = 2.45 AA)
!
! The maximum number of HBs (sum of HBs acting as a donor and as an acceptor) that a single 
! molecule can form is determined by the parameter max_hb_number.
!
! Information about HBs found is stored in 4 arrays:
!    hb_number(species index, molecule index) = number of HBs (donor + acceptor) formed by each molecule
!    hb_list(frame index, donor index, molecule index, 4) = (acceptor index, molecule index)
!    hb_histo(species index, max_hb_number+1) = histogram of number of molecules having 0, 1, ..., max_hb_number HBs
!    hb_species_histo(donor index, acceptor index) = histogram containing total number of HBs between two species
!
! The hb_list array will contain the information about the pairs of H-bonded molecules, stored as:
!    hb_list(frame index, donor index, molecule index, 1) = acceptor index
!    hb_list(frame index, donor index, molecule index, 2) = molecule index
!
! For each possible donor the program looks among the possible acceptors if there is an HB satisfying
! the three conditions. Note that a donor can share its H with 2 different acceptors, but no with more.
! If for a given donor the program finds that there are 3 acceptors that fulfill all the conditions, it
! will stop.
!
! If a donor H is found to be shared by 2 molecules, the information about the 2nd acceptor is in
!    hb_list(frame index, donor index, molecule index, 3) = acceptor index
!    hb_list(frame index, donor index, molecule index, 4) = molecule index
!
! With this, it is possible to find a pair of H-bonded molecules using a loop (I) over all possible donors:
!    
!    donor:    species = nmoldonor(I) 
!              molecule = loop (J) over nummols(nmoldonor(I))
!
!    acceptor: species = nmoldonor(hb_list(IFRAME, I, J, 1)) (nmoldonor(hb_list(IFRAME, I, J, 3)) for the 2nd acceptor) 
!              molecule = hb_list(IFRAME, I, J, 2) (hb_list(IFRAME, I, J, 4) for the 2nd acceptor)
!
! After all HBs in a given frame have been found, the hb_histo array is updated, so that for each species
! in the system (including those which are neither donor nor acceptors) hb_histo will store the number of
! molecules not having any HB (hb_histo(species index, 1)), having 1 HB (hb_histo(species index, 2)), etc.
! up to max_hb_number (hb_histo(species index, max_hb_number+1))
!
! The following has not been tested, but if a molecule has more than one donor group (e.g. glycerol), 
! it is possible to define each of them as a different donor (this is why the donor index is not the 
! same as the species index). The same applies to the possible acceptors in a single molecule.
! However note that the code will not check for intra-molecular HBs. 
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!       2. Output file name
!       3. User description of donor/acceptor species and HB definition
!
!	Writes:
!	1. Output file with average number of HBs per molecular species, and the fraction of molecules having
!          N HBs + details about the number of HBs formed between each possible donor/acceptor pair.
!
!  Last change: 24/02/2021
!

PROGRAM HB	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES,   ONLY : frame_cell
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,       ONLY : TRAJECT_CART_2_FRACT
   
   IMPLICIT NONE
   
   INTEGER, PARAMETER :: max_hb_number = 9

   CHARACTER ( LEN = LINE_LENGTH)   :: traject_file, ff_file, output_file
   REAL ( KIND = DP ), DIMENSION(3) :: sdist, s1, s2, s3, s12, s13
   LOGICAL                          :: LDONOR, LACCEPTOR

   INTEGER :: UNIT, IOSTAT, IFRAME, I, J, K, L, I1, J1, I2, J2, IMOL1, IMOL2, nframes, progress, ndonor, naccept
   INTEGER :: ndistXZ_cond, nfull_cond, ndistHZ_cond, nshared, cindex
   
   REAL ( KIND = DP ) :: rmax_XZ, theta_max, rmax_HZ, rx, ry, rz, r, tmp, d12, d13, costheta, theta
   REAL ( KIND = DP ) :: hb_fraction, hb_average
 
   INTEGER, ALLOCATABLE, DIMENSION(:)       :: nmoldonor, nxdonor, nhdonor, nmolaccept, nxaccept
   INTEGER, ALLOCATABLE, DIMENSION(:,:)     :: hb_number, hb_histo, hb_species_histo
   INTEGER, ALLOCATABLE, DIMENSION(:,:,:,:) :: hb_list

   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr, sss

   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') output_file
   
   ! Define H-bond:  R-Xdonor-Hdonor ... Zaccept (often X = Z = Oxygen)
   ! (distance Xdonor-Zaccept < rmax_XZ, angle X-H...Z < theta_max)
   WRITE (OUTPUT,'("Number of donors?")')
   READ (INPUT,*) ndonor
   ALLOCATE ( nmoldonor(ndonor), nxdonor(ndonor), nhdonor(ndonor) )
   DO I = 1,ndonor
      WRITE (OUTPUT,'("Donor ", I3)') I
      WRITE (OUTPUT,' ("Molecule index, X (donor) atom index, donor H index ?")')
      READ (INPUT,*) nmoldonor(I), nxdonor(I), nhdonor(I)
   END DO
   WRITE (OUTPUT,'("Number of acceptors?")')
   READ (INPUT,*) naccept
   ALLOCATE ( nmolaccept(naccept), nxaccept(naccept) )
   DO I = 1,naccept
      WRITE (OUTPUT,'("Acceptor ", I3)') I
      WRITE (OUTPUT,' ("Molecule index, Z (acceptor) atom index?")')
      READ (INPUT,*) nmolaccept(I), nxaccept(I)
   END DO
   WRITE (OUTPUT,'("Maximum X...Z distance to define HB?")')
   READ (INPUT,*) rmax_XZ
   WRITE (OUTPUT,'("Maximum X-H...Z angle to define HB?")')
   READ (INPUT,*) theta_max
   WRITE (OUTPUT,'("Maximum H...Z distance to define HB?")')
   READ (INPUT,*) rmax_HZ
   WRITE (OUTPUT,*)
   

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
         WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
         READ (INPUT,*) nummols(I) 
         WRITE (OUTPUT,'("Number of atoms in this molecule?")')
         READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 
      
 
   ! Read trajectory and transform to fractional coordinates
   ! Needed in order to determine minimum image distance for a non-orthorombic system
      
   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   PRINT *,'Finished reading trajectory!'
   CALL TRAJECT_CART_2_FRACT (nframes, rrr, sss)
   
   
   ! Initialize counting variables and arrays
   ALLOCATE ( hb_list(nframes, ndonor, maxval(nummols), 4) ) !list of acceptors for each HB
   ALLOCATE ( hb_number(numspecies, maxval(nummols)) )     
   ALLOCATE ( hb_histo(numspecies, max_hb_number+1) )
   ALLOCATE ( hb_species_histo(ndonor, naccept) )
   hb_list      = 0
   hb_number    = 0
   hb_histo     = 0 
   hb_species_histo = 0  
   ndistXZ_cond = 0  !number of pairs fulfilling X...Z distance criterion
   nfull_cond   = 0  !number of pairs fulfilling distance + angle criteria
   ndistHZ_cond = 0  !number of pairs fulfilling the 3 criteria
   nshared      = 0  !number of X-Hs shared by two acceptors

   ! Compute lists of HBs
   
   frames_loop: DO IFRAME = 1,nframes
   
      hb_number = 0

      progress = nframes / 20
      IF (progress < 1) progress = 1
      IF (MOD(IFRAME,progress) == 0) PRINT *,'HB list calculation. Completed ',(100*IFRAME/nframes),' %'
      
      donor_loop: DO I1 = 1,ndonor
         acceptor_loop: DO I2 = 1,naccept
	 
            IMOL1 = nmoldonor(I1)
            IMOL2 = nmolaccept(I2)
	    
            donor_molecules_loop: DO J1 = 1,nummols(IMOL1)
               acceptor_molecules_loop:   DO J2 = 1,nummols(IMOL2) 

                  IF ( (IMOL1 == IMOL2) .AND. (J1 == J2) ) CYCLE acceptor_molecules_loop
               
                  ! Check potential Xdonor - Zacceptor distance
                  sdist = sss(1:3, nxdonor(IMOL1), J1, IMOL1, IFRAME) - &
                          sss(1:3, nxaccept(IMOL2), J2, IMOL2, IFRAME)
                  sdist = sdist - ANINT(sdist)
                  rx = sdist(1)*frame_cell(1,1,IFRAME) + sdist(2)*frame_cell(2,1,IFRAME) + sdist(3)*frame_cell(3,1,IFRAME)
                  ry = sdist(1)*frame_cell(1,2,IFRAME) + sdist(2)*frame_cell(2,2,IFRAME) + sdist(3)*frame_cell(3,2,IFRAME)
                  rz = sdist(1)*frame_cell(1,3,IFRAME) + sdist(2)*frame_cell(2,3,IFRAME) + sdist(3)*frame_cell(3,3,IFRAME)
                  r = SQRT( rx*rx + ry*ry + rz*rz )
                    
                  IF (r.le.rmax_XZ) THEN 
		   
                     ndistXZ_cond = ndistXZ_cond + 1
		     
                     s1 = sss(1:3, nxdonor(I1), J1, IMOL1, IFRAME)
                     s2 = sss(1:3, nhdonor(I1), J1, IMOL1, IFRAME)
                     s2 = s2 - ANINT(s2-s1)
                     s3 = sss(1:3, nxaccept(I2), J2, IMOL2, IFRAME)
                     s3 = s3 - ANINT(s3-s1)
                     s12 = s2 - s1		
                     s13 = s3 - s1		
                     d12 = SQRT( DOT_PRODUCT ( s12, s12 ) )
                     d13 = SQRT( DOT_PRODUCT ( s13, s13 ) )
                     costheta = DOT_PRODUCT ( s12, s13 ) / d12 / d13
                     theta = ACOS(costheta) * 180.0 / PI
		     
                     IF (theta.le.theta_max) THEN
		     
                        nfull_cond = nfull_cond + 1
                     
                        ! Check potential Hdonor - Zacceptor distance
                        sdist = sss(1:3, nhdonor(I1), J1, IMOL1, IFRAME) - &
                                sss(1:3, nxaccept(I2), J2, IMOL2, IFRAME)
                        sdist = sdist - ANINT(sdist)
                        rx = sdist(1)*frame_cell(1,1,IFRAME) + sdist(2)*frame_cell(2,1,IFRAME) + sdist(3)*frame_cell(3,1,IFRAME)
                        ry = sdist(1)*frame_cell(1,2,IFRAME) + sdist(2)*frame_cell(2,2,IFRAME) + sdist(3)*frame_cell(3,2,IFRAME)
                        rz = sdist(1)*frame_cell(1,3,IFRAME) + sdist(2)*frame_cell(2,3,IFRAME) + sdist(3)*frame_cell(3,3,IFRAME)
                        r = SQRT( rx*rx + ry*ry + rz*rz )
			
			IF (r.le.rmax_HZ) THEN 
		   
                           ndistHZ_cond = ndistHZ_cond + 1

			   ! All conditions fulfilled, update HB list
			   
                           IF (hb_list(IFRAME, I1, J1,1).eq.0) THEN
			
                              hb_list(IFRAME, I1, J1, 1) = I2
                              hb_list(IFRAME, I1, J1, 2) = J2
			   
                              hb_number(IMOL1, J1) = hb_number(IMOL1, J1) + 1
                              hb_number(IMOL2, J2) = hb_number(IMOL2, J2) + 1
			      
			      hb_species_histo(I1, I2) = hb_species_histo(I1, I2) + 1
			   
                           ELSE
			   
			      !This H was already "accepted" by another molecule --> shared HB

                              IF (hb_list(IFRAME, I1, J1,3).eq.0) THEN
			      
                                 nshared = nshared + 1
                              
			         hb_list(IFRAME, I1, J1, 3) = I2
                                 hb_list(IFRAME, I1, J1, 4) = J2
                              
			         hb_number(IMOL1, J1) = hb_number(IMOL1, J1) + 1
                                 hb_number(IMOL2, J2) = hb_number(IMOL2, J2) + 1
			      
			         hb_species_histo(I1, I2) = hb_species_histo(I1, I2) + 1
                              
			      ELSE 
                              
			         PRINT *, ' '
                                 PRINT *, 'X(O)-H bond shared by 3 acceptors?'
                                 PRINT *, 'Frame = ', IFRAME
                                 PRINT *, 'Donor is        ', I1, nmoldonor(I1), J1, nxdonor(nmoldonor(I1))
                                 PRINT *, '1st acceptor is ', hb_list(IFRAME, I1, J1, 1), nmolaccept(hb_list(IFRAME, I1, J1, 1)), &
                                                           hb_list(IFRAME, I1, J1, 2), nxaccept(hb_list(IFRAME, I1, J1, 1))
                                 PRINT *, '2nd acceptor is ', hb_list(IFRAME, I1, J1, 3), nmolaccept(hb_list(IFRAME, I1, J1, 3)), &
                                                           hb_list(IFRAME, I1, J1, 4), nxaccept(hb_list(IFRAME, I1, J1, 3))
                                 PRINT *, '3rd acceptor is ', I2, nmolaccept(I2), J2, nxaccept(I2)
                                 STOP
				 
                              END IF !end else too many acceptors
                           
			   END IF ! end updating HB list 
			   
			END IF ! end check distance H...Z
			                           
                     END IF ! end check angle H-X...Z
		     
                  END IF ! end check distance X...Z
			
			
               END DO acceptor_molecules_loop	
            END DO donor_molecules_loop	
	    
         END DO acceptor_loop	
      END DO donor_loop	

      DO I1 = 1,numspecies
         DO J1 = 1, nummols(I1)
            hb_histo(I1, hb_number(I1,J1)+1) = hb_histo(I1, hb_number(I1,J1)+1) + 1
         END DO
      END DO   
      
   END DO frames_loop
   
   UNIT = NEXT_UNIT ( )	
   OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT,'("# Calculation of % of H-bonds.")')
   WRITE (UNIT,'("# A donor H can be shared by a maximum of 2 acceptors.")')
   WRITE (UNIT,'("# ")') 
   WRITE (UNIT,'("# Number of potential HBs fulfilling distance criterion = ", I10)') ndistXZ_cond
   WRITE (UNIT,'("# Number of HBs fulfilling distance + angle criteria    = ", I10)') nfull_cond
   WRITE (UNIT,'("# Number of HBs fulfilling the three criteria           = ", I10)') ndistHZ_cond
   WRITE (UNIT,'("# Number of X-Hs shared by two acceptors                = ", I10)') nshared
   WRITE (UNIT,'(" ")') 
   WRITE (UNIT,'(" ")') 

   WRITE (UNIT,'("# Total number of HBs per molecule for each molecular species in the system:")')
   DO I1 = 1,numspecies
      WRITE (UNIT,'(" ")') 
      WRITE (UNIT, '(" Species index =  ", I2 )') I1
      hb_average = 0.0
      DO J1 = 1, max_hb_number+1
         hb_fraction = hb_histo(I1,J1)/float(nummols(I1))/float(nframes)
         hb_average = hb_average + (J1-1)*hb_fraction
         WRITE (UNIT, '("   % molecules with ", I1, "HBs = ", F6.2 )') J1-1, 100.0*hb_fraction
      END DO
      WRITE (UNIT, '("   Average number of HBs per molecule= ", F6.2 )') hb_average
      WRITE (UNIT,'(" ")') 
   END DO 
   WRITE (UNIT,'(" ")') 
   WRITE (UNIT,'(" ")') 
   
   WRITE (UNIT,'("# Analysis of HBs between different donors --> acceptors.")')
   DO I1 = 1,ndonor
      WRITE (UNIT,'(" ")') 
      WRITE (UNIT, '(" Donor index =  ", I2 )') I1
      DO I2 = 1, naccept
         WRITE (UNIT,'(" ")') 
         WRITE (UNIT, '("     Acceptor index =  ", I2 )') I2
         WRITE (UNIT, '("         Total number of HBs found                = ", I6)') hb_species_histo(I1,I2)
         WRITE (UNIT, '("         Average number of HBs per donor molecule = ", F6.2 )') &
	      float(hb_species_histo(I1,I2))/float(nummols(nmoldonor(I1)))/float(nframes)
      END DO
   END DO
   WRITE (UNIT,'(" ")') 
   WRITE (UNIT,'(" ")') 
   
   WRITE (UNIT,'("# Analysis of HBs between different acceptors  --> donors.")')
   DO I2 = 1,naccept
      WRITE (UNIT,'(" ")') 
      WRITE (UNIT, '(" Acceptor index =  ", I2 )') I2
      DO I1 = 1, ndonor
         WRITE (UNIT,'(" ")') 
         WRITE (UNIT, '("     Donor index =  ", I2 )') I1
         WRITE (UNIT, '("         Total number of HBs found                   = ", I6)') hb_species_histo(I1,I2)
         WRITE (UNIT, '("         Average number of HBs per acceptor molecule = ", F6.2 )') &
	      float(hb_species_histo(I1,I2))/float(nummols(nmolaccept(I2)))/float(nframes)
      END DO
   END DO
       
   CLOSE(UNIT)

         
END PROGRAM HB

