!
! Program to calculate HBs and clusters (only for a single species in simulation and same donor/acceptor atom)
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Writes:
!	1. ...
!
!  Last change: 21/12/2015
!

PROGRAM HB_CLUSTER	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES,   ONLY : frame_cell, average_cell_volume
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,       ONLY : TRAJECT_CART_2_FRACT
   
   IMPLICIT NONE
   
   INTEGER, PARAMETER :: max_hb_number = 5, max_size = 1000
   LOGICAL, PARAMETER :: print_clusters = .False.

   CHARACTER ( LEN = LINE_LENGTH)                      :: traject_file, ff_file, output_file, fnm, title
   REAL ( KIND = DP ), DIMENSION(3)                    :: sdist, s1, s2, s3, s12, s13

   INTEGER :: UNIT, IOSTAT, IFRAME, I, J, J2, K, IATOM, I1, I2, nframes, progress, nx, nh, ndist_cond, nfull_cond, ncluster, &
              nmolCluster, nhbChain, nhbBranch, nhbTerm, clusterIndex, number_points, bin

   REAL ( KIND = DP ) :: rmax, thmax, rx, ry, rz, r, tmp, d12, d13, costheta, theta, sss_last(3), total_clusters, &
                         tmp0, tmp1, tmp2, tmp3, tmp4, tmp5, tmp6, tmp7, tmp8, max_dist, rdf_max, rdf_dr, xa, xb, xx, &
                         shell_volume, n_ideal, gr_intra, gr_inter

   INTEGER, ALLOCATABLE, DIMENSION(:)   :: hb_number, mol_assigned, mol_cluster, hist_intra, hist_inter
   INTEGER, ALLOCATABLE, DIMENSION(:,:) :: hb_list
   
   INTEGER, DIMENSION(max_hb_number+1) :: hb_percent
   INTEGER, DIMENSION(max_size)        :: cluster_size_number, cluster_size_molecules
   INTEGER, DIMENSION(max_size)        :: ring_number, chain_number, branch1_number, branch2_number, branch3_number, branch4_number

   REAL ( KIND = DP ), DIMENSION(max_size) :: cluster_max_dist

   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr, sss
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:)     :: sss_cluster

   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,' ("Index of donor/acceptor atom?")')
   READ (INPUT,*) nx
   WRITE (OUTPUT,' ("Index of H atom?")')
   READ (INPUT,*) nh
   WRITE (OUTPUT,'("Maximum X(O)...X(O) distance to define HB?")')
   READ (INPUT,*) rmax
   WRITE (OUTPUT,'("Maximum X(O)-H...X(O) angle to define HB?")')
   READ (INPUT,*) thmax
   WRITE (OUTPUT,'("Maximum distance to compute the r.d.f. (give 0 if not needed)?")')
   READ (INPUT,*) rdf_max
   WRITE (OUTPUT,'("Delta r value to use in r.d.f. grid?")')
   READ (INPUT,*) rdf_dr
   WRITE (OUTPUT,*)
   

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF (NUMSPECIES.GT.1) STOP 'This program works only with a single species'
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	     WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	     READ (INPUT,*) nummols(I) 
	     WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	     READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 
     
   IF (NUMSPECIES.GT.1) STOP 'This program works only with a single species'  
 
   ! Read trajectory and transform to fractional coordinates
   ! Needed in order to determine minimum image distance for a non-orthorombic system
      
   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   PRINT *,'Finished reading trajectory!'
   CALL TRAJECT_CART_2_FRACT (nframes, rrr, sss)
   
   ALLOCATE (sss_cluster(3, maxval(numatoms), maxval(nummols)))
   
   
   ! Initialize counting variables and arrays
   ALLOCATE ( hb_list(maxval(nummols), max_hb_number) )  !list of molecules joined to a given one
   ALLOCATE ( hb_number(maxval(nummols)) )               !number of HBs for each molecule in system
   ALLOCATE ( mol_assigned(maxval(nummols)) )            !index to determine if molecule has already been assigned to a cluster
   ALLOCATE ( mol_cluster(maxval(nummols)) )             !cluster number to which the molecule has been assigned 
   ndist_cond = 0  !number of pairs fulfilling distance criterion
   nfull_cond = 0  !number of pairs fulfilling distance + angle criteria


   ! Cluster statistics
   hb_percent = 0
   cluster_size_number = 0
   cluster_size_molecules = 0
   cluster_max_dist = 0.0
   ring_number = 0
   chain_number = 0
   branch1_number = 0
   branch2_number = 0
   branch3_number = 0
   branch4_number = 0

   ! Cluster statistics and g_XX(r) intra- and inter-clusters
   IF (rdf_max > 0) THEN
      number_points = INT(rdf_max/rdf_dr) 
      ALLOCATE ( hist_intra(number_points), hist_inter(number_points) )
      hist_intra = 0; hist_inter = 0
   END IF

   ! Compute lists of HBs
   
   frames_loop: DO IFRAME = 1,nframes

      !progress = nframes / 20
      !IF (progress < 1) progress = 1
      !IF (MOD(IFRAME,progress) == 0) PRINT *,'HB list calculation. Completed ',(100*IFRAME/nframes),' %'
      
      hb_list = 0
      hb_number = 0

      mol1_loop: DO I1 = 1,nummols(1)
         mol2_loop:   DO I2 = 1,nummols(1) 

            IF ( (I1 == I2) ) CYCLE mol2_loop
               
			! Check potential Xdonor - Xacceptor distance
			sdist = sss(1:3, nx, I1, 1, IFRAME) -  sss(1:3, nx, I2, 1, IFRAME)
			sdist = sdist - ANINT(sdist)
            rx = sdist(1)*frame_cell(1,1,IFRAME) + sdist(2)*frame_cell(2,1,IFRAME) + sdist(3)*frame_cell(3,1,IFRAME)
            ry = sdist(1)*frame_cell(1,2,IFRAME) + sdist(2)*frame_cell(2,2,IFRAME) + sdist(3)*frame_cell(3,2,IFRAME)
            rz = sdist(1)*frame_cell(1,3,IFRAME) + sdist(2)*frame_cell(2,3,IFRAME) + sdist(3)*frame_cell(3,3,IFRAME)
            r = SQRT( rx*rx + ry*ry + rz*rz )
                    
            IF (r.le.rmax) THEN  
                ndist_cond = ndist_cond + 1
                s1 = sss(1:3, nx, I1, 1, IFRAME)
                s2 = sss(1:3, nh, I1, 1, IFRAME)
                s2 = s2 - ANINT(s2-s1)
                s3 = sss(1:3, nx, I2, 1, IFRAME)
                s3 = s3 - ANINT(s3-s1)
	            s12 = s2 - s1	!O-H	
	            s13 = s3 - s1   !O...O
                d12 = SQRT( DOT_PRODUCT ( s12, s12 ) )
                d13 = SQRT( DOT_PRODUCT ( s13, s13 ) )
		        costheta = DOT_PRODUCT ( s12, s13 ) / d12 / d13
		        theta = ACOS(costheta) * 180.0 / PI
                IF (theta.le.thmax) THEN
                    nfull_cond = nfull_cond + 1
                    hb_number(I1) = hb_number(I1) + 1
                    hb_number(I2) = hb_number(I2) + 1
                    IF (hb_number(I1).gt.max_hb_number) STOP 'Increase max_hb_number'
                    IF (hb_number(I2).gt.max_hb_number) STOP 'Increase max_hb_number'
                    hb_list(I1, hb_number(I1)) = I2
                    hb_list(I2, hb_number(I2)) = I1
                END IF
            END IF

         END DO mol2_loop	
	  END DO mol1_loop	
      
      DO I1 = 1, nummols(1)
         hb_percent(hb_number(I1)+1) = hb_percent(hb_number(I1)+1) + 1
      END DO

      
      ! Build clusters and get number of clusters in frame
      ncluster = 0
      mol_assigned = 0
      mol_cluster = 0
      sss_cluster = sss(1:3, :, :, 1, IFRAME)
      DO I = 1, nummols(1)
         IF (mol_assigned(I)==1) CYCLE
         ! Start new cluster
         ncluster = ncluster + 1
         sss_last = sss_cluster(:, nx, I)
         CALL ADD_CLUSTER (ncluster, I, hb_list, hb_number, mol_cluster, mol_assigned, max_hb_number, sss_cluster, sss_last, nx)
      END DO
      
      PRINT *, 'FRAME = ', IFRAME, ' - NUMBER OF CLUSTERS = ', NCLUSTER
      
      DO I = 1, ncluster
         nmolCluster = 0
         nhbChain = 0
         nhbBranch = 0
         nhbTerm = 0
         max_dist = 0.0
         ! Determine number of molecules composing cluster
         DO J = 1, nummols(1)
            IF (mol_cluster(J)==I) THEN
               nmolCluster = nmolCluster+1
               nhbChain = nhbChain + hb_number(J)
               IF (hb_number(J)==1) nhbTerm = nhbTerm + 1
               IF (hb_number(J)>2) nhbBranch = nhbBranch + 1
               ! Maximum distance between O atoms in cluster
               DO J2 = 1, nummols(1)
                  IF (J2 == J) CYCLE
                  IF (mol_cluster(J2)==I) THEN
                     sdist = sss_cluster(1:3, nx, J2) -  sss_cluster(1:3, nx, J)
                     rx = sdist(1)*frame_cell(1,1,IFRAME) + sdist(2)*frame_cell(2,1,IFRAME) + sdist(3)*frame_cell(3,1,IFRAME)
                     ry = sdist(1)*frame_cell(1,2,IFRAME) + sdist(2)*frame_cell(2,2,IFRAME) + sdist(3)*frame_cell(3,2,IFRAME)
                     rz = sdist(1)*frame_cell(1,3,IFRAME) + sdist(2)*frame_cell(2,3,IFRAME) + sdist(3)*frame_cell(3,3,IFRAME)
                     r = SQRT( rx*rx + ry*ry + rz*rz )
                     IF (r > max_dist) max_dist = r
                     IF (rdf_max > 0) THEN
                        !g_XX(r) intra-clusters 
                        IF (r < rdf_max) THEN
                           bin = INT(r/rdf_dr) + 1
                           hist_intra(bin) = hist_intra(bin) + 1
                        ELSE
                           !either intra contributions for r > rdf_max are neglected --> nothing to do
                           !or the closest image is checked and if r_image < rdf_max then counted either as intra or as inter
                           !sdist = sdist - ANINT(sdist)
                           !rx = sdist(1)*frame_cell(1,1,IFRAME) + sdist(2)*frame_cell(2,1,IFRAME) + sdist(3)*frame_cell(3,1,IFRAME)
                           !ry = sdist(1)*frame_cell(1,2,IFRAME) + sdist(2)*frame_cell(2,2,IFRAME) + sdist(3)*frame_cell(3,2,IFRAME)
                           !rz = sdist(1)*frame_cell(1,3,IFRAME) + sdist(2)*frame_cell(2,3,IFRAME) + sdist(3)*frame_cell(3,3,IFRAME)
                           !r = SQRT( rx*rx + ry*ry + rz*rz )
                           !IF (r < rdf_max) THEN
                           !   hist_intra(bin) = hist_intra(bin) + 1
                           !   hist_inter(bin) = hist_inter(bin) + 1
                           !END IF
                        END IF
                     END IF   
                  ELSE
                     IF (rdf_max > 0) THEN                  
                        !g_XX(r) inter-clusters
                        sdist = sss_cluster(1:3, nx, J2) -  sss_cluster(1:3, nx, J)
                        sdist = sdist - ANINT(sdist)
                        rx = sdist(1)*frame_cell(1,1,IFRAME) + sdist(2)*frame_cell(2,1,IFRAME) + sdist(3)*frame_cell(3,1,IFRAME)
                        ry = sdist(1)*frame_cell(1,2,IFRAME) + sdist(2)*frame_cell(2,2,IFRAME) + sdist(3)*frame_cell(3,2,IFRAME)
                        rz = sdist(1)*frame_cell(1,3,IFRAME) + sdist(2)*frame_cell(2,3,IFRAME) + sdist(3)*frame_cell(3,3,IFRAME)
                        r = SQRT( rx*rx + ry*ry + rz*rz )
                        IF (r < rdf_max) THEN
                           bin = INT(r/rdf_dr) + 1
                           hist_inter(bin) = hist_inter(bin) + 1
                        END IF
                     END IF   
                  END IF
               END DO 
            END IF   
         END DO
         IF (nmolCluster < max_size) THEN 
            clusterIndex = nmolCluster
         ELSE
            clusterIndex = max_size
         END IF         
         cluster_size_number(clusterIndex)    = cluster_size_number(clusterIndex) + 1
         cluster_size_molecules(clusterIndex) = cluster_size_molecules(clusterIndex) + nmolCluster
         cluster_max_dist(clusterIndex) = max_dist

         ! Special cases
         
         IF (nhbBranch == 0) THEN

            ! Pure ring
            IF (nhbChain == nmolCluster*2) THEN
               ring_number(clusterIndex) = ring_number(clusterIndex) + nmolCluster
               IF (print_clusters) THEN
	              WRITE (fnm,'("shape1_",I4.4,"_",I6.6,".xyz")') I, iframe
                  title = 'Isolated ring'
                  CALL WRITE_CLUSTER(fnm, title, I, mol_cluster, nmolCluster, nhbChain, nhbBranch, nhbTerm, sss_cluster, iframe)
               END IF                  
            END IF

            ! Pure linear chain
            IF ((nhbChain > 0).and.(nhbChain == nmolCluster*2-2)) THEN
               chain_number(clusterIndex) = chain_number(clusterIndex) + nmolCluster
               IF (print_clusters) THEN
	              WRITE (fnm,'("shape2_",I4.4,"_",I6.6,".xyz")') I, iframe
                  title = 'Linear chain'
                  CALL WRITE_CLUSTER(fnm, title, I, mol_cluster, nmolCluster, nhbChain, nhbBranch, nhbTerm, sss_cluster, iframe)
               END IF
            END IF
         END IF  

         IF (nhbBranch == 1) THEN

            IF (nhbTerm == 3) THEN
               branch1_number(clusterIndex) = branch1_number(clusterIndex) + nmolCluster
               IF (print_clusters) THEN
	              WRITE (fnm,'("shape3_",I4.4,"_",I6.6,".xyz")') I, iframe
                  title = 'Three-branch star'
                 CALL WRITE_CLUSTER(fnm, title, I, mol_cluster, nmolCluster, nhbChain, nhbBranch, nhbTerm, sss_cluster, iframe)
               END IF 
               
            ELSE
               branch2_number(clusterIndex) = branch2_number(clusterIndex) + nmolCluster
               IF (print_clusters) THEN
	              WRITE (fnm,'("shape4_",I4.4,"_",I6.6,".xyz")') I, iframe
                  title = 'Loop + chain'
                  CALL WRITE_CLUSTER(fnm, title, I, mol_cluster, nmolCluster, nhbChain, nhbBranch, nhbTerm, sss_cluster, iframe)
               END IF
            END IF 
            
         END IF

         IF (nhbBranch > 1) THEN
         
            IF (nhbTerm == 2*nhbBranch) THEN
               branch3_number(clusterIndex) = branch3_number(clusterIndex) + nmolCluster
               IF (print_clusters) THEN
	              WRITE (fnm,'("shape5_",I4.4,"_",I6.6,".xyz")') I, iframe
                  title = 'Branched chain'
                  CALL WRITE_CLUSTER(fnm, title, I, mol_cluster, nmolCluster, nhbChain, nhbBranch, nhbTerm, sss_cluster, iframe)
               END IF 
               
            ELSE
               branch4_number(clusterIndex) = branch4_number(clusterIndex) + nmolCluster
               IF (print_clusters) THEN
	              WRITE (fnm,'("shape6_",I4.4,"_",I6.6,".xyz")') I, iframe
                  title = 'Branched chain with loops'
                  CALL WRITE_CLUSTER(fnm, title, I, mol_cluster, nmolCluster, nhbChain, nhbBranch, nhbTerm, sss_cluster, iframe)
               END IF                  
            END IF
            
         END IF
         
      END DO

   END DO frames_loop
   
   PRINT *, ' '
   PRINT *, 'Number of potential HBs fulfilling distance criterion = ', ndist_cond
   PRINT *, 'Number of HBs fulfilling distance + angle criteria    = ', nfull_cond, '(', 100.*nfull_cond/ndist_cond,' %)'
   PRINT *, 'Maximum number of HBs found in one molecule           = ', maxval(hb_number)
   PRINT *, ' '

   UNIT = NEXT_UNIT ( )	
   OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT,'("# Average number of H-bonds per molecule:")')
   DO I = 1, max_hb_number+1
      WRITE (UNIT, '("   % molecules with ", I1, "HBs = ", F6.2 )') I-1, 100.0*hb_percent(I)/float(nummols(1))/float(nframes)
   END DO
   WRITE (UNIT, '("#")')

   WRITE (UNIT, '("# Cluster sizes:")')
   WRITE (UNIT,'("#   N", 11x, "n", 13x, "%mol", 6x, "% in rings", 3x, "% in chains", 2x)', advance="no") 
   WRITE (UNIT,'("% in 3-starts", 2x, "% loop+chain", 2x, "% branches", 2x, "% branches+loops", 9x, "ROO_max")', advance="no")
   total_clusters =  sum(cluster_size_number)             
   tmp0 = 100.0 / float(nummols(1)) / float(nframes)
   DO I = 1, max_size
      tmp1 = 100.0*float(cluster_size_number(I)) / total_clusters
      tmp2 = float(cluster_size_molecules(I)) * tmp0
      tmp3 = float(ring_number(I)) * tmp0
      tmp4 = float(chain_number(I)) * tmp0
      tmp5 = float(branch1_number(I)) * tmp0
      tmp6 = float(branch2_number(I)) * tmp0
      tmp7 = float(branch3_number(I)) * tmp0
      tmp8 = float(branch4_number(I)) * tmp0
      WRITE (UNIT, '(1X, I4, 6X, 8(F10.5,4X), 6X, F10.3)') I, tmp1, tmp2, tmp3, tmp4, tmp5, tmp6, tmp7, tmp8, cluster_max_dist(I)
   END DO
   WRITE (UNIT, '("# Sum = ", 3x, 8(F10.5,4X))') 100.0*sum(cluster_size_number)/total_clusters, &
      tmp0*sum(cluster_size_molecules), tmp0*sum(ring_number), tmp0*sum(chain_number), &
      tmp0*sum(branch1_number), tmp0*sum(branch2_number), tmp0*sum(branch3_number), tmp0*sum(branch4_number)
      
   IF (rdf_max > 0) THEN

      ! Write g_XX(r) intra- and inter-clusters
      UNIT = NEXT_UNIT ( )	
      OPEN ( UNIT, FILE = 'gr_clusters_XX.dat', ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
      WRITE (UNIT,'(''# g(r) intra- and inter-clusters'')')
      WRITE (UNIT,'(''#'',3x,''r/A'',4x,''g(r) intra'',2x,''g(r) inter'',2x,''sum'')')
      DO K = 1,number_points
         xa = REAL(K-1)*rdf_dr
         xb = xa + rdf_dr
         xx = 0.5 * (xa + xb)
         shell_volume = 4.0 * pi * (xb**3 - xa**3) / 3.0
         n_ideal = REAL( nummols(1) * (nummols(1)-1.0) ) / average_cell_volume
         gr_intra = REAL(hist_intra(K))/ REAL(nframes) / shell_volume / n_ideal
         gr_inter = REAL(hist_inter(K))/ REAL(nframes) / shell_volume / n_ideal
         WRITE (UNIT,"(1x,f7.3,3(2x,f10.6))") xx, gr_intra, gr_inter, gr_intra+gr_inter
      END DO

   END IF
      
END PROGRAM HB_CLUSTER

RECURSIVE SUBROUTINE ADD_CLUSTER (NC, IMOL, HBLIST, HBN, MOLCLUSTER, MOLASSIGNED, MAXHB, SSSCLUSTER, SSSLAST, IX)
   
   USE DEFINITIONS, ONLY :  DP
   USE GENERAL_VARIABLES
   
   IMPLICIT NONE
   INTEGER :: NC, IMOL, I, INEW, MAXHB, IX, IATOM
   INTEGER :: HBLIST(maxval(nummols),MAXHB), HBN(maxval(nummols)), MOLCLUSTER(maxval(nummols)), MOLASSIGNED(maxval(nummols))
   REAL ( KIND = DP ), DIMENSION(3,maxval(numatoms),maxval(nummols)) :: SSSCLUSTER
   REAL ( KIND = DP ), DIMENSION(3) :: SSSLAST, DIF
   
   MOLCLUSTER(IMOL) = NC
   MOLASSIGNED(IMOL) = 1

   DIF = SSSCLUSTER(:, IX, IMOL) - SSSLAST
   SSSCLUSTER(1,IX,IMOL) = SSSCLUSTER(1,IX,IMOL) - ANINT(DIF(1))
   SSSCLUSTER(2,IX,IMOL) = SSSCLUSTER(2,IX,IMOL) - ANINT(DIF(2))
   SSSCLUSTER(3,IX,IMOL) = SSSCLUSTER(3,IX,IMOL) - ANINT(DIF(3))
   SSSLAST = SSSCLUSTER(:, IX, IMOL)
   DO IATOM = 1, numatoms(1)
      DIF = SSSCLUSTER(:, IATOM, IMOL) - SSSCLUSTER(:, IX, IMOL)
      SSSCLUSTER(1,IATOM,IMOL) = SSSCLUSTER(1,IATOM,IMOL) - ANINT(DIF(1))
      SSSCLUSTER(2,IATOM,IMOL) = SSSCLUSTER(2,IATOM,IMOL) - ANINT(DIF(2))
      SSSCLUSTER(3,IATOM,IMOL) = SSSCLUSTER(3,IATOM,IMOL) - ANINT(DIF(3))
   END DO

   DO I = 1, HBN(IMOL)
      INEW = HBLIST(IMOL,I)
      IF (MOLASSIGNED(INEW)==1) CYCLE
      CALL ADD_CLUSTER (NC, INEW, HBLIST, HBN, MOLCLUSTER, MOLASSIGNED, MAXHB, SSSCLUSTER, SSSLAST, IX)
   END DO
   
END SUBROUTINE ADD_CLUSTER

SUBROUTINE WRITE_CLUSTER (FILENAME, TITLE, NC, MOLCLUSTER, nmolCluster, nhbChain, nhbBranch, nhbTerm, SSSCLUSTER, iframe)
                  
   USE DEFINITIONS, ONLY : LINE_LENGTH, DP
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES, ONLY : frame_cell
   USE FORCEFIELD_VARIABLES,   ONLY : atomname
   
   IMPLICIT NONE
   CHARACTER ( LEN = LINE_LENGTH) :: filename, title
   INTEGER :: NC, nmolCluster, nhbChain, nhbBranch, nhbTerm, iframe, J, IATOM
   INTEGER :: MOLCLUSTER(maxval(nummols))
   REAL ( KIND = DP ), DIMENSION(3,maxval(numatoms),maxval(nummols)) :: SSSCLUSTER
   REAL ( KIND = DP) :: rx, ry, rz

   OPEN (11, FILE=filename, ACTION = "WRITE", STATUS = "UNKNOWN")
   WRITE (11, '(I6)') nmolCluster*numatoms(1)
   WRITE (11, '(A20, 2X, ", size = ", I4, 4X, "(", I4, 2X, I4, 2X, I4")")') title, nmolCluster, nhbChain, nhbBranch, nhbTerm
   DO J = 1, nummols(1)
      IF (MOLCLUSTER(J)==NC) THEN
         DO IATOM = 1, numatoms(1)
            rx = SSSCLUSTER(1, IATOM, J) * frame_cell(1,1,IFRAME) + &
                 SSSCLUSTER(2, IATOM, J) * frame_cell(2,1,IFRAME) + &
                 SSSCLUSTER(3, IATOM, J) * frame_cell(3,1,IFRAME)
            ry = SSSCLUSTER(1, IATOM, J) * frame_cell(1,2,IFRAME) + &
                 SSSCLUSTER(2, IATOM, J) * frame_cell(2,2,IFRAME) + &
                 SSSCLUSTER(3, IATOM, J) * frame_cell(3,2,IFRAME)
            rz = SSSCLUSTER(1, IATOM, J) * frame_cell(1,3,IFRAME) + &
                 SSSCLUSTER(2, IATOM, J) * frame_cell(2,3,IFRAME) + &
                 SSSCLUSTER(3, IATOM, J) * frame_cell(3,3,IFRAME)
            WRITE (11, '(A6, 5X, 3(F8.3,2X))') atomname(IATOM,1), rx, ry, rz
         END DO
      END IF   
   END DO
   CLOSE (11)
      
END SUBROUTINE WRITE_CLUSTER
