!
! Reads a trajectory file and writes PDB files of close imidazolium pairs
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Writes:
!	1. A series of PDB files
!
!  Last change: 22/03/2010
!

PROGRAM IMIDAZOLIUM_PAIRS_PDB	

   USE DEFINITIONS,          ONLY : LINE_LENGTH, DP
   USE IO_UNITS,             ONLY : INPUT, OUTPUT
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES, ONLY : pbc_jumps
   USE FORCEFIELD_READ,      ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE, ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,     ONLY : TRAJECT_UNWRAP, TRAJECT_JOIN_MOLECULES
   USE CONFIG_READWRITE,     ONLY : CONFIG_PDB_WRITE
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH)      :: traject_file, ff_file, output_file, pdb_file
   CHARACTER ( LEN = 4)                :: frame_number
   CHARACTER ( LEN = 3)                :: m1, m2
   INTEGER                             :: I, IFRAME, J1, J2, J3, nfirst, nlast, ngap, n_br, nmols_im, nmols_br, nmols_pairs
   REAL ( KIND = DP )                  :: rmin, rmax, rmax_br, center1x, center1y, center1z, center2x, center2y, center2z, &
                                          centerx, centery, centerz, dist, rx, ry, rz, dist_br_1, dist_br_2, dist_br_3, dist_br

   INTEGER, ALLOCATABLE, DIMENSION(:)                    :: index
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: traject_r
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:)   :: config_pos


   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of first frame to write as a PDB file?")')
   READ (INPUT,*) nfirst
   WRITE (OUTPUT,'("Number of last frame to write as a PDB file?")')
   READ (INPUT,*) nlast
   WRITE (OUTPUT,'("Selected gap between PDB files to write?")')
   READ (INPUT,*) ngap
   WRITE (OUTPUT,'("Minimum distance between imidazolium centers?")')
   READ (INPUT,*) rmin
   WRITE (OUTPUT,'("Maximum distance between imidazolium centers?")')
   READ (INPUT,*) rmax
   WRITE (OUTPUT,'("Maximum distance between Br and pair center?")')
   READ (INPUT,*) rmax_br
   WRITE (OUTPUT,'("Name of output file (without extension .pdb)?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,*)
   

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"? ")', ADVANCE='NO') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule? ")', ADVANCE='NO')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF
   
   !Allocate index for Br atoms
   ALLOCATE (index(nummols(2))); index = 0 

   
   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ ( traject_file, nlast, traject_r )
   PRINT *,'Finished reading trajectory!'
   IF (pbc_jumps) CALL TRAJECT_UNWRAP ( nlast, traject_r )  
   CALL TRAJECT_JOIN_MOLECULES ( nlast, traject_r )
   

   DO IFRAME = nfirst, nlast, ngap 
      ! only for imidazolium = species 1 & ring defined by atoms 1-5 & br = species 2
      DO J1 = 1, nummols(1)-1
	 center1x = SUM ( traject_r (1, 1:5, J1, 1, IFRAME) ) 
	 center1y = SUM ( traject_r (2, 1:5, J1, 1, IFRAME) ) 
	 center1z = SUM ( traject_r (3, 1:5, J1, 1, IFRAME) ) 
	 center1x = center1x / 5.
	 center1y = center1y / 5.
	 center1z = center1z / 5.
         DO J2 = J1+1, nummols(1)
	    center2x = SUM ( traject_r (1, 1:5, J2, 1, IFRAME) ) 
	    center2y = SUM ( traject_r (2, 1:5, J2, 1, IFRAME) ) 
	    center2z = SUM ( traject_r (3, 1:5, J2, 1, IFRAME) ) 
	    center2x = center2x / 5.
	    center2y = center2y / 5.
	    center2z = center2z / 5.
            dist = SQRT ( (center2x - center1x)**2 + (center2y - center1y)**2 + (center2z - center1z)**2 )

            IF ( (dist >= rmin) .AND. (dist <= rmax) ) THEN

	       centerx = ( center1x + center2x ) / 2.
	       centery = ( center1y + center2y ) / 2.
	       centerz = ( center1z + center2z ) / 2.
	       n_br = 0
	       DO J3 = 1, nummols(2)
	          rx = traject_r (1, 1, J3, 2, IFRAME) - center1x
	          ry = traject_r (2, 1, J3, 2, IFRAME) - center1y
	          rz = traject_r (3, 1, J3, 2, IFRAME) - center1z
		  dist_br_1 = SQRT ( rx**2 + ry**2 + rz**2)
	          rx = traject_r (1, 1, J3, 2, IFRAME) - center2x
	          ry = traject_r (2, 1, J3, 2, IFRAME) - center2y
	          rz = traject_r (3, 1, J3, 2, IFRAME) - center2z
		  dist_br_2 = SQRT ( rx**2 + ry**2 + rz**2)
	          rx = traject_r (1, 1, J3, 2, IFRAME) - centerx
	          ry = traject_r (2, 1, J3, 2, IFRAME) - centery
	          rz = traject_r (3, 1, J3, 2, IFRAME) - centerz
		  dist_br_3 = SQRT ( rx**2 + ry**2 + rz**2)
		  dist_br = MIN ( dist_br_1,dist_br_2, dist_br_3 )
		  IF ( dist_br <= rmax_br) THEN
		     n_br = n_br + 1
		     index(n_br) = J3
		  END IF
	       END DO	  

	       WRITE (OUTPUT,'("Pair ",I3,1x,I3,1x,"dist = ",F6.3,2x," N_br = ",I2,", numbers: ",10(I3,1X))') &
	       J1, J2, dist, n_br, (index(J3),J3=1,n_br)
               
               nmols_im = nummols(1)
               nmols_br = nummols(2)
	       nmols_pairs = MAX ( 2, n_br)
               IF (ALLOCATED(config_pos)) DEALLOCATE(config_pos)
               ALLOCATE ( config_pos ( 3, maxval(numatoms), nmols_pairs, 2 ) ); config_pos = 0.0
               nummols(1) = 2; nummols(2) = n_br
               WRITE (frame_number,'(I4.4)') IFRAME
               WRITE (m1,'(I3.3)') J1
               WRITE (m2,'(I3.3)') J2
	       pdb_file = output_file(1:LEN_TRIM(output_file))//frame_number//'_'//m1//'_'//m2//'.pdb'
               config_pos ( 1:3, :, 1, 1 ) = traject_r (1:3, :, J1, 1, IFRAME)
               config_pos ( 1:3, :, 2, 1 ) = traject_r (1:3, :, J2, 1, IFRAME)
	       DO J3 = 1, n_br
	          config_pos ( 1:3, 1, J3, 2 ) = traject_r (1:3, 1, index(J3), 2, IFRAME)
	       END DO
               CALL CONFIG_PDB_WRITE ( pdb_file, config_pos)
	       ! Restore values
               nummols(1) = nmols_im
	       nummols(2) = nmols_br

	    END IF
	      
         END DO
      END DO

   END DO
   
END PROGRAM IMIDAZOLIUM_PAIRS_PDB
