!
! Reads a trajectory file and writes PDB files of close imidazolium pairs
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Writes:
!	1. A series of PDB files
!
!  Last change: 19/04/2010
!

PROGRAM IMIDAZOLIUM_PAIRS_PDB_2	

   USE DEFINITIONS,          ONLY : LINE_LENGTH, DP
   USE IO_UNITS,             ONLY : INPUT, OUTPUT
   USE CHEMPHYSMATH_CONSTANTS,ONLY: PI
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES, ONLY : pbc_jumps, frame_cell, average_cell_volume
   USE FORCEFIELD_READ,      ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE, ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,     ONLY : TRAJECT_UNWRAP, TRAJECT_JOIN_MOLECULES, TRAJECT_CART_2_FRACT
   USE CONFIG_READWRITE,     ONLY : CONFIG_PDB_WRITE
   USE LINEAR_ALGEBRA,       ONLY : NORMALIZE, CROSS_PRODUCT
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH)      :: traject_file, ff_file, output_file, pdb_file
   CHARACTER ( LEN = 4)                :: frame_number
   CHARACTER ( LEN = 3)                :: m1, m2
   INTEGER                             :: I, IFRAME, J1, J2, J3, nfirst, nlast, ngap, nimidazolium, nmols_im, nmols_br
   REAL ( KIND = DP )                  :: rmin, rmax, rmax_br, center1x, center1y, center1z, center2x, center2y, center2z, &
                                          centerx, centery, centerz, distx, disty, distz, dist, rx, ry, rz, dot, theta
   REAL ( KIND = DP ), DIMENSION(3)    :: mol1_12, mol1_13, mol1_axis, mol2_12, mol2_13, mol2_axis 

   INTEGER, ALLOCATABLE, DIMENSION(:)                    :: index
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: traject_r, traject_s
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:)   :: config_pos


   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of first frame to write as a PDB file?")')
   READ (INPUT,*) nfirst
   WRITE (OUTPUT,'("Number of last frame to write as a PDB file?")')
   READ (INPUT,*) nlast
   WRITE (OUTPUT,'("Selected gap between PDB files to write?")')
   READ (INPUT,*) ngap
   WRITE (OUTPUT,'("Minimum distance between imidazolium centers?")')
   READ (INPUT,*) rmin
   WRITE (OUTPUT,'("Maximum distance between imidazolium centers?")')
   READ (INPUT,*) rmax
   WRITE (OUTPUT,'("Number of selected imidazolium molecule?")')
   READ (INPUT,*) nimidazolium
   WRITE (OUTPUT,'("Name of output file (without extension .pdb)?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,*)
   

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"? ")', ADVANCE='NO') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule? ")', ADVANCE='NO')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF
   
   
   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ ( traject_file, nlast, traject_r )
   PRINT *,'Finished reading trajectory!'
   IF (pbc_jumps) CALL TRAJECT_UNWRAP ( nlast, traject_r )  
   CALL TRAJECT_JOIN_MOLECULES ( nlast, traject_r )
   CALL TRAJECT_CART_2_FRACT ( nlast, traject_r, traject_s)
   

   DO IFRAME = nfirst, nlast, ngap 
      ! only for imidazolium = species 1 & ring defined by atoms 1-5 & br = species 2
      DO J1 = nimidazolium, nimidazolium
	 center1x = SUM ( traject_s (1, 1:5, J1, 1, IFRAME) ) 
	 center1y = SUM ( traject_s (2, 1:5, J1, 1, IFRAME) ) 
	 center1z = SUM ( traject_s (3, 1:5, J1, 1, IFRAME) ) 
	 center1x = center1x / 5.
	 center1y = center1y / 5.
	 center1z = center1z / 5.
         DO J2 = 1, nummols(1)
	    IF (J1 == J2) CYCLE
	    center2x = SUM ( traject_s (1, 1:5, J2, 1, IFRAME) ) 
	    center2y = SUM ( traject_s (2, 1:5, J2, 1, IFRAME) ) 
	    center2z = SUM ( traject_s (3, 1:5, J2, 1, IFRAME) ) 
	    center2x = center2x / 5.
	    center2y = center2y / 5.
	    center2z = center2z / 5.
	    distx = center2x - center1x
	    disty = center2y - center1y
	    distz = center2z - center1z
	    distx = distx - ANINT(distx)
	    disty = disty - ANINT(disty)
	    distz = distz - ANINT(distz)
            rx = distx*frame_cell(1,1,IFRAME) + disty*frame_cell(2,1,IFRAME) + distz*frame_cell(3,1,IFRAME)
            ry = distx*frame_cell(1,2,IFRAME) + disty*frame_cell(2,2,IFRAME) + distz*frame_cell(3,2,IFRAME)
            rz = distx*frame_cell(1,3,IFRAME) + disty*frame_cell(2,3,IFRAME) + distz*frame_cell(3,3,IFRAME)
            dist = SQRT ( rx**2 + ry**2 + rz**2 )

            IF ( (dist >= rmin) .AND. (dist <= rmax) ) THEN

               mol1_12(1:3) = traject_r (1:3, 2, J1, 1, IFRAME) - traject_r (1:3, 1, J1, 1, IFRAME)
               mol1_12(1:3) = NORMALIZE( mol1_12(1:3) )
               mol1_13(1:3) = traject_r (1:3, 3, J1, 1, IFRAME) - traject_r (1:3, 1, J1, 1, IFRAME)
               mol1_13(1:3) = NORMALIZE( mol1_13(1:3) )
               mol1_axis(1:3) = CROSS_PRODUCT( mol1_12(1:3), mol1_13(1:3) )
               mol1_axis(1:3) = NORMALIZE( mol1_axis(1:3) )
               mol2_12(1:3) = traject_r (1:3, 2, J2, 1, IFRAME) - traject_r (1:3, 1, J2, 1, IFRAME)
               mol2_12(1:3) = NORMALIZE( mol2_12(1:3) )
               mol2_13(1:3) = traject_r (1:3, 3, J2, 1, IFRAME) - traject_r (1:3, 1, J2, 1, IFRAME)
               mol2_13(1:3) = NORMALIZE( mol2_13(1:3) )
               mol2_axis(1:3) = CROSS_PRODUCT( mol2_12(1:3), mol2_13(1:3) )
               mol2_axis(1:3) = NORMALIZE( mol2_axis(1:3) )

               dot = DOT_PRODUCT( mol1_axis(1:3), mol2_axis(1:3) )
	       theta = ACOS(dot)*180.0/pi

	       WRITE (OUTPUT,'("Pair ",I3,1x,I3,1x,"dist = ",F6.3,1x,"angle = ",F6.3)') J1, J2, dist, theta
               
               nmols_im = nummols(1)
               nmols_br = nummols(2)
               IF (ALLOCATED(config_pos)) DEALLOCATE(config_pos)
               ALLOCATE ( config_pos ( 3, numatoms(1), 2, 1 ) ); config_pos = 0.0
               numspecies = 1; nummols(1) = 2
               WRITE (frame_number,'(I4.4)') IFRAME
               WRITE (m1,'(I3.3)') J1
               WRITE (m2,'(I3.3)') J2
	       pdb_file = output_file(1:LEN_TRIM(output_file))//frame_number//'_'//m1//'_'//m2//'.pdb'
               config_pos ( 1:3, :, 1, 1 ) = traject_r (1:3, :, J1, 1, IFRAME)
               config_pos ( 1:3, :, 2, 1 ) = traject_r (1:3, :, J2, 1, IFRAME)
               CALL CONFIG_PDB_WRITE ( pdb_file, config_pos)
	       ! Restore values
               nummols(1) = nmols_im
	       nummols(2) = nmols_br

	    END IF
	      
         END DO
      END DO

   END DO
   
END PROGRAM IMIDAZOLIUM_PAIRS_PDB_2
