!
! Program to calculate particular intramolecular electrostatic contributions
!
! 	Reads:
!	1. HISTORY + FIELD from DL_POLY
!
!	Writes:
!	1. A file containing the average electrostatic energy of the requested pairs.
!
!  Last change: 14/3/2011
!

PROGRAM INTRAMOLECULAR_COUL	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI, ELECTRON_CHARGE, NAVOGADRO, KE_COULOMB, ANGSTROM
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES,   ONLY : pbc_jumps
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE FORCEFIELD_VARIABLES,   ONLY : atomname, atomcharge
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,       ONLY : TRAJECT_UNWRAP, TRAJECT_JOIN_MOLECULES
   USE LINEAR_ALGEBRA,         ONLY : NORM  
   
   IMPLICIT NONE

   INTEGER, PARAMETER :: max_pairs = 99
   
   CHARACTER ( LEN = LINE_LENGTH) :: traject_file, ff_file, output_file

   INTEGER :: IFRAME, I, J, K, IPAIR, IOSTAT, UNIT, nframes
   INTEGER, ALLOCATABLE, DIMENSION(:)   :: npairs
   INTEGER, ALLOCATABLE, DIMENSION(:,:) :: a1, a2

   REAL ( KIND = DP )               :: energy_unit, dist, coul_energy, total_coul
   REAL ( KIND = DP ), DIMENSION(3) :: r

   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)       :: factor


   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (compulsory)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,*)
   
   
   ! Get system information
   
   CALL FIELD_DLPOLY_READ (ff_file)
   ALLOCATE ( npairs(max_pairs) ); npairs = 0
   ALLOCATE ( a1(max_pairs, numspecies) ); a1 = 0
   ALLOCATE ( a2(max_pairs, numspecies) ); a2 = 0
   ALLOCATE ( factor(max_pairs, numspecies) ); factor = 0.0
       
   
   ! Read trajectory 
      
   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   IF (pbc_jumps) CALL TRAJECT_UNWRAP (nframes, rrr)  
   CALL TRAJECT_JOIN_MOLECULES (nframes, rrr)
   

   ! Open output file
   
    UNIT = NEXT_UNIT ( )	
    OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )


    ! Write information and read intramolecular pairs to compute

   WRITE (OUTPUT, *)
   WRITE (OUTPUT, '("Number of species = ",I3)') numspecies
   DO I = 1, numspecies
      WRITE (OUTPUT, '("Species ",I2,".")') I
      WRITE (OUTPUT, '("No. molecules = ",I4,".")') nummols(I)
      WRITE (OUTPUT, '("No. atoms in molecule = ",I3,".")') numatoms(I)
      WRITE (OUTPUT, '("Atom names:   ",20(A8,1X))') ( atomname(K,I), K = 1, numatoms(I) )
      WRITE (OUTPUT, '("Atom charges: ",20(F8.4,1X))') ( atomcharge(K,I), K = 1, numatoms(I) )
      WRITE (OUTPUT, '("Number of pairs to calculate?")', ADVANCE='NO')
      READ (INPUT, *) npairs(I)
      IF (npairs(I) > max_pairs) STOP 'Increase max_pairs'
      IF (npairs(I) > 0) THEN
         WRITE (OUTPUT, '("Give the atom numbers of a pair and the scaling factor to employ")')
         DO IPAIR = 1, npairs(I)
	    READ (INPUT,*) a1(IPAIR,I), a2(IPAIR,I), factor(IPAIR,I)
	 END DO
      END IF
   END DO 

   ! To convert to kJ/mol 
   
   energy_unit = ELECTRON_CHARGE * ELECTRON_CHARGE / ANGSTROM    ! from e^2/A to C^2/m
   energy_unit = energy_unit * KE_COULOMB                        ! * 1/(4*pi*epsilon_0) --> N*m = J
   energy_unit = energy_unit * NAVOGADRO / 1000.0                ! J --> kJ/mol

   
   ! Compute average intramolecular coulombic energies
      
   species_loop: DO I = 1, numspecies
      IF (npairs(I) > 0) THEN
         WRITE (UNIT,*)
         WRITE (UNIT,'("Intramolecular coulombic energies for species ",I3)') I	
	 total_coul = 0.0
	 DO IPAIR = 1, npairs(I)
	    coul_energy = 0.0
            molecules_loop: DO J = 1, nummols(I)
               frames_loop: DO IFRAME = 1,nframes
	          r = rrr(1:3, a2(IPAIR,I), J, I, IFRAME) - rrr(1:3, a1(IPAIR,I), J, I, IFRAME)
		  dist = NORM(r)
		  coul_energy = coul_energy + energy_unit*atomcharge(a1(IPAIR,I),I)*atomcharge(a2(IPAIR,I),I)*factor(IPAIR,I)/dist
	       END DO frames_loop	
            END DO molecules_loop
	    coul_energy = coul_energy / REAL(nframes)
	    WRITE (UNIT,'("Pair ",I3," - ",I3," , Coulomb Energy (kJ/mol) = ",G12.6)') a1(IPAIR,I), a2(IPAIR,I), coul_energy	
            total_coul = total_coul + coul_energy
	 END DO
	 WRITE (UNIT,'("Total Coulomb Energy (kJ/mol) = ",G12.6)') total_coul
      END IF	    
   END DO species_loop
   
   CLOSE (UNIT)
   
END PROGRAM INTRAMOLECULAR_COUL
