!
! Program to calculate the mean square displacement.
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Writes:
!	1. An ASCII file with 5 columns containing the time (in ps), the m.s.d. in the x, y, and z
!          directions and the total m.s.d.(in A^2).
!
! Note that no weighting is applied.
!
!  Last change: 29/12/2009
!

PROGRAM MEAN_SQUARE_DISPLACEMENT	

   USE DEFINITIONS,          ONLY : LINE_LENGTH, DP
   USE IO_UNITS,             ONLY : INPUT, OUTPUT
   USE FILES,                ONLY : NEXT_UNIT
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES, ONLY : total_atoms, tstep, pbc_jumps, frame_nstep
   USE FORCEFIELD_READ
   USE TRAJECTORY_READWRITE, ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,     ONLY : TRAJECT_UNWRAP
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH)   :: traject_file, ff_file, output_file
   INTEGER                          :: IFRAME, I, J, K, T, T0, IOSTAT, UNIT, nframes, progress
   REAL ( KIND = DP ), DIMENSION(3) :: disp

   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)       :: msd


   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,*)
   
 
   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 


   ! Read trajectory and remove jumps due to PBC 
      
   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   PRINT *,'Finished reading trajectory!'
   IF (pbc_jumps) CALL TRAJECT_UNWRAP (nframes, rrr)  


   ! Initialize histograms
   
   IF ( ALLOCATED (msd) ) DEALLOCATE(msd)
   ALLOCATE ( msd(0:nframes-1,4) ); msd = 0.0


   ! Compute contribution to MSD from each atom
   
   DO T = 0, nframes-1      !time difference
      progress = nframes / 20
      IF (progress < 1) progress = 1
      IF (MOD(T,progress) == 0) PRINT *,'MSD calculation. Completed ',(100*T/nframes),' %'
      DO T0 = 1, nframes-t  !loop over different time origins
         DO I = 1, numspecies
            DO J = 1, nummols(I)
               DO K = 1, numatoms(I) 

	          disp = rrr ( 1:3, K, J, I, T0+T ) - rrr ( 1:3, K, J, I, T0) 
	          msd(T,1:3) = msd(T,1:3) + disp**2
	          msd(T,4) = msd(T,4) + DOT_PRODUCT ( disp, disp )

	       END DO
	    END DO
         END DO 
      END DO 
   END DO 
   
   ! Normalize
   DO IFRAME = 0, nframes-1
      msd(IFRAME,:) = msd(IFRAME,:) / total_atoms / real(nframes-IFRAME)  
   END DO

   
   ! Write output file

   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'Output file for MSD could not be created!'

   ! Write information about analysis
   WRITE ( UNIT, "(A,A40)", IOSTAT = IOSTAT ) "# MSD analysis using trajectory file: ", traject_file
   WRITE ( UNIT, "(A,I8)", IOSTAT = IOSTAT )  "# Number of frames used in analysis = ", nframes
   WRITE ( UNIT, "(A,I8)", IOSTAT = IOSTAT )  "# Number of atoms used in analysis  = ", total_atoms
   WRITE ( UNIT, "(A)", IOSTAT = IOSTAT )     "# "
   WRITE ( UNIT, "(A)", IOSTAT = IOSTAT )     "# t, MSD_x, MSD_y, MSD_z, MSD_total"
   WRITE ( UNIT, "(A)", IOSTAT = IOSTAT )     "# "
   DO IFRAME = 0,nframes-1
      WRITE ( UNIT, "(5(G12.6,2x))", IOSTAT = IOSTAT )  tstep*( frame_nstep(IFRAME+1)-frame_nstep(1) ), msd(IFRAME,:)
   END DO
   
   CLOSE (UNIT)
   
END PROGRAM MEAN_SQUARE_DISPLACEMENT
