!
! Program to calculate the mean square displacement and estimate the
! error bars using the approach of Smith & Gillan, CCP5 newsletter (1996).
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Writes:
!	1. An ASCII file with 3 columns containing the time (in ps), the total m.s.d.(in A^2) and
!          the estimated error from Eqn. (38) in Smith & Gillan's paper.
!
!
!  Last change: 13/06/2014
!

PROGRAM MEAN_SQUARE_DISPLACEMENT	

   USE DEFINITIONS,          ONLY : LINE_LENGTH, DP
   USE IO_UNITS,             ONLY : INPUT, OUTPUT
   USE FILES,                ONLY : NEXT_UNIT
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES, ONLY : total_atoms, tstep, pbc_jumps, frame_nstep
   USE FORCEFIELD_READ
   USE TRAJECTORY_READWRITE, ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,     ONLY : TRAJECT_UNWRAP
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH)   :: traject_file, ff_file, output_file
   INTEGER                          :: IFRAME, I, J, K, T, T0, IOSTAT, UNIT, nframes, max_diff, N0, N0p, progress
   REAL ( KIND = DP ), DIMENSION(3) :: disp
   REAL ( KIND = DP )               :: error1, error2

   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)         :: msd


   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,*)
   
 
   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 


   ! Read trajectory and remove jumps due to PBC 
      
   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   PRINT *,'Finished reading trajectory!'
   IF (pbc_jumps) CALL TRAJECT_UNWRAP (nframes, rrr)  


   ! Initialize histograms: Longest sub-trajectory is M/2
   
   IF ( ALLOCATED (msd) ) DEALLOCATE(msd)
   max_diff = nframes/2
   ALLOCATE ( msd(0:max_diff-1) ); msd = 0.0


   ! Compute contribution to MSD from each atom
   
   DO T = 0, max_diff-1      !time difference
      progress = nframes / 20
      IF (progress < 1) progress = 1
      IF (MOD(T,progress) == 0) PRINT *,'MSD calculation. Completed ',(100*T/max_diff),' %'
      DO T0 = 1, nframes-t  !loop over different time origins
         DO I = 1, numspecies
            DO J = 1, nummols(I)
               DO K = 1, numatoms(I) 

	          disp = rrr ( 1:3, K, J, I, T0+T ) - rrr ( 1:3, K, J, I, T0) 
	          msd(T) = msd(T) + DOT_PRODUCT ( disp, disp )

	       END DO
	    END DO
         END DO 
      END DO 
   END DO 
   
   ! Normalize
   DO IFRAME = 0, max_diff-1
      msd(IFRAME) = msd(IFRAME) / total_atoms / real(nframes-IFRAME)  
   END DO

   
   ! Write output file

   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'Output file for MSD could not be created!'

   ! Write information about analysis
   WRITE ( UNIT, "(A,A40)", IOSTAT = IOSTAT ) "# MSD analysis using trajectory file: ", traject_file
   WRITE ( UNIT, "(A)", IOSTAT = IOSTAT )     "# Computing error with following parameters: "
   WRITE ( UNIT, "(A,I8)", IOSTAT = IOSTAT )  "# M (number of data points) = ", nframes
   WRITE ( UNIT, "(A,I8)", IOSTAT = IOSTAT )  "# N (number of atoms)  = ", total_atoms
   WRITE ( UNIT, "(A)", IOSTAT = IOSTAT )     "# "
   WRITE ( UNIT, "(A)", IOSTAT = IOSTAT )     "# t, MSD_total, error"
   WRITE ( UNIT, "(A)", IOSTAT = IOSTAT )     "# "
   
   DO IFRAME = 0,max_diff-1
   
      N0 = nframes/(IFRAME+1)  !max number of independent origins for subtrajectory of length IFRAME
      N0p = nframes - IFRAME   !number of origins
      error1 = msd(IFRAME) * SQRT(IFRAME*(4.0*N0p-IFRAME)/total_atoms) / (3.0*N0)
      error2 = msd(IFRAME) * SQRT(2.0*IFRAME/(3.0*total_atoms*nframes))
   
      WRITE ( UNIT, "(4(G12.6,2x))", IOSTAT = IOSTAT )  tstep*( frame_nstep(IFRAME+1)-frame_nstep(1) ), msd(IFRAME), error1, error2
   
   END DO
   
   CLOSE (UNIT)
   
END PROGRAM MEAN_SQUARE_DISPLACEMENT
