!
! Program to calculate the mean square displacement.
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Writes:
!	1. A series of ASCII files (one per atom) with 5 columns containing 
!          the time (in ps), the m.s.d. in the x, y, and z
!          directions and the total m.s.d.(in A^2).
!
! Note that no weighting is applied.
!
!  Last change: 19/03/2014
!

PROGRAM MEAN_SQUARE_DISPLACEMENT_INDIVIDUAL	

   USE DEFINITIONS,          ONLY : LINE_LENGTH, DP
   USE IO_UNITS,             ONLY : INPUT, OUTPUT
   USE FILES,                ONLY : NEXT_UNIT
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES, ONLY : tstep, pbc_jumps, frame_nstep
   USE FORCEFIELD_READ
   USE TRAJECTORY_READWRITE, ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,     ONLY : TRAJECT_UNWRAP
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH)   :: traject_file, ff_file, output_file
   CHARACTER ( LEN = 4 )            :: index_name
   INTEGER                          :: IFRAME, I, J, K, T, T0, IOSTAT, UNIT, UNIT2, nframes, progress, index, inorm
   REAL ( KIND = DP ), DIMENSION(4) :: average_displacement
   REAL ( KIND = DP ), DIMENSION(3) :: disp

   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)       :: msd


   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,*)
   
 
   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 


   ! Read trajectory and remove jumps due to PBC 
      
   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   PRINT *,'Finished reading trajectory!'
   IF (pbc_jumps) CALL TRAJECT_UNWRAP (nframes, rrr)  


   ! Initialize histograms
   
   IF ( ALLOCATED (msd) ) DEALLOCATE(msd)
   ALLOCATE ( msd(0:nframes-1,4) )

   ! Open output file

   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'Output file for MSD could not be created!'
   WRITE ( UNIT, "(A,A40)", IOSTAT = IOSTAT ) "# Index - Average displacement "

   ! Compute MSD for each atom
   
   index = 0
   DO I = 1, numspecies
      DO J = 1, nummols(I)
         DO K = 1, numatoms(I) 
	 
	    index = index + 1
	    WRITE (index_name,"(I4.4)") index
            UNIT2 = NEXT_UNIT ( )	
            OPEN ( UNIT2, FILE = 'msd.'//index_name, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )

            msd = 0.0
	    
            DO T = 0, nframes-1      !time difference
               progress = nframes / 20
               IF (MOD(T,progress) == 0) PRINT *,'MSD calculation. Completed ',(100*T/nframes),' %'
               DO T0 = 1, nframes-t  !loop over different time origins
	          disp = rrr ( 1:3, K, J, I, T0+T ) - rrr ( 1:3, K, J, I, T0) 
	          msd(T,1:3) = msd(T,1:3) + disp**2
	          msd(T,4) = msd(T,4) + DOT_PRODUCT ( disp, disp )
	       END DO
	    END DO   
	    	  
            ! Normalize and compute average displacement during 2nd half of trajectory
            average_displacement = 0.
	    inorm = 0
            DO IFRAME = 0, nframes-1
               msd(IFRAME,:) = msd(IFRAME,:) / real(nframes-IFRAME)  
	       IF (IFRAME > nframes/2) THEN
	          average_displacement = average_displacement + msd(IFRAME,:)
		  inorm = inorm + 1
	       END IF	  
            END DO

            ! Write msd
            WRITE ( UNIT2, "(A)", IOSTAT = IOSTAT )     "# t, MSD_x, MSD_y, MSD_z, MSD_total"
            DO IFRAME = 0,nframes-1
               WRITE ( UNIT2, "(5(G12.6,2x))", IOSTAT = IOSTAT )  tstep*( frame_nstep(IFRAME+1)-frame_nstep(1) ), msd(IFRAME,:)
            END DO
	    CLOSE (UNIT2)
	    
	    ! Write average displacement
	    WRITE ( UNIT, "(I4,3x,4(F12.6,2x))", IOSTAT = IOSTAT ) index, average_displacement / real(inorm)

	 END DO
      END DO
   END DO   	 

   
END PROGRAM MEAN_SQUARE_DISPLACEMENT_INDIVIDUAL
