!===============================================================================
! This file is part of MG_DLP_analysis - a set of programs to perform
! different analysis on Molecular Dynamics simulations, mainly intended
! to handle trajectories generated by DL_POLY (HISTORY format).
!
! Copyright (C) 2021 Miguel A. Gonzalez
!
!    MG_DLP_analysis is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    MG_DLP_analysis is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with MG_DLP_analysis.  If not, see <https://www.gnu.org/licenses/>.
!
!===============================================================================
!
! Module containing different utils to determine molecular orientation. 
!
! Subroutines:
!
!      DEFINE_MOLECULAR_ORIENTATION_GEN (molecule, weight, triade, mol_center, mol_axes)
!      DEFINE_ORIENTATION_IMIDAZOLIUM_1 (molecule, weight, triade, mol_center, mol_axes)
!
! Main variables:
!
!       molecule (1:3=(X,Y,Z), natoms) : atomic positions (IN)
!       weight (natoms) : weight given to each atom to calculate center (IN)
!       triade (3) : three integers giving the numbers of three atoms to define first axes and plane of molecule (IN)
!       mol_center (1:3=(X,Y,Z)) : position of molecular center     
!       mol_axes (1:3=(X,Y,Z), 1:3=(axis 1, axis 2, axis 3)) : three vectors defining the orientation axes     
!
!  Last change: 26/12/2009
!
!===============================================================================

MODULE ORIENTATION_UTILS

USE DEFINITIONS,    ONLY : ATOM_NAME_LENGTH, DP
USE LINEAR_ALGEBRA, ONLY : NORMALIZE, CROSS_PRODUCT

IMPLICIT NONE
PUBLIC

CONTAINS

   SUBROUTINE DEFINE_MOLECULAR_ORIENTATION_GEN (molecule, weight, triade, mol_center, mol_axes)

   REAL ( KIND = DP ), INTENT(IN), ALLOCATABLE, DIMENSION(:,:) :: molecule
   REAL ( KIND = DP ), INTENT(IN), ALLOCATABLE, DIMENSION(:)   :: weight
   INTEGER, INTENT(IN), DIMENSION(3)                           :: triade
   REAL ( KIND = DP ), INTENT(OUT), DIMENSION(3)               :: mol_center
   REAL ( KIND = DP ), INTENT(OUT), DIMENSION(3,3)             :: mol_axes
   
   INTEGER                          :: I, natoms
   REAL ( KIND = DP )               :: total_weight
   REAL ( KIND = DP ), DIMENSION(3) :: tmp_axis
   
   
   ! Check input
   natoms = SIZE(molecule,2)
   IF ( natoms /= SIZE(weight) ) STOP 'DEFINE_MOLECULAR_ORIENTATION_GEN: Error!!! Different number of atoms and weights.'
   IF ( MAXVAL(triade) > natoms ) STOP 'DEFINE_MOLECULAR_ORIENTATION_GEN: Error!!! Atoms chosen > natoms.'
   
   ! Define center
   
   mol_center = 0; total_weight = 0.0
   DO I = 1, natoms
      total_weight = total_weight + weight(I)
   END DO
   IF (total_weight <= 0) STOP 'DEFINE_MOLECULAR_ORIENTATION_GEN: Error!!! Total weight <= 0'
   DO I = 1, natoms
      mol_center(1:3) = mol_center(1:3) + weight(I) * molecule(1:3,I)
   END DO 
   mol_center = mol_center / total_weight

  ! Define first axis in the direction atom1-->atom2
       
   mol_axes(1:3,1) = molecule(1:3,triade(2)) - molecule(1:3,triade(1))       
   mol_axes(1:3,1) = NORMALIZE( mol_axes(1:3,1) )
       
   ! Define temporary axis in the direction atom1-->atom3
         
   tmp_axis(1:3) = molecule(1:3,triade(3)) - molecule(1:3,triade(1))       
   tmp_axis(1:3) = NORMALIZE( tmp_axis(1:3) )
              
   ! Define third axis perpendicular to previous axes
       
   mol_axes(1:3,3) = CROSS_PRODUCT( mol_axes(1:3,1), tmp_axis(1:3) )
   mol_axes(1:3,3) = NORMALIZE( mol_axes(1:3,3) )
              
   ! Define second axis from axes one and three
       
   mol_axes(1:3,2) = CROSS_PRODUCT( mol_axes(1:3,3), mol_axes(1:3,1) )
   mol_axes(1:3,2) = NORMALIZE( mol_axes(1:3,2) )

   
   END SUBROUTINE DEFINE_MOLECULAR_ORIENTATION_GEN


   SUBROUTINE DEFINE_ORIENTATION_IMIDAZOLIUM_1 (molecule, triade, mol_center, mol_axes)

   REAL ( KIND = DP ), INTENT(IN), ALLOCATABLE, DIMENSION(:,:) :: molecule
   INTEGER, INTENT(IN), DIMENSION(3)                           :: triade
   REAL ( KIND = DP ), INTENT(OUT), DIMENSION(3)               :: mol_center
   REAL ( KIND = DP ), INTENT(OUT), DIMENSION(3,3)             :: mol_axes
   
   INTEGER                          :: natoms
   REAL ( KIND = DP )               :: tmp_mod, tmp_cos, tmp_dist
   REAL ( KIND = DP ), DIMENSION(3) :: tmp_axis
   
   
   ! Check input
   natoms = SIZE(molecule,2)
   IF ( MAXVAL(triade) > natoms ) STOP 'DEFINE_ORIENTATION_IMIDAZOLIUM_1: Error!!! Atoms chosen > natoms.'
   

  ! Define first axis in the direction atom1-->atom2
       
   mol_axes(1:3,1) = molecule(1:3,triade(2)) - molecule(1:3,triade(1))       
   mol_axes(1:3,1) = NORMALIZE( mol_axes(1:3,1) )
       
   ! Define center as the point of proyection of atom 3 into the first axis
       
   tmp_axis(1:3) = molecule(1:3,triade(3)) - molecule(1:3,triade(1))  !axis atom3-atom2
   tmp_mod = SQRT( DOT_PRODUCT( tmp_axis, tmp_axis ) )                !distance atom3-atom2
   tmp_cos = DOT_PRODUCT( mol_axes(1:3,1), tmp_axis ) / tmp_mod       !angle atom3-atom2-atom1
   tmp_dist = tmp_mod * tmp_cos                                       !proyection atom3-atom2 line into first axis
   mol_center(1:3) = molecule(1:3,triade(1)) + mol_axes(1:3,1) * tmp_dist
 
   ! Define second axis
         
   mol_axes(1:3,2) = molecule(1:3,triade(3)) - mol_center(1:3)        
   mol_axes(1:3,2) = NORMALIZE( mol_axes(1:3,2) )
              
   ! Define third axis perpendicular to previous axes
       
   mol_axes(1:3,3) = CROSS_PRODUCT( mol_axes(1:3,1), mol_axes(1:3,2) )
   mol_axes(1:3,3) = NORMALIZE( mol_axes(1:3,3) )

   
   END SUBROUTINE DEFINE_ORIENTATION_IMIDAZOLIUM_1


END MODULE ORIENTATION_UTILS   

   
