!
! Program to calculate orientational correlations around a selected type of molecule as a function of r.
! Similar to orientational_order, but allows to work with different species.
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Options to compute molecular centers and reference axes:
!	1. General 
!	2. Imidazolium
!
!	Writes:
!	1. One file containing the ODF as a function of r
!       2. A series of files named 'angle_distribution_spec_XX' (one for each type of molecule) containing 
!          the population distribution vs theta for different distance ranges.
!
!
!  Last change: 22/6/2011
!

PROGRAM ORIENTATIONAL_DISTRIBUTIONS	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES,   ONLY : frame_cell, average_cell_volume, pbc_jumps
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,       ONLY : TRAJECT_UNWRAP, TRAJECT_JOIN_MOLECULES, TRAJECT_CART_2_FRACT
   USE ORIENTATION_UTILS,      ONLY : DEFINE_MOLECULAR_ORIENTATION_GEN
   USE LINEAR_ALGEBRA,         ONLY : NORMALIZE
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH)     :: traject_file, ff_file, output_file
   CHARACTER ( LEN = 2)               :: name_add
   INTEGER                            :: IFRAME, I, J, K, I1, I2, J1, J2, UNIT, IOSTAT
   INTEGER                            :: nframes, selected_species, progress, nvector
   INTEGER                            :: number_points, number_r, bin, binth, binr
   INTEGER, DIMENSION(3)              :: triademol
   REAL ( KIND = DP ), DIMENSION(3)   :: mol_center, sdist
   REAL ( KIND = DP ), DIMENSION(3,3) :: mol_axes


   REAL ( KIND = DP ) :: rmax, dr, dtheta, rx, ry, rz, r, dot, theta, total_weight, sinth
  
   INTEGER, ALLOCATABLE, DIMENSION(:,:)                  :: triade, norm
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)         :: weightmol, out, rnorm
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)       :: molecule, weight, ocf
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr, sss, traj_mol_center, traj_mol_axes
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:)     :: hist_thdist
						      

   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file for orientational correlation function?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,'("Maximum distance to compute orientational correlation function?")')
   READ (INPUT,*) rmax
   WRITE (OUTPUT,'("Delta r value to use in o.c.f. grid?")')
   READ (INPUT,*) dr
   WRITE (OUTPUT,'("Delta th value to use in (r,angle) grid?")')
   READ (INPUT,*) dtheta
   WRITE (OUTPUT,'("Vector to use in orientation calculation (1, 2 or 3)?")')
   READ (INPUT,*) nvector
   IF (nvector < 1) THEN
      PRINT *, 'Using nvector = 1'
      nvector = 1
   END IF
   IF (nvector > 3) THEN
      PRINT *, 'Using nvector = 3'
      nvector = 3
   END IF
   WRITE (OUTPUT,*)  

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) )  DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
      WRITE ( OUTPUT, '("System information:")' ) 
      WRITE ( OUTPUT, '("Number of different species = ",I3)') numspecies
      DO I = 1,numspecies
         WRITE ( OUTPUT, "('Molecule ',I3,' contains ',I3,' atoms')" ) I,numatoms(I)
      END DO
      WRITE ( OUTPUT, * )
   END IF 
      
   ! Give desired weights to calculate the COM of each molecule & 3 atoms to define the molecular orientation
   ! Axis 1: Along vector atom1-->atom2
   ! Axis 2: Perpendicular to 1 and in plane formed by the three atoms
   ! Axis 3: Axis 1 x Axis 2
   
   ALLOCATE ( weight ( maxval(numatoms), numspecies ) );   weight = 0.0
   ALLOCATE ( weightmol ( maxval(numatoms) ) );   weightmol = 0.0
   ALLOCATE ( triade ( 3, numspecies ) );   triade = 0
   ALLOCATE ( molecule ( 3, maxval(numatoms) ) ); molecule = 0.0
   DO I = 1,numspecies
      WRITE (OUTPUT,'("Molecule type ",I3,": Number of atoms = ",I4)') I,numatoms(I)
      WRITE (OUTPUT,'("Give weights of all atoms ")')
      READ ( INPUT, * ) ( weight(J,I), J=1,numatoms(I))
      IF (numatoms(I) >= 3) THEN
         WRITE (OUTPUT,'("Give numbers of 3 atoms to define the molecular orientation")')
	 READ ( INPUT, * ) (triade(J,I), J=1,3)
      END IF
   END DO
   WRITE (OUTPUT,'("Number of the selected species? ")')
   READ (INPUT,*) selected_species
   IF (selected_species > numspecies) STOP 'Error! Not so many species in trajectory'

   
   ! Read trajectory and unwrap and re-join molecules

   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   PRINT *,'Finished reading trajectory!'
   IF (pbc_jumps) CALL TRAJECT_UNWRAP (nframes, rrr)  
   CALL TRAJECT_JOIN_MOLECULES (nframes, rrr)


   ! Now calculate trajectory of molecular centers and axes
     
   IF (ALLOCATED(traj_mol_center)) DEALLOCATE(traj_mol_center)
   IF (ALLOCATED(traj_mol_axes))   DEALLOCATE(traj_mol_axes)
   ALLOCATE ( traj_mol_center ( 3, 1, maxval(nummols), numspecies, nframes ) ); traj_mol_center = 0.0 
   ALLOCATE ( traj_mol_axes ( 3, 3, maxval(nummols), numspecies, nframes ) );   traj_mol_axes = 0.0
   
   DO IFRAME = 1, nframes

      DO I = 1, numspecies

         DO J = 1, nummols(I)
   
            molecule(1:3,:) = rrr ( 1:3, :, J, I, IFRAME)
            weightmol = weight(:,I)
            triademol = triade(:,I)

            IF (numatoms(I) >= 3) THEN

	       CALL DEFINE_MOLECULAR_ORIENTATION_GEN (molecule, weightmol, triademol, mol_center, mol_axes)

            ELSE

               mol_center = 0.0; total_weight = 0.0
               DO K = 1, numatoms(I)
                  mol_center(1:3) = mol_center(1:3) + weight(K,I) * molecule(1:3,K)
                  total_weight = total_weight + weight(K,I)
               END DO 
               mol_center = mol_center / total_weight
               IF (numatoms(I) == 2) THEN
                  ! Define first axis as bond between both atoms
                  mol_axes(1:3,1) = molecule(1:3,2) - molecule(1:3,1)	 
                  mol_axes(1:3,1) = NORMALIZE( mol_axes(1:3,1) )
               ELSE 
                  ! Use the cartesian axes
                  mol_axes(1:3,1) =  (/ 1.0d0, 0.0d0, 0.0d0 /)
               END IF
               mol_axes(1:3,2) =  (/ 0.0d0, 1.0d0, 0.0d0 /)
               mol_axes(1:3,3) =  (/ 0.0d0, 0.0d0, 1.0d0 /)

            END IF
            
	    traj_mol_center(1:3,1,J,I,IFRAME) = mol_center(1:3)
	    traj_mol_axes(1:3,1:3,J,I,IFRAME) = mol_axes(1:3,1:3)

         END DO	 

      END DO
      
   END DO
   !print *,'traj_mol_center & traj_mol_axes computed'

   ! Calculate trajectory of centers in fractional units
   
   CALL TRAJECT_CART_2_FRACT (nframes, traj_mol_center, sss)
   
   ! Initialize histograms
   
   number_points = int(rmax/dr) + 1
   IF ( ALLOCATED(ocf) ) DEALLOCATE(ocf)
   ALLOCATE ( ocf ( numspecies, number_points ) ); ocf = 0.0
   IF ( ALLOCATED(norm) ) DEALLOCATE(norm)
   ALLOCATE ( norm ( numspecies, number_points ) ); norm = 0
   
   number_r = INT(rmax) + 1
   IF (number_r > 20) PRINT *,'Output file will contain more than one row per theta value. Number of distances = ',number_r
   IF ( ALLOCATED(hist_thdist) )      DEALLOCATE(hist_thdist)
   ALLOCATE ( hist_thdist ( numspecies, 181, number_r ) );    hist_thdist = 0.
   

   ! Compute histograms
   
   frames_loop: DO IFRAME = 1,nframes

      progress = nframes / 20
      IF (progress < 1) progress = 1
      IF (MOD(IFRAME,progress) == 0) PRINT *,'ODF calculation. Completed ',(100*IFRAME/nframes),' %'
   
      mol1_species_loop: DO I1 = selected_species,selected_species
      mol1_molecules_loop: DO J1 = 1,nummols(I1)

         mol2_species_loop: DO I2 = 1,numspecies
         mol2_molecules_loop: DO J2 = 1,nummols(I2)
		        
	    IF ( (I1 == I2) .AND. (J1 == J2) ) CYCLE mol2_molecules_loop
			
	    ! Obtain distance between molecular centers
			
	    sdist= sss(1:3,1,J1,I1,IFRAME) - sss(1:3,1,J2,I2,IFRAME)
	    sdist = sdist - ANINT(sdist)
            rx = sdist(1)*frame_cell(1,1,IFRAME) + sdist(2)*frame_cell(2,1,IFRAME) + sdist(3)*frame_cell(3,1,IFRAME)
            ry = sdist(1)*frame_cell(1,2,IFRAME) + sdist(2)*frame_cell(2,2,IFRAME) + sdist(3)*frame_cell(3,2,IFRAME)
            rz = sdist(1)*frame_cell(1,3,IFRAME) + sdist(2)*frame_cell(2,3,IFRAME) + sdist(3)*frame_cell(3,3,IFRAME)
            r = SQRT( rx*rx + ry*ry + rz*rz )
		     
	    IF (r.le.rmax) THEN
	    
               bin = INT(r/dr) + 1
               norm(I2,bin) = norm(I2,bin) + 1
               dot = DOT_PRODUCT ( traj_mol_axes(1:3,nvector,J1,I1,IFRAME), &
                                   traj_mol_axes(1:3,nvector,J2,I2,IFRAME) )
               ocf(I2,bin) = ocf(I2,bin) + 0.5*(3.0*dot*dot-1.0)
	       theta = ACOS(dot)*180.0/pi
	       binr = INT(r) + 1
	       binth = INT(theta/1.0) + 1
  	       sinth = SIN(ACOS(dot))
	       IF (sinth < 1.0e-3) sinth = 1.0e-3
	       hist_thdist(I2,binth,binr) = hist_thdist(I2,binth,binr) + (1./sinth) !weight each point by 1/sin(theta) to take into account solid angle

            ENDIF
			
	 END DO mol2_molecules_loop	
	 END DO mol2_species_loop

      END DO mol1_molecules_loop	
      END DO mol1_species_loop	

   END DO frames_loop
   
    
   ! Write results

   
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT,'("# Orientational correlation functions")')
   WRITE (UNIT,'("#",3x,"r/A",3x,"OCF for each species")')
 
   IF ( ALLOCATED(out) ) DEALLOCATE(out)
   ALLOCATE ( out ( numspecies ) ); out = 0.0
   IF ( ALLOCATED(rnorm) ) DEALLOCATE(rnorm)
   ALLOCATE ( rnorm ( number_r ) ); rnorm = 0.0


   DO I = 1, number_points
      r = REAL(I-1)*dr + 0.5*dr
      DO J = 1, numspecies
      	 IF (norm(J,I) > 0) THEN
            out(J) = ocf(J,I) / REAL(norm(J,I))
         ELSE
            out(J) = 0.0d0
         END IF
      END DO      
      WRITE (UNIT,"(1x,f7.3,10(2x,f10.6))") r, (out(J),J=1,numspecies)
   END DO
   CLOSE (UNIT)
   
   DO J = 1, numspecies
      WRITE (name_add,"(I2.2)") J
      OPEN ( UNIT, FILE = 'angle_distribution_spec_'//name_add, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
      WRITE (UNIT,'("# Histogram of distribution of distances vs angle")')
      WRITE (UNIT,'("# Divided by number of frames and number of molecules.")')
      WRITE (UNIT,'("# Solid angle taken into account by weighting each point by 1/sin(theta).")')
      WRITE (UNIT,'("# Theta --- Distances 0-Rmax with dr = 1.0 A")') 
      !Normalisation factor so that for each distance the integral over theta is 1
      DO K = 1, number_r
         rnorm(K) = SUM( hist_thdist(J,:,K) )
         IF (rnorm(K) > 0.0) hist_thdist(J,:,K) = hist_thdist(J,:,K)/rnorm(K)
      END DO
      DO I = 1,180
         r = REAL(I-1) + 0.5
         WRITE (UNIT,"(1x,f7.3,1x,40(1x,F6.4))") r, (hist_thdist(J,I,K), K=1,number_r)
      END DO
      CLOSE (UNIT)
   END DO



END PROGRAM ORIENTATIONAL_DISTRIBUTIONS
