!
! Program to calculate orientational correlations as a function of r.
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Options to compute molecular centers and reference axes:
!	1. General 
!	2. Imidazolium
!
!	Writes:
!	1. One file with 5 columns containing r and the r.d.f. of the molecular centers 
!          and the ODF(r) for the three axes, as well as the sum of the modulus for the third axes.
!       2. Four files containing the population distribution vs r for different angle ranges:
!          full_ --> total numbers of pairs
!          norm_ --> normalized by number of pairs found at each r
!          wsin_ --> divided by solid angle factor sin(theta)
!          normwsin_ --> normalized by number of pairs and sin(theta)
!       3. Four files containing the population distribution vs theta for different distance ranges:
!          full_ --> total numbers of pairs
!          norm_ --> normalized by number of pairs found at each r
!          wsin_ --> divided by solid angle factor sin(theta)
!          normwsin_ --> normalized by number of pairs and sin(theta)
!
! Note that the program can read any trajectory, but it will treat only a single molecular type,
! so one single species needs to be chosen. Requires extension if correlations between two different
! types of molecule are needed. 
!
! Note that the population distributions are written using F5.3, so the statistics are poor for low
! numbers. Selecting 'Y' to the question of normalizing maximum to 9.999 may help a little bit.
!
!  Last change: 5/1/2009
!

PROGRAM ORIENTATIONAL_ORDER	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES,   ONLY : frame_cell, average_cell_volume, pbc_jumps
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,       ONLY : TRAJECT_UNWRAP, TRAJECT_JOIN_MOLECULES, TRAJECT_CART_2_FRACT
   USE ORIENTATION_UTILS,      ONLY : DEFINE_MOLECULAR_ORIENTATION_GEN, DEFINE_ORIENTATION_IMIDAZOLIUM_1
   
   IMPLICIT NONE

   LOGICAL                            :: normalize_maximum = .TRUE. 
   CHARACTER ( LEN = LINE_LENGTH)     :: traject_file, ff_file, output_file_1, output_file_2, output_file_3
   CHARACTER ( LEN = 1)               :: answer
   INTEGER                            :: IFRAME, I, J, J1, J2, UNIT1, UNIT2, UNIT3, UNIT4, UNIT5, UNIT6, UNIT7, UNIT8, UNIT9, IOSTAT
   INTEGER                            :: nframes, option, selected_species, tmp_nummols, progress
   INTEGER                            :: number_points, number_th, number_r, bin, binth, binr
   INTEGER, DIMENSION(3)              :: triade
   REAL ( KIND = DP ), DIMENSION(3)   :: mol_center, sdist, tmp_axis
   REAL ( KIND = DP ), DIMENSION(3,3) :: mol_axes


   REAL ( KIND = DP ) :: rmax, dr, dtheta, rx, ry, rz, r, dot, xa, xb, xx, shell_volume, n_ideal, &
                         rdf, theta3, sinth, out1, out2, out3, out4, norm_value_1, norm_value_2, &
			 norm_value_3, norm_value_4, norm_value_5, norm_value_6, norm_value_7, norm_value_8
  
   INTEGER, ALLOCATABLE, DIMENSION(:)                    :: norm1, norm2, hist
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)       :: molecule
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)         :: weight
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr, sss, traj_mol_center, traj_mol_axes
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)       :: ocf, hist_distth, hist_thdist, hist_distth_sa, hist_thdist_sa, &
                                                            hist_distth_2, hist_thdist_2, hist_distth_sa_2, hist_thdist_sa_2
						      

   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file for orientational correlation function?")')
   READ (INPUT,'(A)') output_file_1
   WRITE (OUTPUT,'("Maximum distance to compute orientational correlation function?")')
   READ (INPUT,*) rmax
   WRITE (OUTPUT,'("Delta r value to use in o.c.f. grid?")')
   READ (INPUT,*) dr
   WRITE (OUTPUT,'("Name of output file for (r,angle) histogram?")')
   READ (INPUT,'(A)') output_file_2
   WRITE (OUTPUT,'("Delta th value to use in (r,angle) grid?")')
   READ (INPUT,*) dtheta
   WRITE (OUTPUT,'("Name of output file for (angle,r) histogram?")')
   READ (INPUT,'(A)') output_file_3
   WRITE (OUTPUT,'("Normalize so that maximum of each distribution = 9.999?")')
   READ (INPUT,'(A)') answer
   IF ( (answer /= "Y") .AND. (answer /= "y") ) normalize_maximum = .FALSE.
   WRITE (OUTPUT,*)

   ! Select mode to calculate molecular orientation

   WRITE (OUTPUT,'("Choose option to determine molecular center and axes:")')
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("1. General: Choose one type of molecule and give atom weights and three atom numbers.")')
   WRITE (OUTPUT,'("            Center: COM of molecule computed with the given weights")')
   WRITE (OUTPUT,'("            Axis 1: Along vector atom1-->atom2")')
   WRITE (OUTPUT,'("            Axis 2: Perpendicular to 1 and in plane formed by the three atoms")')
   WRITE (OUTPUT,'("            Axis 3: Axis 1 x Axis 2")')
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("2. Imidazolium 1: Give corresponding species number ")')
   WRITE (OUTPUT,'("                  and the atom numbers of N3(atom 1), N1(atom 2), and C2 (atom 3)")')
   WRITE (OUTPUT,'("                  Center: Along N3-N1 vector, at bisector with C2")')
   WRITE (OUTPUT,'("                  Axis 1: Center-N1")')
   WRITE (OUTPUT,'("                  Axis 2: Center-C2")')
   WRITE (OUTPUT,'("                  Axis 3: Axis 1 x Axis 2")')
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("Selected option?")')
   READ (INPUT,*) option
   WRITE (OUTPUT,*)
  

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) )  DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
      WRITE ( OUTPUT, '("System information:")' ) 
      WRITE ( OUTPUT, '("Number of different species = ",I3)') numspecies
      DO I = 1,numspecies
         WRITE ( OUTPUT, "('Molecule ',I3,' contains ',I3,' atoms')" ) I,numatoms(I)
      END DO
      WRITE ( OUTPUT, * )
   END IF 
      

   ! Selection
   
   SELECT CASE (option)
   
   CASE (1)

      WRITE (OUTPUT,'("Number of the chosen species in trajectory? ")')
      READ (INPUT,*) selected_species
      IF (selected_species > numspecies) STOP 'Error! Not so many species in trajectory'
      IF (ALLOCATED(molecule)) DEALLOCATE(molecule)
      IF (ALLOCATED(weight))   DEALLOCATE(weight)
      ALLOCATE ( molecule ( 3, numatoms(selected_species) ) ); molecule = 0.0
      ALLOCATE ( weight ( numatoms(selected_species) ) );   weight = 0.0
      WRITE (OUTPUT,'("Number of atoms in selected type = ",I4)') numatoms(selected_species)
      WRITE (OUTPUT,'("Give weight for each atom ")')
      DO I = 1,numatoms(selected_species)
         WRITE ( OUTPUT, "('Atom',I3,': Weight?')" ) I
	 READ ( INPUT, * ) weight(I)
      END DO
      WRITE (OUTPUT,'("Now give numbers in molecule of selected triplet of atoms ")')
      DO I = 1,3
         WRITE ( OUTPUT, "('Atom',I3,'?')" ) I
	 READ ( INPUT, * ) triade(I)
      END DO
   
   CASE (2)

      WRITE (OUTPUT,'("Number of the chosen species in trajectory? ")')
      READ (INPUT,*) selected_species
      IF (selected_species > numspecies) STOP 'Error! Not so many species in trajectory'
      IF (ALLOCATED(molecule)) DEALLOCATE(molecule)
      ALLOCATE ( molecule ( 3, numatoms(selected_species) ) ); molecule = 0.0
      WRITE (OUTPUT,'("Now give the corresponding number indicating the order in the molecule of N3, N1, and C2 (in this order).")')
      DO I = 1,3
         WRITE ( OUTPUT, "('Atom',I3,'?')" ) I
	 READ ( INPUT, * ) triade(I)
      END DO
    
   CASE DEFAULT
      STOP 'Not valid option. Program terminated'
   
   END SELECT
   

   ! Read trajectory and unwrap and re-join molecules

   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   PRINT *,'Finished reading trajectory!'
   IF (pbc_jumps) CALL TRAJECT_UNWRAP (nframes, rrr)  
   CALL TRAJECT_JOIN_MOLECULES (nframes, rrr)


   ! Now calculate trajectory of molecular centers and axes
     
   IF (ALLOCATED(traj_mol_center)) DEALLOCATE(traj_mol_center)
   IF (ALLOCATED(traj_mol_axes))   DEALLOCATE(traj_mol_axes)
   ALLOCATE ( traj_mol_center ( 3, 1, nummols(numspecies), 1, nframes ) ); traj_mol_center = 0.0 
   ALLOCATE ( traj_mol_axes ( 3, 3, nummols(numspecies), 1, nframes ) );   traj_mol_axes = 0.0
   
   DO IFRAME = 1, nframes
   
      DO J = 1, nummols(selected_species)
   
         molecule(1:3,:) = rrr ( 1:3, :, J, selected_species, IFRAME)
	
	 SELECT CASE (option)
	 CASE (1)
	    CALL DEFINE_MOLECULAR_ORIENTATION_GEN (molecule, weight, triade, mol_center, mol_axes)
	 CASE (2)   
	    CALL DEFINE_ORIENTATION_IMIDAZOLIUM_1 (molecule, triade, mol_center, mol_axes)
         CASE DEFAULT
            STOP 'Not valid option. Program terminated'
         END SELECT
	 
	 traj_mol_center(1:3,1,J,1,IFRAME) = mol_center(1:3)
	 traj_mol_axes(1:3,1:3,J,1,IFRAME) = mol_axes(1:3,1:3)
	 
      END DO
      
   END DO

   ! Redefine general variables and calculate trajectory of centers in fractional units
   
   tmp_nummols = nummols(selected_species)
   numspecies = 1
   DEALLOCATE (nummols); ALLOCATE (nummols(1)); nummols(1) = tmp_nummols
   DEALLOCATE (numatoms); ALLOCATE (numatoms(1)); numatoms(1) = 1
   CALL TRAJECT_CART_2_FRACT (nframes, traj_mol_center, sss)
   
   ! Initialize histograms
   
   number_points = int(rmax/dr) + 1
   IF ( ALLOCATED(norm1) ) DEALLOCATE(norm1)
   IF ( ALLOCATED(hist) ) DEALLOCATE(hist)
   IF ( ALLOCATED(ocf) ) DEALLOCATE(ocf)
   ALLOCATE ( norm1 (number_points ), hist ( number_points ) ); norm1 = 0; hist = 0
   ALLOCATE ( ocf ( 4, number_points ) );                       ocf = 0.0
   
   number_th = INT(180./dtheta) + 1
   IF (number_th > 20) PRINT *,'Output file will contain more than one row per r value. Number of angles = ',number_th
   IF ( ALLOCATED(hist_distth) )      DEALLOCATE(hist_distth)
   IF ( ALLOCATED(hist_distth_sa) )   DEALLOCATE(hist_distth_sa)
   IF ( ALLOCATED(hist_distth_2) )    DEALLOCATE(hist_distth_2)
   IF ( ALLOCATED(hist_distth_sa_2) ) DEALLOCATE(hist_distth_sa_2)
   ALLOCATE ( hist_distth ( number_points, number_th ) );      hist_distth = 0.
   ALLOCATE ( hist_distth_sa ( number_points, number_th ) );   hist_distth_sa = 0.
   ALLOCATE ( hist_distth_2 ( number_points, number_th ) );    hist_distth_2 = 0.
   ALLOCATE ( hist_distth_sa_2 ( number_points, number_th ) ); hist_distth_sa_2 = 0.
      
   number_r = INT(rmax) + 1
   IF (number_r > 20) PRINT *,'Output file will contain more than one row per theta value. Number of distances = ',number_r
   IF ( ALLOCATED(norm2) )            DEALLOCATE(norm1)
   IF ( ALLOCATED(hist_thdist) )      DEALLOCATE(hist_thdist)
   IF ( ALLOCATED(hist_thdist_sa) )   DEALLOCATE(hist_thdist_sa)
   IF ( ALLOCATED(hist_thdist_2) )    DEALLOCATE(hist_thdist_2)
   IF ( ALLOCATED(hist_thdist_sa_2) ) DEALLOCATE(hist_thdist_sa_2)
   ALLOCATE ( norm2 ( number_r ) ); norm2 = 0
   ALLOCATE ( hist_thdist ( 181, number_r ) );    hist_thdist = 0.
   ALLOCATE ( hist_thdist_sa ( 181, number_r ) ); hist_thdist_sa = 0.
   ALLOCATE ( hist_thdist_2 ( 181, number_r ) );  hist_thdist_2   = 0.
   ALLOCATE ( hist_thdist_sa_2(181, number_r) );  hist_thdist_sa_2(:,:) = 0.
   

   ! Compute histograms
   
   frames_loop: DO IFRAME = 1,nframes

      progress = nframes / 20
      IF (progress < 1) progress = 1
      IF (MOD(IFRAME,progress) == 0) PRINT *,'ODF calculation. Completed ',(100*IFRAME/nframes),' %'

      mol1_molecules_loop: DO J1 = 1,nummols(1)
         mol2_molecules_loop: DO J2 = 1,nummols(1)
		        
	    IF ( J1 == J2 ) CYCLE mol2_molecules_loop
			
	    ! Obtain distance between molecular centers
			
	    sdist= sss(1:3,1,J1,1,IFRAME) - sss(1:3,1,J2,1,IFRAME)
	    sdist = sdist - ANINT(sdist)
            rx = sdist(1)*frame_cell(1,1,IFRAME) + sdist(2)*frame_cell(2,1,IFRAME) + sdist(3)*frame_cell(3,1,IFRAME)
            ry = sdist(1)*frame_cell(1,2,IFRAME) + sdist(2)*frame_cell(2,2,IFRAME) + sdist(3)*frame_cell(3,2,IFRAME)
            rz = sdist(1)*frame_cell(1,3,IFRAME) + sdist(2)*frame_cell(2,3,IFRAME) + sdist(3)*frame_cell(3,3,IFRAME)
            r = SQRT( rx*rx + ry*ry + rz*rz )
		     
	    IF (r.le.rmax) THEN
	    
               bin = INT(r/dr) + 1
               norm1(bin) = norm1(bin) + 1
               hist(bin) = hist(bin) + 1 
               dot = DOT_PRODUCT ( traj_mol_axes(1:3,1,J1,1,IFRAME), traj_mol_axes(1:3,1,J2,1,IFRAME) )
               ocf(1,bin) = ocf(1,bin) + dot
               dot = DOT_PRODUCT ( traj_mol_axes(1:3,2,J1,1,IFRAME), traj_mol_axes(1:3,2,J2,1,IFRAME) )
               ocf(2,bin) = ocf(2,bin) + dot
               dot = DOT_PRODUCT ( traj_mol_axes(1:3,3,J1,1,IFRAME), traj_mol_axes(1:3,3,J2,1,IFRAME) )
               ocf(3,bin) = ocf(3,bin) + dot
               ocf(4,bin) = ocf(4,bin) + abs(dot)
	       
	       theta3 = ACOS(dot)*180.0/pi
	       binth = INT(theta3/dtheta) + 1
	       hist_distth(bin,binth) = hist_distth(bin,binth) + 1. !full histogram
	       
	       !weight each point by 1/sin(theta) to take into account solid angle
	       sinth = SIN(ACOS(dot))
	       IF (sinth < 1.0e-3) sinth = 1.0e-3
	       hist_distth_sa(bin,binth) = hist_distth_sa(bin,binth) + (1./sinth) 
	       
	       binr = INT(r) + 1
	       norm2(binr) = norm2(binr) + 1 
	       binth = INT(theta3/1.0) + 1
	       hist_thdist(binth,binr) = hist_thdist(binth,binr) + 1. !full histogram
	       
	       !weight each point by 1/sin(theta) to take into account solid angle
	       hist_thdist_sa(binth,binr) = hist_thdist_sa(binth,binr) + (1./sinth)

            ENDIF
			
	 END DO mol2_molecules_loop	
      END DO mol1_molecules_loop	

   END DO frames_loop
   
   
   !Normalize histograms by number of frames and number of molecules
   
   hist_distth = hist_distth / REAL(nframes) / REAL(nummols(1))
   hist_thdist = hist_thdist / REAL(nframes) / REAL(nummols(1))
   hist_distth_sa = hist_distth_sa / REAL(nframes) / REAL(nummols(1))
   hist_thdist_sa = hist_thdist_sa / REAL(nframes) / REAL(nummols(1))
   
   !Normalize histograms by number of pairs found at each r
   
   DO I = 1,number_points
      IF (norm1(I) > 0) THEN
         DO J = 1,number_th
            hist_distth_2(I,J) = hist_distth(I,J) * REAL(nummols(1)) / REAL(norm1(I))
            hist_distth_sa_2(I,J) = hist_distth_sa(I,J) * REAL(nummols(1)) / REAL(norm1(I))
         END DO
      END IF
   END DO

   DO I = 1,181
      DO J = 1,number_r 
         IF (norm2(J) > 0) THEN 
            hist_thdist_2(I,J) = hist_thdist(I,J) * REAL(nummols(1)) / norm2(J)
            hist_thdist_sa_2(I,J) = hist_thdist_sa(I,J) * REAL(nummols(1)) / norm2(J)
	 END IF
      END DO
   END DO
   
   IF (normalize_maximum) THEN
     
      norm_value_1 = maxval(hist_distth) / 9.999
      hist_distth = hist_distth / norm_value_1
      norm_value_2 = maxval(hist_thdist) / 9.999
      hist_thdist = hist_thdist / norm_value_2
      norm_value_3 = maxval(hist_distth_sa) / 9.999
      hist_distth_sa = hist_distth_sa / norm_value_3
      norm_value_4 = maxval(hist_thdist_sa) / 9.999
      hist_thdist_sa = hist_thdist_sa / norm_value_4
      norm_value_5 = maxval(hist_distth_2) / 9.999
      hist_distth_2 = hist_distth_2 / norm_value_5
      norm_value_6 = maxval(hist_thdist_2) / 9.999
      hist_thdist_2 = hist_thdist_2 / norm_value_6
      norm_value_7 = maxval(hist_distth_sa_2) / 9.999
      hist_distth_sa_2 = hist_distth_sa_2 / norm_value_7
      norm_value_8 = maxval(hist_thdist_sa_2) / 9.999
      hist_thdist_sa_2 = hist_thdist_sa_2 / norm_value_8
   END IF
    
   ! Write results

   !Orientational correlation function vs r
   
   UNIT1 = NEXT_UNIT ( )
   OPEN ( UNIT1, FILE = output_file_1, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT1,'("# Orientational correlation functions")')
   WRITE (UNIT1,'("#",3x,"r/A",3x,"RDF centers",4x,"OCF 1",7x,"OCF 2",6x,"OCF 3",6x,"|OCF 3|")')

   !Histograms of population as a function of r (for different angle values)
   
   !Full
   UNIT2 = NEXT_UNIT ( )
   OPEN ( UNIT2, FILE = 'full_'//output_file_2, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT2,'("# Histogram of distribution of angles vs r")')
   WRITE (UNIT2,'("# Divided by number of frames and number of molecules.")')
   WRITE (UNIT2,'("# Not normalized by solid angle")')
   IF (normalize_maximum) WRITE (UNIT2,'("# Divided by ",G10.4," so maximum is 9.999")') norm_value_1
   WRITE (UNIT2,'("# r/A --- Angles 0-180 with dtheta = ",F6.2)') dtheta
   
   !Weighted by solid angle considered as sin(theta)
   UNIT3 = NEXT_UNIT ( )
   OPEN ( UNIT3, FILE = 'wsin_'//output_file_2, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT3,'("# Histogram of distribution of angles vs r")')
   WRITE (UNIT3,'("# Divided by number of frames and number of molecules.")')
   WRITE (UNIT3,'("# Solid angle taken into account by weighting each point by 1/sin(theta).")')
   IF (normalize_maximum) WRITE (UNIT3,'("# Divided by ",G10.4," so maximum is 9.999")') norm_value_3
   WRITE (UNIT3,'("# r/A --- Angles 0-180 with dtheta = ",F6.2)') dtheta
   
   !Normalized by number of pairs found at each r
   UNIT4 = NEXT_UNIT ( )
   OPEN ( UNIT4, FILE = 'norm_'//output_file_2, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT4,'("# Histogram of distribution of angles vs r")')
   WRITE (UNIT4,'("# Divided by number of frames and number of pairs found at each r.")')
   WRITE (UNIT4,'("# Not normalized by solid angle")')
   IF (normalize_maximum) WRITE (UNIT4,'("# Divided by ",G10.4," so maximum is 9.999")') norm_value_5
   WRITE (UNIT4,'("# r/A --- Angles 0-180 with dtheta = ",F6.2)') dtheta
   
   !Normalized by number of pairs found at each r and weighted by solid angle considered as sin(theta)
   UNIT5 = NEXT_UNIT ( )
   OPEN ( UNIT5, FILE = 'normwsin_'//output_file_2, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT5,'("# Histogram of distribution of angles vs r")')
   WRITE (UNIT5,'("# Divided by number of frames and number of pairs found at each r.")')
   WRITE (UNIT5,'("# Solid angle taken into account by weighting each point by 1/sin(theta)")')
   IF (normalize_maximum) WRITE (UNIT5,'("# Divided by ",G10.4," so maximum is 9.999")') norm_value_7
   WRITE (UNIT5,'("# r/A --- Angles 0-180 with dtheta = ",F6.2)') dtheta


   !Histograms of population as a function of angle (for different r values)
   
   !Full
   UNIT6 = NEXT_UNIT ( )
   OPEN ( UNIT6, FILE = 'full_'//output_file_3, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT6,'("# Histogram of distribution of distances vs angle")')
   WRITE (UNIT6,'("# Divided by number of frames and number of molecules.")')
   WRITE (UNIT6,'("# Not normalized by solid angle.")')
   IF (normalize_maximum) WRITE (UNIT6,'("# Divided by ",G10.4," so maximum is 9.999")') norm_value_2
   WRITE (UNIT6,'("# Theta --- Distances 0-Rmax with dr = 1.0 A")') 
   
   !Weighted by solid angle considered as sin(theta)
   UNIT7 = NEXT_UNIT ( )
   OPEN ( UNIT7, FILE = 'wsin_'//output_file_3, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT7,'("# Histogram of distribution of distances vs angle")')
   WRITE (UNIT7,'("# Divided by number of frames and number of molecules.")')
   WRITE (UNIT7,'("# Solid angle taken into account by weighting each point by 1/sin(theta).")')
   IF (normalize_maximum) WRITE (UNIT7,'("# Divided by ",G10.4," so maximum is 9.999")') norm_value_4
   WRITE (UNIT7,'("# Theta --- Distances 0-Rmax with dr = 1.0 A")') 
   
   !Normalized by number of pairs found at each r
   UNIT8 = NEXT_UNIT ( )
   OPEN ( UNIT8, FILE = 'norm_'//output_file_3, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT8,'("# Histogram of distribution of distances vs angle")')
   WRITE (UNIT8,'("# Divided by number of frames and number of pairs found at each r.")')
   WRITE (UNIT8,'("# Not normalized by solid angle.")')
   IF (normalize_maximum) WRITE (UNIT8,'("# Divided by ",G10.4)') norm_value_6
   WRITE (UNIT8,'("# Theta --- Distances 0-Rmax with dr = 1.0 A")') 
   
   !Normalized by number of pairs found at each r and weighted by solid angle considered as sin(theta)
   UNIT9 = NEXT_UNIT ( )
   OPEN ( UNIT9, FILE = 'normwsin_'//output_file_3, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT9,'("# Histogram of distribution of distances vs angle")')
   WRITE (UNIT9,'("# Divided by number of frames and number of pairs found at each r.")')
   WRITE (UNIT9,'("# Solid angle taken into account by weighting each point by 1/sin(theta)")')
   IF (normalize_maximum) WRITE (UNIT9,'("# Divided by ",G10.4," so maximum is 9.999")') norm_value_8
   WRITE (UNIT9,'("# Theta --- Distances 0-Rmax with dr = 1.0 A")') 
    
   DO I = 1,number_points
   
      xa = REAL(I-1)*dr
      xb = xa + dr
      xx = 0.5 * (xa + xb)
      shell_volume = 4.0 * pi * (xb**3 - xa**3) / 3.0
      n_ideal = REAL( nummols(1) * (nummols(1)-1.0) ) / average_cell_volume
      rdf = REAL(hist(I))/ REAL(nframes) / shell_volume / n_ideal
      
      IF (norm1(I) > 0) THEN
         out1 = ocf(1,I) / REAL(norm1(I))
         out2 = ocf(2,I) / REAL(norm1(I))
         out3 = ocf(3,I) / REAL(norm1(I))
         out4 = ocf(4,I) / REAL(norm1(I))
      ELSE
         out1 = 0.0d0
         out2 = 0.0d0
         out3 = 0.0d0
         out4 = 0.0d0
      END IF      

      WRITE (UNIT1,"(1x,f7.3,5(2x,f10.6))") xx, rdf, out1, out2, out3, out4

      WRITE (UNIT2,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_distth(I,J), J=1,number_th)
      WRITE (UNIT3,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_distth_sa(I,J), J=1,number_th)
      WRITE (UNIT4,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_distth_2(I,J), J=1,number_th)
      WRITE (UNIT5,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_distth_sa_2(I,J), J=1,number_th)


   END DO
   
   DO I = 1,180

      xa = REAL(I-1)*1.0
      xb = xa + 1.0
      xx = 0.5 * (xa + xb)
      WRITE (UNIT6,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_thdist(I,J), J=1,number_r)
      WRITE (UNIT7,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_thdist_sa(I,J), J=1,number_r)
      WRITE (UNIT8,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_thdist_2(I,J), J=1,number_r)
      WRITE (UNIT9,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_thdist_sa_2(I,J), J=1,number_r)

   END DO

   CLOSE (UNIT1)
   CLOSE (UNIT2)
   CLOSE (UNIT3)
   CLOSE (UNIT4)
   CLOSE (UNIT5)
   CLOSE (UNIT6)
   CLOSE (UNIT7)
   CLOSE (UNIT8)
   CLOSE (UNIT9)


END PROGRAM ORIENTATIONAL_ORDER
