!
! Program to calculate the reorientational autocorrelation function.
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Options to compute molecular centers and reference axes:
!	1. General 
!	2. Imidazolium
!
!	Writes:
!	1. One file containing the time autocorrelation function <r(t).r(0)> for the
!          three molecular axes of reference.
!
! Note that the program can read any trajectory, but it will treat only a single molecular type,
! so one single species needs to be chosen. 
!
!  Last change: 29/12/2009
!

PROGRAM REORIENT_CORREL	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES,   ONLY : frame_nstep, tstep, pbc_jumps
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,       ONLY : TRAJECT_UNWRAP, TRAJECT_JOIN_MOLECULES
   USE ORIENTATION_UTILS,      ONLY : DEFINE_MOLECULAR_ORIENTATION_GEN, DEFINE_ORIENTATION_IMIDAZOLIUM_1
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH)     :: traject_file, ff_file, output_file
   INTEGER                            :: IFRAME, I, J, T, T0, UNIT, IOSTAT
   INTEGER                            :: nframes, option, selected_species, progress
   INTEGER, DIMENSION(3)              :: triade
   REAL ( KIND = DP ), DIMENSION(3)   :: mol_center
   REAL ( KIND = DP ), DIMENSION(3,3) :: mol_axes

   REAL ( KIND = DP ) :: dot, cos1, cos2, cos3
   
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)       :: molecule
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)         :: weight
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr, traj_mol_axes
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)       :: oacf
   

   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file for reorientational autocorrelation function?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,*)

   ! Select mode to calculate molecular orientation

   WRITE (OUTPUT,'("Choose option to determine molecular center and axes:")')
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("1. General: Choose one type of molecule and give atom weights and three atom numbers.")')
   WRITE (OUTPUT,'("            Center: COM of molecule computed with the given weights")')
   WRITE (OUTPUT,'("            Axis 1: Along vector atom1-->atom2")')
   WRITE (OUTPUT,'("            Axis 2: Perpendicular to 1 and in plane formed by the three atoms")')
   WRITE (OUTPUT,'("            Axis 3: Axis 1 x Axis 2")')
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("2. Imidazolium 1: Give corresponding species number ")')
   WRITE (OUTPUT,'("                  and the atom numbers of N3(atom 1), N1(atom 2), and C2 (atom 3)")')
   WRITE (OUTPUT,'("                  Center: Along N3-N1 vector, at bisector with C2")')
   WRITE (OUTPUT,'("                  Axis 1: Center-N1")')
   WRITE (OUTPUT,'("                  Axis 2: Center-C2")')
   WRITE (OUTPUT,'("                  Axis 3: Axis 1 x Axis 2")')
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("Selected option?")')
   READ (INPUT,*) option
   WRITE (OUTPUT,*)
  

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) )  DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
      WRITE ( OUTPUT, '("System information:")' ) 
      WRITE ( OUTPUT, '("Number of different species = ",I3)') numspecies
      DO I = 1,numspecies
         WRITE ( OUTPUT, "('Molecule ',I3,' contains ',I3,' atoms')" ) I,numatoms(I)
      END DO
      WRITE ( OUTPUT, * )
   END IF 
      

   ! Selection
   
   SELECT CASE (option)
   
   CASE (1)

      WRITE (OUTPUT,'("Number of the chosen species in trajectory? ")')
      READ (INPUT,*) selected_species
      IF (selected_species > numspecies) STOP 'Error! Not so many species in trajectory'
      IF (ALLOCATED(molecule)) DEALLOCATE(molecule)
      IF (ALLOCATED(weight))   DEALLOCATE(weight)
      ALLOCATE ( molecule ( 3, numatoms(selected_species) ) ); molecule = 0.0
      ALLOCATE ( weight ( numatoms(selected_species) ) );   weight = 0.0
      WRITE (OUTPUT,'("Number of atoms in selected type = ",I4)') numatoms(selected_species)
      WRITE (OUTPUT,'("Give weight for each atom ")')
      DO I = 1,numatoms(selected_species)
         WRITE ( OUTPUT, "('Atom',I3,': Weight?')" ) I
	 READ ( INPUT, * ) weight(I)
      END DO
      WRITE (OUTPUT,'("Now give numbers in molecule of selected triplet of atoms ")')
      DO I = 1,3
         WRITE ( OUTPUT, "('Atom',I3,'?')" ) I
	 READ ( INPUT, * ) triade(I)
      END DO
    
   CASE (2)

      WRITE (OUTPUT,'("Number of the chosen species in trajectory? ")')
      READ (INPUT,*) selected_species
      IF (selected_species > numspecies) STOP 'Error! Not so many species in trajectory'
      IF (ALLOCATED(molecule)) DEALLOCATE(molecule)
      ALLOCATE ( molecule ( 3, numatoms(selected_species) ) ); molecule = 0.0
      WRITE (OUTPUT,'("Now give the corresponding number indicating the order in the molecule of N3, N1, and C2 (in this order).")')
      DO I = 1,3
         WRITE ( OUTPUT, "('Atom',I3,'?')" ) I
	 READ ( INPUT, * ) triade(I)
      END DO
    
   CASE DEFAULT
      STOP 'Not valid option. Program terminated'
   
   END SELECT
   

   ! Read trajectory and unwrap and re-join molecules

   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   PRINT *,'Finished reading trajectory!'
   IF (pbc_jumps) CALL TRAJECT_UNWRAP (nframes, rrr)  
   CALL TRAJECT_JOIN_MOLECULES (nframes, rrr)


   ! Now calculate trajectory of molecular axes
     
   IF (ALLOCATED(traj_mol_axes))   DEALLOCATE(traj_mol_axes)
   ALLOCATE ( traj_mol_axes ( 3, 3, nummols(numspecies), 1, nframes ) );   traj_mol_axes = 0.0
   
   DO IFRAME = 1, nframes
   
      DO J = 1, nummols(selected_species)
   
         molecule(1:3,:) = rrr ( 1:3, :, J, selected_species, IFRAME)

	 SELECT CASE (option)
	 CASE (1)
	    CALL DEFINE_MOLECULAR_ORIENTATION_GEN (molecule, weight, triade, mol_center, mol_axes)
	 CASE (2)   
	    CALL DEFINE_ORIENTATION_IMIDAZOLIUM_1 (molecule, triade, mol_center, mol_axes)
         CASE DEFAULT
            STOP 'Not valid option. Program terminated'
         END SELECT

	 traj_mol_axes(1:3,1:3,J,1,IFRAME) = mol_axes(1:3,1:3)
	 
      END DO
      
   END DO


   ! Initialize histograms
   
   IF ( ALLOCATED(oacf) ) DEALLOCATE(oacf)
   ALLOCATE ( oacf ( 3, 0:nframes-1 ) ); oacf = 0.0
   

   ! Compute histograms
   
   DO T = 0, nframes-1      !time difference

      progress = nframes / 20
      IF (progress < 1) progress = 1
      IF (MOD(T,progress) == 0) PRINT *,'OACF calculation. Completed ',(100*T/nframes),' %'
   
      DO T0 = 1, nframes - T !loop over different time origins
            
	 DO J = 1, nummols(selected_species)
            
	    cos1 = DOT_PRODUCT ( traj_mol_axes(1:3, 1, J, 1, T0+T),traj_mol_axes(1:3, 1, J, 1, T0) )
	    cos2 = DOT_PRODUCT ( traj_mol_axes(1:3, 2, J, 1, T0+T),traj_mol_axes(1:3, 2, J, 1, T0) )
	    cos3 = DOT_PRODUCT ( traj_mol_axes(1:3, 3, J, 1, T0+T),traj_mol_axes(1:3, 3, J, 1, T0) ) 
            oacf(1,T) = oacf(1,T) + 0.5 * (3*cos1*cos1 - 1.0) 
            oacf(2,T) = oacf(2,T) + 0.5 * (3*cos2*cos2 - 1.0)
            oacf(3,T) = oacf(3,T) + 0.5 * (3*cos3*cos3 - 1.0) 

         END DO

      END DO

   END DO 
   

   ! Normalize
   DO IFRAME = 0, nframes-1
      oacf(:,IFRAME) =    oacf(:,IFRAME) / REAL(nummols(selected_species)) / REAL(nframes-IFRAME)
   END DO

    
   ! Write results
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE ( UNIT,'("# Reorientational autocorrelation function (P2)")')
   WRITE ( UNIT, "(A,A40)", IOSTAT = IOSTAT ) "# Analysis using trajectory file: ", traject_file
   WRITE ( UNIT, "(A,I8)", IOSTAT = IOSTAT )  "# Number of frames used in analysis = ", nframes
   WRITE ( UNIT, "(A,I8)", IOSTAT = IOSTAT )  "# Number of molecules used in analysis  = ", nummols(selected_species)
   WRITE ( UNIT, "(A)", IOSTAT = IOSTAT )     "# "
   WRITE ( UNIT, "(A)", IOSTAT = IOSTAT )     "# t, OACF_axis1, OACF_axis2, OACF_axis3"
   WRITE ( UNIT, "(A)", IOSTAT = IOSTAT )     "# "
   DO IFRAME = 0,nframes-1
      WRITE ( UNIT, "(4(G12.6,2x))", IOSTAT = IOSTAT )  tstep*( frame_nstep(IFRAME+1)-frame_nstep(1) ), oacf(1:3,IFRAME)
   END DO


END PROGRAM REORIENT_CORREL
