!
! Program to rotate a single molecule ...
!
! 	Reads:
!	1. CONFIG (+FIELD) from DL_POLY
!
!	Writes:
!	1. File detailing ...
!
!  Last change: 4/10/2013
!

PROGRAM ROTATE_MOLECULE	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
   USE GENERAL_VARIABLES
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE CONFIG_READWRITE,       ONLY : CONFIG_DLPOLY_READ, CONFIG_DLPOLY_WRITE
   USE CONFIG_UTILS,           ONLY : CONFIG_JOIN_MOLECULES
   USE ORIENTATION_UTILS,      ONLY : DEFINE_MOLECULAR_ORIENTATION_GEN
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH)     :: config_file, ff_file, output_file
   CHARACTER ( LEN = 3 )              :: partial_name
   INTEGER                            :: I, ISPEC, IMOL, IATOM, THETA, UNIT, IOSTAT
   INTEGER, DIMENSION(3)              :: triade
   REAL ( KIND = DP ), DIMENSION(3)   :: mol_center
   REAL ( KIND = DP ), DIMENSION(3,3) :: mol_axes, rot_matrix

   REAL ( KIND = DP ) :: X, Y, Z, angle
  
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)         :: weight
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)       :: r_molecule, r_zero
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:)   :: config_pos
						      

   ! Get input
   
   WRITE (OUTPUT,'("Coordinates (CONFIG) file name?")')
   READ (INPUT,'(A)') config_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,*)
  
   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) )  DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
      WRITE ( OUTPUT, '("System information:")' ) 
      WRITE ( OUTPUT, '("Number of different species = ",I3)') numspecies
      DO I = 1,numspecies
         WRITE ( OUTPUT, "('Molecule ',I3,' contains ',I3,' atoms')" ) I,numatoms(I)
      END DO
      WRITE ( OUTPUT, * )
   END IF 
   
   ISPEC = 1
   IMOL = 1
   WRITE ( OUTPUT, '("Note that only 1st molecule in config file is used!")' ) 
   WRITE ( OUTPUT, * ) 
   

   ! Give desired weights to calculate the COM of each molecule & 3 atoms to define the molecular orientation
   ! Axis 1: Along vector atom1-->atom2
   ! Axis 2: Perpendicular to 1 and in plane formed by the three atoms
   ! Axis 3: Axis 1 x Axis 2
   
   ALLOCATE ( weight ( numatoms(ISPEC) ) );        weight = 0.0
   ALLOCATE ( r_molecule ( 3, numatoms(ISPEC) ) ); r_molecule = 0.0
   ALLOCATE ( r_zero ( 3, numatoms(ISPEC)) );      r_zero = 0.0

   
   WRITE (OUTPUT,'("Molecule type ",I3,": Number of atoms = ",I4)') ISPEC, numatoms(ISPEC)
   WRITE (OUTPUT,'("Give weights of all atoms. The COM will be the rotation center!")')
   READ ( INPUT, * ) ( weight(I), I=1,numatoms(ISPEC))
   IF (numatoms(ISPEC) >= 3) THEN
      WRITE (OUTPUT,'("Give numbers of 3 atoms to define the molecular orientation")')
      READ ( INPUT, * ) (triade(I), I=1,3)
   END IF
   WRITE (OUTPUT,'("Give number of atom to be rotated!")')
   READ ( INPUT, * ) IATOM
  

   ! Read coordinates 
      
   CALL CONFIG_DLPOLY_READ (config_file, config_pos)
   CALL CONFIG_JOIN_MOLECULES (config_pos)
 
   r_molecule(1:3,:) = config_pos ( 1:3, :, IMOL, ISPEC)

   CALL DEFINE_MOLECULAR_ORIENTATION_GEN (r_molecule, weight, triade, mol_center, mol_axes)

   rot_matrix = TRANSPOSE(mol_axes) 
   
   !Remove COM
   
   DO I = 1, numatoms(ISPEC)
      r_molecule(:,I) = r_molecule(:,I) - mol_center(:)
   END DO   
      
   DO I = 1, numatoms(ISPEC)
      X = r_molecule ( 1, I) 
      Y = r_molecule ( 2, I) 
      Z = r_molecule ( 3, I)
      r_zero ( 1, I) = X*rot_matrix(1,1) + Y*rot_matrix(1,2) + Z*rot_matrix(1,3)
      r_zero ( 2, I) = X*rot_matrix(2,1) + Y*rot_matrix(2,2) + Z*rot_matrix(2,3)
      r_zero ( 3, I) = X*rot_matrix(3,1) + Y*rot_matrix(3,2) + Z*rot_matrix(3,3)
   END DO
 
   ! Coordinates of atom to be rotated

   X = r_zero ( 1, IATOM) 
   Y = r_zero ( 2, IATOM) 
   Z = r_zero ( 3, IATOM)
   
   ! Set variables to write configurations
   
   IF ( ALLOCATED(config_pos) )  DEALLOCATE(config_pos)
   numspecies = 1; nummols = 1
   ALLOCATE ( config_pos ( 3, maxval(numatoms), maxval(nummols), numspecies ) ) 

   DO THETA = 0, 180

      angle = REAL(THETA) * PI / 180.

      ! Rotation matrix around X axis

      rot_matrix(1,1) = 1.
      rot_matrix(1,2) = 0.
      rot_matrix(1,3) = 0.
      rot_matrix(2,1) = 0.
      rot_matrix(2,2) = cos(angle)
      rot_matrix(2,3) = -sin(angle)
      rot_matrix(3,1) = 0.
      rot_matrix(3,2) = sin(angle)
      rot_matrix(3,3) = cos(angle)
     
      r_zero( 1, IATOM) = X*rot_matrix(1,1) + Y*rot_matrix(1,2) + Z*rot_matrix(1,3)
      r_zero( 2, IATOM) = X*rot_matrix(2,1) + Y*rot_matrix(2,2) + Z*rot_matrix(2,3)
      r_zero( 3, IATOM) = X*rot_matrix(3,1) + Y*rot_matrix(3,2) + Z*rot_matrix(3,3)

      config_pos(:,:,1,1)= r_zero

      WRITE (partial_name,"(I3.3)") THETA
      output_file = 'CONFIG'//partial_name
      CALL CONFIG_DLPOLY_WRITE ( output_file, config_pos )

   END DO
     
END PROGRAM ROTATE_MOLECULE
