!
! Program to calculate the spatial distribution function.
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Options to compute molecular centers and reference axes:
!	1. General 
!	2. Imidazolium
!
!	Writes:
!	1. A file containing the SDF.
!       2. A file called rdf.checking that gives the corresponding pair distribution function for checking purposes.
!       3. A file called refmol.pos that gives the atomic positions of each reference molecule used (only for 1st frame)
!
!  Last change: 7/1/2009
!

PROGRAM SPATIAL_DISTRIBUTION_FUNCTION_SELF	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
   USE GENERAL_VARIABLES
   USE CONFIG_VARIABLES,       ONLY : cell
   USE TRAJECTORY_VARIABLES,   ONLY : pbc_jumps, frame_cell, average_cell_volume
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   USE GENERAL_UTILS,          ONLY : INVERT_CELL, ROTATE_CELL
   USE TRAJECTORY_UTILS,       ONLY : TRAJECT_UNWRAP, TRAJECT_JOIN_MOLECULES
   USE ORIENTATION_UTILS,      ONLY : DEFINE_MOLECULAR_ORIENTATION_GEN, DEFINE_ORIENTATION_IMIDAZOLIUM_1
   USE CONFIG_UTILS,           ONLY : CONFIG_TRANSLATE, CONFIG_ROTATE, CONFIG_CART_2_FRACT
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH)     :: traject_file, ff_file, output_file
   INTEGER                            :: IFRAME, I, J, K, I2, J2, K2, IOSTAT, UNIT, bin, progress, ncheck, npoints
   INTEGER                            :: option, nframes, reference_species, atom2_species, atom2_number, ix, iy, iz 
   INTEGER, DIMENSION(3)              :: triade
   REAL ( KIND = DP )                 :: rmax, dr, det, rx, ry, rz, r, xa, xb, xx, shell_volume, n_ideal, rdf
   REAL ( KIND = DP ), DIMENSION(3)   :: mol_center, vector, sdist
   REAL ( KIND = DP ), DIMENSION(3,3) :: mol_axes, rot_matrix
  
   INTEGER, ALLOCATABLE, DIMENSION(:)                    :: check_rdf
   INTEGER, ALLOCATABLE, DIMENSION(:,:,:)                :: sdf
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:)   :: config_pos, config_fract
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)       :: molecule
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)         :: weight


   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file for spatial distribution function?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,'("Maximum distance to compute s.d.f.?")')
   READ (INPUT,*) rmax
   WRITE (OUTPUT,'("Delta r value to use in s.d.f. grid?")')
   READ (INPUT,*) dr
   WRITE (OUTPUT,*)

   ! Select mode to calculate molecular orientation

   WRITE (OUTPUT,'("Choose option to determine molecular center and axes:")')
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("1. General: Choose one type of molecule and give atom weights and three atom numbers.")')
   WRITE (OUTPUT,'("            Center: COM of molecule computed with the given weights")')
   WRITE (OUTPUT,'("            Axis 1: Along vector atom1-->atom2")')
   WRITE (OUTPUT,'("            Axis 2: Perpendicular to 1 and in plane formed by the three atoms")')
   WRITE (OUTPUT,'("            Axis 3: Axis 1 x Axis 2")')
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("2. Imidazolium 1: Give corresponding species number ")')
   WRITE (OUTPUT,'("                  and the atom numbers of N3(atom 1), N1(atom 2), and C2 (atom 3)")')
   WRITE (OUTPUT,'("                  Center: Along N3-N1 vector, at bisector with C2")')
   WRITE (OUTPUT,'("                  Axis 1: Center-N1")')
   WRITE (OUTPUT,'("                  Axis 2: Center-C2")')
   WRITE (OUTPUT,'("                  Axis 3: Axis 1 x Axis 2")')
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("Selected option?")')
   READ (INPUT,*) option
   WRITE (OUTPUT,*)
   

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) )  DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file.")')
      CALL FIELD_DLPOLY_READ (ff_file)
      WRITE ( OUTPUT, '("System information:")' ) 
      WRITE ( OUTPUT, '("Number of different species = ",I3)') numspecies
      DO I = 1,numspecies
         WRITE ( OUTPUT, "('Molecule ',I3,' contains ',I3,' atoms')" ) I,numatoms(I)
      END DO
      WRITE ( OUTPUT, * )
   END IF 
      

   ! Selection for reference molecule
   
   SELECT CASE (option)
   
   CASE (1)

      WRITE (OUTPUT,'("Number of the species chosen as REFERENCE? ")')
      READ (INPUT,*) reference_species
      IF (reference_species > numspecies) STOP 'Error! Not so many species in trajectory'
      IF (ALLOCATED(molecule)) DEALLOCATE(molecule)
      IF (ALLOCATED(weight))   DEALLOCATE(weight)
      ALLOCATE ( molecule ( 3, numatoms(reference_species) ) ); molecule = 0.0
      ALLOCATE ( weight ( numatoms(reference_species) ) );   weight = 0.0
      WRITE (OUTPUT,'("Number of atoms in selected type = ",I4)') numatoms(reference_species)
      WRITE (OUTPUT,'("Give weight for each atom ")')
      DO I = 1,numatoms(reference_species)
         WRITE ( OUTPUT, "('Atom',I3,': Weight?')" ) I
	 READ ( INPUT, * ) weight(I)
      END DO
      WRITE (OUTPUT,'("Now give numbers in molecule of selected triplet of atoms ")')
      DO I = 1,3
         WRITE ( OUTPUT, "('Atom',I3,'?')" ) I
	 READ ( INPUT, * ) triade(I)
      END DO
    
   CASE (2)

      WRITE (OUTPUT,'("Number of the species chosen as reference in trajectory? ")', ADVANCE='NO' )
      READ (INPUT,*) reference_species
      IF (reference_species > numspecies) STOP 'Error! Not so many species in trajectory'
      IF (ALLOCATED(molecule)) DEALLOCATE(molecule)
      ALLOCATE ( molecule ( 3, numatoms(reference_species) ) ); molecule = 0.0
      WRITE (OUTPUT,'("Now give the corresponding number indicating the order in the molecule of N3, N1, and C2 (in this order).")')
      DO I = 1,3
         WRITE ( OUTPUT, "('Atom',I3,'? ')", ADVANCE='NO' ) I
	 READ ( INPUT, * ) triade(I)
      END DO

   CASE DEFAULT
      STOP 'Not valid option. Program terminated'
   
   END SELECT


   ! Select atom to compute the SDF

   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("And now choose a particular atom to compute its spatial density around the reference molecule.")')
   WRITE (OUTPUT,'("Species number? ")', ADVANCE='NO' )
   READ (INPUT,*) atom2_species
   IF (atom2_species /= reference_species) STOP 'SELF SDF!' 
   WRITE (OUTPUT,'("Atom number in molecule? ")', ADVANCE='NO' )
   READ (INPUT,*) atom2_number
   

   ! Read trajectory and unwrap and re-join molecules

   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   PRINT *,'Finished reading trajectory!'
   IF (pbc_jumps) CALL TRAJECT_UNWRAP (nframes, rrr)  
   CALL TRAJECT_JOIN_MOLECULES (nframes, rrr)
     
   
   ! Allocate arrays containing positions for a single configuration
   
   IF (ALLOCATED(config_pos))   DEALLOCATE ( config_pos)
   IF (ALLOCATED(config_fract)) DEALLOCATE ( config_fract)
   ALLOCATE ( config_pos ( 3, maxval(numatoms), maxval(nummols), numspecies ) ); config_pos = 0.0
   ALLOCATE ( config_fract ( 3, maxval(numatoms), maxval(nummols), numspecies ) ); config_fract = 0.0

    
   ! Initialize histograms
   
   ncheck = int(rmax/0.05) + 1   ! Dimension of array for r.d.f. used for checking
   IF ( ALLOCATED(check_rdf) ) DEALLOCATE(check_rdf)
   ALLOCATE ( check_rdf ( ncheck ) ); check_rdf = 0
   npoints = int(rmax/dr) + 1    ! Dimension in each direction of the array for s.d.f.
   IF ( ALLOCATED(sdf) ) DEALLOCATE(sdf)
   ALLOCATE ( sdf ( -npoints:npoints, -npoints:npoints, -npoints:npoints ) ); sdf = 0

   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = 'refmol.pos', ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )

   ! Compute sdf
   
   frames_loop: DO IFRAME = 1,nframes

      progress = nframes / 20
      IF (progress < 1) progress = 1
      IF (MOD(IFRAME,progress) == 0) PRINT *,'SDF calculation. Completed ',(100*IFRAME/nframes),' %'
      WRITE (UNIT,'("# Frame = ",I3)') IFRAME
   
      molecules1_loop: DO J = 1, nummols(reference_species)  !reference system
 
         ! Corresponding atomic configuration for this frame. 
	 ! Needs to reread for every molecule J because config_pos is modified in the loop
         config_pos(:,:,:,:) = rrr(:,:,:,:,IFRAME)
   
	 ! Calculate center and molecular axes of reference molecule	 
         molecule(1:3,:) = rrr ( 1:3, :, J, reference_species, IFRAME) 
	 SELECT CASE (option)
	 CASE (1)
	    CALL DEFINE_MOLECULAR_ORIENTATION_GEN (molecule, weight, triade, mol_center, mol_axes)
	 CASE (2)   
	    CALL DEFINE_ORIENTATION_IMIDAZOLIUM_1 (molecule, triade, mol_center, mol_axes)
         CASE DEFAULT
            STOP 'Not valid option. Program terminated'
         END SELECT

         ! Translate full configuration so that center of current molecule is at origin
         vector = -mol_center(1:3)
         CALL CONFIG_TRANSLATE (config_pos, vector)

         ! Define rotation matrix and rotate system
	 rot_matrix = TRANSPOSE(mol_axes) 
	 CALL CONFIG_ROTATE (config_pos, rot_matrix)

         ! Write coordinates of reference molecule to use when plotting SDF using LAMP 
	 IF (IFRAME == 1) THEN
	    DO K = 1, numatoms(reference_species)
	       vector = config_pos ( 1:3, K, J, reference_species) 
	       vector = vector + rmax   !Volume in LAMP given as box from 0 to 2*Rmax, so this is needed to replace the molecule in the center of the box
	       WRITE (UNIT,'("# Molecule ",I3,", atom ",I3,". Coordinates = ",3(F6.3,1x))') J, K, vector
	    END DO
	 END IF

         ! Rotate also the cell
	 cell = frame_cell(1:3, 1:3, IFRAME)
         CALL ROTATE_CELL (cell, rot_matrix)    

         ! And now get positions in fractional coordinates
	 CALL CONFIG_CART_2_FRACT (config_pos, config_fract)

         ! Compute SDF
         I2 = atom2_species
         J2 = J
         K2 = atom2_number		        

	 !IF ( (I2 == reference_species) .AND. (J2 == J) ) CYCLE molecules2_loop

	 ! Distance to center is directly position in fractional units, as center is in (0,0,0)
	 sdist = config_fract(1:3, K2, J2, I2)	
	 sdist = sdist - ANINT(sdist)
         rx = sdist(1)*cell(1,1) + sdist(2)*cell(2,1) + sdist(3)*cell(3,1)
         ry = sdist(1)*cell(1,2) + sdist(2)*cell(2,2) + sdist(3)*cell(3,2)
         rz = sdist(1)*cell(1,3) + sdist(2)*cell(2,3) + sdist(3)*cell(3,3)
	 r = SQRT( rx*rx + ry*ry + rz*rz )

         IF (r <= rmax) THEN
	    
               ! Check RDF
	       bin = INT(r/0.05) + 1
               check_rdf(bin) = check_rdf(bin) + 1 
	       
	       !Determine corresponding voxel and make 3D-histogram
	       ix = ANINT(rx/dr) + 1
	       iy = ANINT(ry/dr) + 1
	       iz = ANINT(rz/dr) + 1
	       sdf(ix,iy,iz) = sdf(ix,iy,iz) + 1
	       
	 END IF
	 
      END DO molecules1_loop	 
      
   END DO frames_loop

   CLOSE (UNIT) !Close the file refmol.pos used to print the coordinates of the reference molecules used

   ! Write sdf
   
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT,'("# SDF array [ix,iy,iz]. Nx=Ny=Nz = ",I3,". Array size = ",I3,"^3. Rmax = ",F5.2)') &
    npoints, 2*npoints+1, rmax
   WRITE (UNIT,'("# REFERENCE: species number = ",I3,". SDF around reference for atom ",I3," of species ",I3)') &
    reference_species, atom2_number, atom2_species
   WRITE (UNIT,'("# Increment order 1: -Nx --> Nx ; 2: -Ny --> Ny; 3: -Nz --> Nz")')
   DO K = -npoints, npoints
      DO J = -npoints, npoints
	 WRITE (UNIT,"(20(I5,1x))") ( sdf(I,J,K), I = -npoints, npoints)
      END DO   
   END DO
   CLOSE (UNIT)

   ! Write rdf for checking
   
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = 'rdf.checking', ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT,'("# RDF computed during SDF calculation")')
   WRITE (UNIT,'("#",3x,"r/A",3x,"RDF pairs")')
   DO I = 1,ncheck
      xa = REAL(I-1)*0.05
      xb = xa + 0.05
      xx = 0.5 * (xa + xb)
      shell_volume = 4.0 * pi * (xb**3 - xa**3) / 3.0
      n_ideal = REAL( nummols(reference_species) * (nummols(atom2_species)-1.0) ) / average_cell_volume
      rdf = REAL(check_rdf(I))/ REAL(nframes) / shell_volume / n_ideal
      WRITE (UNIT,"(1x,f7.3,3x,f10.6)") xx, rdf
   END DO
   CLOSE (UNIT)
        
   
END PROGRAM SPATIAL_DISTRIBUTION_FUNCTION_SELF
