PROGRAM SQ_

! Computes the structure factor from the radial distribution function
! using the following expression:
!
! S(Q) = 1 + 4 * PI * Int(0,infty) dr * r^2 * (sin(Qr)/Qr) * rho * (g(r)-1)
!
! and considering g(r)=1 for r > r_max
!
! Example of use: sq < sq.inp
!
! with sq.inp:
!  gr.dat                #file with g(r)
!  sq.dat                #output file containing S(Q)
!  2                     #number of lines to skip in input file
!  4                     #column corresponding to g(r) (assumed 1st col is r)
!  864                   #number of particles
!  40410.10487           #volume of simulation box (in angstroms^3)


   IMPLICIT NONE

   INTEGER, PARAMETER        :: mmax=1e6, mqmax=3e3
   INTEGER, PARAMETER        :: DP = SELECTED_REAL_KIND(15,3)
   REAL (KIND=DP), PARAMETER :: PI = 3.141592653589793_DP
   
   CHARACTER ( LEN = 80) :: filein, fileout
   INTEGER               :: nskip, ncol, natoms, i, j, ndata, nmax, counter, iostat
   REAL ( KIND = DP )    :: vol, rho, rmax, dr, dummy, dq, qmax, q, sq, rint, rrr

   REAL ( KIND = DP ), DIMENSION(mmax) :: r, gr

   ! Get input
   
   WRITE (*,'("Input file containing g(r)")')
   READ (*,'(A)') filein
   WRITE (*,'("Output file for S(Q)?")')
   READ (*,'(A)') fileout
   WRITE (*,'("Number of lines to skip (header input file)?")')
   READ (*,*) nskip
   WRITE (*,'("Column containing the g(r) data to use?")')    
   READ (*,*) ncol
   WRITE (*,'("Total number of atoms in simulation box?")')     
   READ (*,*) natoms
   WRITE (*,'("Volume of simulation box (in angstroms^3)")')
   READ (*,*) vol  
   WRITE (*,'("Q step in output file?")')
   READ (*,*) dq  
   WRITE (*,'("Maximum Q value in output file?")')
   READ (*,*) qmax  

   rho = real(natoms)/vol

   OPEN (11, FILE = filein, ACTION = "READ", STATUS = "OLD", IOSTAT = IOSTAT )
   OPEN (21, FILE = fileout, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )

   do i = 1,nskip
      read (11,*)
   enddo

   counter = 0
   do 
      counter = counter + 1
      if (counter > mmax) STOP 'File has more than mmax lines. Increase mmax!'
      read (11, iostat=iostat) r(counter), (dummy,j=2,ncol-1), gr(counter)
      if (iostat /= 0) exit
   enddo        
   close (11)

   ndata = counter-1
   rmax = r(ndata)
   dr = r(ndata) - r(ndata-1)   
      
   nmax = int(qmax/dq) + 1
   if (nmax > mqmax) STOP 'Error: nmax > mqmax. Increase nmax!'
   
   write (21,'(''#'',4x,''Q/A-1'',11x,''S(Q)'')')
   write (21,'(''#'')')
   do i = 1,nmax
      q = real(i)*dq
      rint = 0.0
      do j = 1,ndata
         rint = rint + dr*r(j)*r(j)*(sin(q*r(j))/(q*r(j)))*rho*(gr(j)-1.0)
      end do
      sq = 1.0 + 4 * pi * rint
      write (21,*) q,sq
   enddo
      
END PROGRAM SQ_
