!
! Program to calculate torsional distributions
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Writes:
!	1. A file per molecule containing the histogram of torsional distributions for each selected dihedral
!
!  Last change: 22/6/2016
!

PROGRAM TORSIONAL_DISTRIBUTION_BY_MOLECULE	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES,   ONLY : pbc_jumps
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,       ONLY : TRAJECT_UNWRAP, TRAJECT_JOIN_MOLECULES
   USE GENERAL_UTILS,          ONLY : DIHEDRAL_ANGLE
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH) :: traject_file, ff_file, output_file
   CHARACTER ( LEN = 3 )                              :: mol_number

   INTEGER :: IFRAME, IMOL, ITORS, I, J, IOSTAT, UNIT, nframes, numtors, selected_species, idihedral
   INTEGER, ALLOCATABLE, DIMENSION(:,:)   :: dihatom
   INTEGER, ALLOCATABLE, DIMENSION(:,:,:) :: histogram

   REAL ( KIND = DP )               :: dihedral
   REAL ( KIND = DP ), DIMENSION(3) :: r1, r2, r3, r4

   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr


   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,*)
   

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
      WRITE ( OUTPUT, '("System information:")' ) 
      WRITE ( OUTPUT, '("Number of different species = ",I3)') numspecies
      DO I = 1,numspecies
         WRITE ( OUTPUT, "('Molecule ',I3,' contains ',I3,' atoms')" ) I,numatoms(I)
      END DO
      WRITE ( OUTPUT, * )
   END IF 
       
   ! Select torsions to compute

   WRITE (OUTPUT,'("Number of the chosen species in trajectory? ")')
   READ (INPUT,*) selected_species
   IF (selected_species > numspecies) STOP 'Error! Not so many species in trayectory'
   WRITE (OUTPUT,'("Number of atoms in selected type = ",I4)') numatoms(selected_species)
   WRITE (OUTPUT,'("Number of torsions to compute? ")')
   READ (INPUT,*) numtors
   IF (numtors > 20) PRINT *,'Output file will contain more than one row per dihedral value.'
   ALLOCATE ( dihatom ( numtors, 4 ) ); dihatom = 0
   ALLOCATE ( histogram ( numtors, nummols(selected_species), 361 ) ); histogram = 0
   WRITE (OUTPUT,'("For each torsion, give the numbers of the 4 atoms defining the dihedral angle ")')
   DO I = 1,numtors
      WRITE (OUTPUT, "('Torsion ',I3,'?')" ) I
      READ ( INPUT, * ) (dihatom(I,J), J = 1, 4)
    END DO

   
   ! Read trajectory 
      
   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   IF (pbc_jumps) CALL TRAJECT_UNWRAP (nframes, rrr)  
   CALL TRAJECT_JOIN_MOLECULES (nframes, rrr)
   

   ! Compute torsions
      
   frames_loop: DO IFRAME = 1,nframes
      molecules_loop: DO IMOL = 1, nummols(selected_species)
         torsions_loop: DO ITORS = 1,numtors
            r1 = rrr(1:3, dihatom(ITORS,1), IMOL, selected_species, IFRAME)
            r2 = rrr(1:3, dihatom(ITORS,2), IMOL, selected_species, IFRAME)
            r3 = rrr(1:3, dihatom(ITORS,3), IMOL, selected_species, IFRAME)
            r4 = rrr(1:3, dihatom(ITORS,4), IMOL, selected_species, IFRAME)
            CALL DIHEDRAL_ANGLE (r1, r2, r3, r4, dihedral)
            if (dihedral < -180.0) dihedral = -180.0
            if (dihedral > 180.0) dihedral = 180.0
            idihedral = 181 + NINT(dihedral)
            histogram(ITORS, IMOL, idihedral) = histogram(ITORS, IMOL, idihedral) + 1
         END DO torsions_loop
      END DO molecules_loop
   END DO frames_loop	

   ! Open output file
   
   DO IMOL = 1, nummols(selected_species)
      WRITE (mol_number,"(I3.3)") IMOL
      UNIT = NEXT_UNIT ( )	
      OPEN ( UNIT, FILE = 'tors_dist_'//mol_number//'.dat', ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
      WRITE (UNIT,"('# Torsion: ',20(1x,I5))") (ITORS, ITORS=1,numtors) 
      DO I = 1,361
         dihedral = REAL(I)-181.0
         WRITE (UNIT,"(1x,F6.1,4x,20(1x,F6.4))") dihedral, (REAL(histogram(ITORS,IMOL,I))/REAL(nframes), ITORS=1,numtors)
      END DO	 
      CLOSE (UNIT)
   END DO

   
END PROGRAM TORSIONAL_DISTRIBUTION_BY_MOLECULE
