!
! Program to calculate the velocity autocorrelation function.
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!  Last change: 05/11/2013
!

PROGRAM VELOCITY_ACF	

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES,   ONLY : frame_nstep, tstep
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH)     :: traject_file, ff_file, output_file
   INTEGER                            :: IFRAME, I, J, K, T, T0, UNIT, IOSTAT, V0NORM
   INTEGER                            :: nframes, progress

   REAL ( KIND = DP ) :: VACF0, V0SQR
   
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr, vvv
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:) :: vacf
   

   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file for velocity autocorrelation function?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,*)


   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) )  DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
      WRITE ( OUTPUT, '("System information:")' ) 
      WRITE ( OUTPUT, '("Number of different species = ",I3)') numspecies
      DO I = 1,numspecies
         WRITE ( OUTPUT, "('Molecule ',I3,' contains ',I3,' atoms')" ) I,numatoms(I)
      END DO
      WRITE ( OUTPUT, * )
   END IF 
      

   ! Read trajectory and unwrap and re-join molecules

   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr, vvv)


   ! Initialize histograms
   
   IF ( ALLOCATED(vacf) ) DEALLOCATE(vacf)
   ALLOCATE ( vacf ( 0:nframes-1 ) ); vacf = 0.0

   ! Compute correlation function
   
   V0NORM = 0
   DO T = 0, nframes-1      !time difference

      progress = nframes / 20
      IF (progress < 1) progress = 1
      IF (MOD(T,progress) == 0) PRINT *,'VACF calculation. Completed ',(100*T/nframes),' %'
   
      DO T0 = 1, nframes - T !loop over different time origins
            
	 DO I = 1, numspecies
	    DO J = 1, nummols(I)
	       DO K = 1, numatoms(I)
                   vacf(T) = vacf(T) + DOT_PRODUCT ( vvv(1:3, K, J, I, T0+T),vvv(1:3, K, J, I, T0) ) 
		   IF (T == 0) V0NORM = V0NORM+1
               END DO
            END DO
         END DO

      END DO

   END DO 
   V0SQR = vacf(0)/REAL(V0NORM)
   

   ! Normalize
   DO IFRAME = 0, nframes-1
      vacf(IFRAME) =    vacf(IFRAME) / REAL(nframes-IFRAME)
   END DO
   VACF0 = vacf(0)
   DO IFRAME = 0, nframes-1
      vacf(IFRAME) =    vacf(IFRAME) / VACF0
   END DO

    
   ! Write results
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE ( UNIT,'("# Velocity autocorrelation function")')
   WRITE ( UNIT, "(A,A40)", IOSTAT = IOSTAT )   "# Analysis using trajectory file: ", traject_file
   WRITE ( UNIT, "(A,I8)", IOSTAT = IOSTAT )    "# Number of frames used in analysis = ", nframes
   WRITE ( UNIT, "(A,G12.6)", IOSTAT = IOSTAT ) "# VACF (t=0) = ", V0SQR
   WRITE ( UNIT, "(A)", IOSTAT = IOSTAT )       "# "
   WRITE ( UNIT, "(A)", IOSTAT = IOSTAT )       "# t, <VACF>"
   WRITE ( UNIT, "(A)", IOSTAT = IOSTAT )       "# "
   DO IFRAME = 0,nframes-1
      WRITE ( UNIT, "(5(G12.6,2x))", IOSTAT = IOSTAT )  tstep*( frame_nstep(IFRAME+1)-frame_nstep(1) ), vacf(IFRAME)
   END DO

END PROGRAM VELOCITY_ACF
