!
! Reads the file(s) produced by the C program voro++ together with a trajectory
! and makes some analysis on molecules selected depending on the characteristics of Voronoi's polyhedra.
!
!  Last change: 25/11/2013
!

PROGRAM VORONOI_NEIGHBOUR_ANALYSIS	
   

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
   USE PARSING,                ONLY : GET_LINE, GET_WORD, GET_INTEGER, GET_REAL, WORD, WRDLEN, EMPTY, PUSH_UNIT, POP_UNIT
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,       ONLY : TRAJECT_UNWRAP, TRAJECT_JOIN_MOLECULES, TRAJECT_CART_2_FRACT, TRAJECT_FRACT_2_CART
   USE ORIENTATION_UTILS,      ONLY : DEFINE_MOLECULAR_ORIENTATION_GEN
   USE LINEAR_ALGEBRA,         ONLY : NORMALIZE
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH) :: voro_file, vol_file, traject_file, ff_file, output_file_1, output_file_2
   CHARACTER ( LEN = 4)           :: frame_number
   INTEGER, PARAMETER             :: maxneighbours = 25
   INTEGER                        :: UNIT, IOSTAT, I, J, K, IFRAME, nfirst, nlast, nframes, nmolecules, index, bin, binth, &
                                     number_vertices, number_edges, number_faces, min_faces, max_faces, ntemp, maxbin, maxbinth, &
				     nvector
   REAL ( KIND = DP )             :: x, y, z, voronoi_volume, rx, ry, rz, temp, total_weight, min_volume, max_volume, dot, &
                                     theta, sinth, dtheta, dr
   LOGICAL                        :: end = .FALSE.
   
   INTEGER, DIMENSION(3)              :: triademol
   REAL ( KIND = DP ), DIMENSION(3)   :: mol_center, sdist
   REAL ( KIND = DP ), DIMENSION(3,3) :: mol_axes
   
   INTEGER, DIMENSION(maxneighbours)            :: neighbour
   REAL ( KIND = DP ), DIMENSION(maxneighbours) :: r
   
   INTEGER, ALLOCATABLE, DIMENSION(:,:)                  :: triade, neighbour_distance
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)         :: weightmol
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)       :: weight, molecule, neighbour_orientation
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr, sss, traj_mol_center, traj_mol_axes

   
   ! Get input for Voronoi files
   
   WRITE (OUTPUT,'("VORONOI INPUT:")')
   WRITE (OUTPUT,'("Voronoi file name (common part)? Assumes files named file_nameXXXX.xyz.vol")')
   READ (INPUT,'(A)') voro_file
   WRITE (OUTPUT,'("First file to use?")')
   READ (INPUT,*) nfirst
   WRITE (OUTPUT,'("Last file to use?")')
   READ (INPUT,*) nlast
   WRITE (OUTPUT,'("Number of molecules in file?")')
   READ (INPUT,*) nmolecules
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("Rules to select subset of Voronoi polyhedron:")')
   WRITE (OUTPUT,'("Minimum number of faces?")')
   READ (INPUT,*) min_faces
   WRITE (OUTPUT,'("Maximum number of faces?")')
   READ (INPUT,*) max_faces
   WRITE (OUTPUT,'("Minimum number of volume?")')
   READ (INPUT,*) min_volume
   WRITE (OUTPUT,'("Maximum number of volume?")')
   READ (INPUT,*) max_volume
   WRITE (OUTPUT,'("Step in distance histogram?")')
   READ (INPUT,*) dr
   WRITE (OUTPUT,'("Step in orientation histogram?")')
   READ (INPUT,*) dtheta
   WRITE (OUTPUT,'("Vector to use in orientation calculation (1, 2 or 3)?")')
   READ (INPUT,*) nvector
   IF (nvector < 1) THEN
      PRINT *, 'Using nvector = 1'
      nvector = 1
   END IF
   IF (nvector > 3) THEN
      PRINT *, 'Using nvector = 3'
      nvector = 3
   END IF
   WRITE (OUTPUT,'("Name of output file to store the distance distribution for each neighbour?")')
   READ (INPUT,'(A)') output_file_1
   WRITE (OUTPUT,'("Name of output file to store the orientation distribution for each neighbour?")')
   READ (INPUT,'(A)') output_file_2
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("TRAJECTORY INPUT: Assumes frame index corresponds to number in Voronoi files.")')
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,*)
   
   nframes = nlast  ! Read only up to nlast frames in HISTORY
   
   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF (numspecies > 1) STOP 'Not ready for more than 1 species'
      IF ( ALLOCATED(nummols) )  DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
      WRITE ( OUTPUT, '("System information:")' ) 
      WRITE ( OUTPUT, '("Number of different species = ",I3)') numspecies
      IF (numspecies > 1) STOP 'Not ready for more than 1 species'
      DO I = 1,numspecies
         WRITE ( OUTPUT, "('Molecule ',I3,' contains ',I3,' atoms')" ) I,numatoms(I)
      END DO
      WRITE ( OUTPUT, * )
   END IF 

   ! Give desired weights to calculate the COM of each molecule & 3 atoms to define the molecular orientation
   ! Axis 1: Along vector atom1-->atom2
   ! Axis 2: Perpendicular to 1 and in plane formed by the three atoms
   ! Axis 3: Axis 1 x Axis 2
   
   ALLOCATE ( weight ( maxval(numatoms), numspecies ) );   weight = 0.0
   ALLOCATE ( weightmol ( maxval(numatoms) ) );   weightmol = 0.0
   ALLOCATE ( triade ( 3, numspecies ) );   triade = 0
   ALLOCATE ( molecule ( 3, maxval(numatoms) ) ); molecule = 0.0
   DO I = 1,numspecies
      WRITE (OUTPUT,'("Molecule type ",I3,": Number of atoms = ",I4)') I,numatoms(I)
      WRITE (OUTPUT,'("Give weights of all atoms ")')
      READ ( INPUT, * ) ( weight(J,I), J=1,numatoms(I))
      IF (numatoms(I) >= 3) THEN
         WRITE (OUTPUT,'("Give numbers of 3 atoms to define the molecular orientation")')
	 READ ( INPUT, * ) (triade(J,I), J=1,3)
      END IF
   END DO

   
   ! Read trajectory and unwrap and re-join molecules

   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   PRINT *,'Finished reading trajectory!'
   IF (pbc_jumps) CALL TRAJECT_UNWRAP (nframes, rrr)  
   CALL TRAJECT_JOIN_MOLECULES (nframes, rrr)

   ! Now calculate trajectory of molecular centers and axes
     
   IF (ALLOCATED(traj_mol_center)) DEALLOCATE(traj_mol_center)
   IF (ALLOCATED(traj_mol_axes))   DEALLOCATE(traj_mol_axes)
   ALLOCATE ( traj_mol_center ( 3, 1, maxval(nummols), numspecies, nframes ) ); traj_mol_center = 0.0 
   ALLOCATE ( traj_mol_axes ( 3, 3, maxval(nummols), numspecies, nframes ) );   traj_mol_axes = 0.0
   
   ! Allocate arrays to contain neighbour distances and list of neighbours & Compute histograms
   
   maxbin = 0
   maxbinth = 180.0 / dtheta
   ALLOCATE ( neighbour_distance (max_faces, 9999 ) ); neighbour_distance = 0
   ALLOCATE ( neighbour_orientation (max_faces, maxbinth ) ); neighbour_orientation = 0.0


   DO IFRAME = 1, nframes

      DO I = 1, numspecies

         DO J = 1, nummols(I)
   
            molecule(1:3,:) = rrr ( 1:3, :, J, I, IFRAME)
            weightmol = weight(:,I)
            triademol = triade(:,I)

            IF (numatoms(I) >= 3) THEN

	       CALL DEFINE_MOLECULAR_ORIENTATION_GEN (molecule, weightmol, triademol, mol_center, mol_axes)

            ELSE

               mol_center = 0.0; total_weight = 0.0
               DO K = 1, numatoms(I)
                  mol_center(1:3) = mol_center(1:3) + weight(K,I) * molecule(1:3,K)
                  total_weight = total_weight + weight(K,I)
               END DO 
               mol_center = mol_center / total_weight
               IF (numatoms(I) == 2) THEN
                  ! Define first axis as bond between both atoms
                  mol_axes(1:3,1) = molecule(1:3,2) - molecule(1:3,1)	 
                  mol_axes(1:3,1) = NORMALIZE( mol_axes(1:3,1) )
               ELSE 
                  ! Use the cartesian axes
                  mol_axes(1:3,1) =  (/ 1.0d0, 0.0d0, 0.0d0 /)
               END IF
               mol_axes(1:3,2) =  (/ 0.0d0, 1.0d0, 0.0d0 /)
               mol_axes(1:3,3) =  (/ 0.0d0, 0.0d0, 1.0d0 /)

            END IF
            
	    traj_mol_center(1:3,1,J,I,IFRAME) = mol_center(1:3)
	    traj_mol_axes(1:3,1:3,J,I,IFRAME) = mol_axes(1:3,1:3)

         END DO	 

      END DO
      
   END DO

   ! Calculate trajectory of centers in fractional units
   
   CALL TRAJECT_CART_2_FRACT (nframes, traj_mol_center, sss)
 
   ! Read Voronoi files
   
   DO IFRAME = nfirst, nlast 
      WRITE (frame_number,'(I4.4)') IFRAME
      vol_file = voro_file(1:LEN_TRIM(voro_file))//frame_number//'.xyz.vol'
      UNIT = NEXT_UNIT ( )
      OPEN ( UNIT, FILE = vol_file, ACTION = "READ", STATUS = "OLD", IOSTAT = IOSTAT )
      CALL PUSH_UNIT ( UNIT )
      DO WHILE (.NOT.end) 
         CALL GET_LINE (end)
         IF (.NOT.end) THEN
	    CALL GET_INTEGER ( index )
	    IF (index > nmolecules) STOP 'Increase number of molecules'
	    CALL GET_REAL ( x )
	    CALL GET_REAL ( y )
	    CALL GET_REAL ( z )
	    CALL GET_INTEGER ( number_vertices )
	    CALL GET_INTEGER ( number_edges )
	    CALL GET_INTEGER ( number_faces )
	    CALL GET_REAL ( voronoi_volume )
            IF (number_faces > maxneighbours) THEN
	       print *, 'Molecule ', index, ' at frame ', IFRAME, 'has number_faces = ', number_faces
	       print *, 'Increase maxneighbours'
	       STOP
	    END IF
            DO I = 1, number_faces
	       CALL GET_INTEGER ( neighbour(I) )
	    END DO
	    
	    !Selection
	    
	    IF (voronoi_volume >= min_volume .AND. voronoi_volume <= max_volume .AND. & 
	        number_faces >= min_faces .AND. number_faces <= max_faces) THEN
		
	       ! Obtain distance between molecular centers
			
	       DO I = 1, number_faces		
	          sdist = sss(1:3,1,index,1,IFRAME) - sss(1:3,1,neighbour(I),1,IFRAME)
	          sdist = sdist - ANINT(sdist)
                  rx = sdist(1)*frame_cell(1,1,IFRAME) + sdist(2)*frame_cell(2,1,IFRAME) + sdist(3)*frame_cell(3,1,IFRAME)
                  ry = sdist(1)*frame_cell(1,2,IFRAME) + sdist(2)*frame_cell(2,2,IFRAME) + sdist(3)*frame_cell(3,2,IFRAME)
                  rz = sdist(1)*frame_cell(1,3,IFRAME) + sdist(2)*frame_cell(2,3,IFRAME) + sdist(3)*frame_cell(3,3,IFRAME)
                  r(I) = SQRT( rx*rx + ry*ry + rz*rz )
	       END DO

               ! Simple sorting
            
               DO I = 1, number_faces-1
                  DO J = I+1, number_faces
                     IF ( r(I) > r(J) ) THEN
                        temp = r(J); r(J)=r(I); r(I)=temp
                        ntemp = neighbour(J); neighbour(J)=neighbour(I); neighbour(I)=ntemp 
                     END IF
                  END DO
               END DO		     
	       
	       ! Check
	       
	       ! print *, 'Voronoi polyhedron for molecule ', index
	       ! print *, 'Number faces = ', number_faces
	       ! print *, 'Neighbouring molecule - distance: '
               ! DO I = 1, number_faces
	       !    print *, neighbour(I), r(I)
               ! END DO	
		
               !Calculate distances and orientations for nearest neighbours

               DO I = 1, number_faces

        	  bin = INT(r(I)/dr) + 1
        	  IF (bin > 9999) STOP 'Too large distance!'
        	  IF (bin > maxbin) maxbin = bin 
        	  neighbour_distance(I,bin) = neighbour_distance(I,bin) + 1
        	  dot = DOT_PRODUCT ( traj_mol_axes(1:3,nvector,index,1,IFRAME), &
                                      traj_mol_axes(1:3,nvector,neighbour(I),1,IFRAME) )
		  theta = ACOS(dot)*180.0/pi
		  binth = INT(theta/dtheta) + 1
		  !weight each point by 1/sin(theta) to take into account solid angle
		  sinth = SIN(ACOS(dot))
		  IF (sinth < 1.0e-3) sinth = 1.0e-3
        	  neighbour_orientation(I,binth) = neighbour_orientation(I,binth) + (1./sinth)

               END DO	
	 
	    END IF 
	    
	    
         END IF
      END DO
      CLOSE ( UNIT )
      CALL POP_UNIT 
      IF (index /= nmolecules) nmolecules = index
      end = .FALSE.
   END DO

   ! Write results

   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file_1, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'Output file for distance histogram could not be created!'
   WRITE (UNIT,'("# Distance distribution for each neighbour")')
   WRITE (UNIT,'("#",3x,"r - Neighbour order")')
   DO J = 1, maxbin
       temp = float(J-1)*dr + 0.5*dr
       WRITE (UNIT,'(F5.2,5x,25(I6,1x))') temp, (neighbour_distance(I,J),I=1,max_faces)
   END DO
   CLOSE (UNIT)

   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = output_file_2, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'Output file for orientation histogram could not be created!'
   WRITE (UNIT,'("# Orientation distribution for each neighbour")')
   WRITE (UNIT,'("# theta - Neighbour order")')
   DO J = 1, maxbinth
       temp = float(J-1)*dtheta + 0.5*dtheta
       WRITE (UNIT,'(F5.1,5x,25(F8.3,1x))') temp, (neighbour_orientation(I,J)/float(nframes),I=1,max_faces)
   END DO
   CLOSE (UNIT)

   
END PROGRAM VORONOI_NEIGHBOUR_ANALYSIS
