!
! Reads the file(s) produced by the C program voro++ and
! analyse them to produce some histograms and do some other
! specific analysis between clusters (defined as a molecule +
! all the molecules in neighbouring Voronoi volumes)
!
!  Last change: 11/03/2013
!

PROGRAM VORONOI_OUTPUT_ANALYSIS	
   

   USE DEFINITIONS,          ONLY : LINE_LENGTH, DP
   USE IO_UNITS,             ONLY : INPUT, OUTPUT
   USE FILES,                ONLY : NEXT_UNIT
   USE PARSING,              ONLY : GET_LINE, GET_WORD, GET_INTEGER, GET_REAL, WORD, WRDLEN, EMPTY, PUSH_UNIT, POP_UNIT
!   USE GENERAL_VARIABLES
!   USE TRAJECTORY_VARIABLES
!   USE FORCEFIELD_READ,      ONLY : FIELD_DLPOLY_READ
!   USE TRAJECTORY_READWRITE, ONLY : HISTORY_DLPOLY_READ
!   USE TRAJECTORY_UTILS,     ONLY : TRAJECT_CART_2_FRACT, TRAJECT_FRACT_2_CART
!   USE CONFIG_READWRITE,     ONLY : CONFIG_XYZ_WRITE
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH) :: voro_file, vol_file, out1_file, out2_file, out3_file
   CHARACTER ( LEN = 4)           :: frame_number
   INTEGER, PARAMETER             :: maxneighbours = 25, n_histovol = 50
   INTEGER                        :: UNIT, IOSTAT, I, J, IFRAME, nfirst, nlast, nframes, nmolecules, index, n_volumes, bin, bin2
   INTEGER                        :: n_histofaces
   REAL ( KIND = DP )             :: x, y, z, av_vertices, av_edges, av_faces, av_volume, dv
   LOGICAL                        :: end = .FALSE.
   INTEGER, DIMENSION(n_histovol) :: histovol
   
   INTEGER, ALLOCATABLE, DIMENSION(:)              :: histofaces
   INTEGER, ALLOCATABLE, DIMENSION(:,:)            :: vertices, edges, faces, histovolfaces
   INTEGER, ALLOCATABLE, DIMENSION(:,:,:)          :: neighbour
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:) :: volume

!   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: traject_r, tmp_s
!   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:)   :: config_pos
   
   ! Get input for Voronoi files
   
   WRITE (OUTPUT,'("VORONOI INPUT:")')
   WRITE (OUTPUT,'("Voronoi file name (common part)? Assumes files named file_nameXXXX.xyz.vol")')
   READ (INPUT,'(A)') voro_file
   WRITE (OUTPUT,'("First file to use?")')
   READ (INPUT,*) nfirst
   WRITE (OUTPUT,'("Last file to use?")')
   READ (INPUT,*) nlast
   WRITE (OUTPUT,'("Number of molecules in file?")')
   READ (INPUT,*) nmolecules
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("Output file for volume distribution?")')
   READ (INPUT,'(A)') out1_file
   WRITE (OUTPUT,'("Output file for face distribution?")')
   READ (INPUT,'(A)') out2_file
   WRITE (OUTPUT,'("Output file for face distribution for each volume?")')
   READ (INPUT,'(A)') out3_file
   
   nframes = nlast - nfirst + 1

   ALLOCATE ( vertices(nframes, nmolecules), edges(nframes, nmolecules), faces(nframes, nmolecules), volume(nframes, nmolecules) )
   ALLOCATE ( neighbour(nframes, nmolecules, maxneighbours) )
   
 
   ! Read Voronoi files
   
   DO IFRAME = nfirst, nlast 
      WRITE (frame_number,'(I4.4)') IFRAME
      vol_file = voro_file(1:LEN_TRIM(voro_file))//frame_number//'.xyz.vol'
      UNIT = NEXT_UNIT ( )
      OPEN ( UNIT, FILE = vol_file, ACTION = "READ", STATUS = "OLD", IOSTAT = IOSTAT )
      CALL PUSH_UNIT ( UNIT )
      DO WHILE (.NOT.end) 
         CALL GET_LINE (end)
         IF (.NOT.end) THEN
	    CALL GET_INTEGER ( index )
	    IF (index > nmolecules) STOP 'Increase number of molecules'
	    CALL GET_REAL ( x )
	    CALL GET_REAL ( y )
	    CALL GET_REAL ( z )
	    CALL GET_INTEGER ( vertices(iframe-nfirst+1, index) )
	    CALL GET_INTEGER ( edges(iframe-nfirst+1, index) )
	    CALL GET_INTEGER ( faces(iframe-nfirst+1, index) )
	    CALL GET_REAL ( volume(iframe-nfirst+1, index) )
            DO I = 1, faces(iframe-nfirst+1, index)
	       CALL GET_INTEGER ( neighbour(iframe-nfirst+1, index, I) )
	    END DO
         END IF
      END DO
      CLOSE ( UNIT )
      CALL POP_UNIT 
      IF (index /= nmolecules) nmolecules = index
      end = .FALSE.
   END DO
   
   n_volumes = SIZE(volume)
   av_vertices = SUM(vertices) / REAL(n_volumes)
   av_edges = SUM(edges) / REAL(n_volumes)
   av_faces = SUM(faces) / REAL(n_volumes)
   av_volume = SUM(volume) / REAL(n_volumes)
   
   print *, 'Number of volumes = ', SIZE(volume)
   print *, 'Average number of vertices = ', av_vertices
   print *, 'Average number of edges = ', av_edges
   print *, 'Average number of faces = ', av_faces
   print *, 'Average volume = ', av_volume
   
   print *, 'Min vol = ',minval(volume)
   print *, 'Max vol = ',maxval(volume)
   
   dv = ( maxval(volume) - minval(volume) ) / REAL(n_histovol-1)
   histovol = 0
   DO IFRAME = 1, nframes
      DO I = 1, nmolecules
         bin = INT( ( volume(IFRAME, I) - minval(volume) ) / dv ) + 1
	 histovol(bin) = histovol(bin) + 1
      END DO
   END DO
   
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = out1_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT,'(''# volume - distribution '')') 
   DO I = 1, n_histovol
      WRITE (UNIT,"(1X,F9.3,4X,I6)") minval(volume) + 0.5*dv + REAL(I-1)*dv, histovol(I)
   END DO
   CLOSE (UNIT)
   print *, 'sum histovol = ',SUM(histovol)
   
   print *, 'Min faces = ', minval(faces)
   print *, 'Max faces = ', maxval(faces)
   n_histofaces = maxval(faces) - minval(faces) + 1
   ALLOCATE ( histofaces(n_histofaces) )
   histofaces = 0
   DO IFRAME = 1, nframes
      DO I = 1, nmolecules
         bin = faces(IFRAME, I) - minval(faces) + 1
	 histofaces(bin) = histofaces(bin) + 1
      END DO
   END DO
   
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = out2_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT,'(''# faces - distribution '')') 
   DO I = 1, n_histofaces
      WRITE (UNIT,"(1X,I3,4X,I6)")  minval(faces) + (I-1), histofaces(I)
   END DO
   CLOSE (UNIT)
   print *, 'sum histofaces = ',SUM(histofaces)
   
   ALLOCATE ( histovolfaces(n_histofaces, n_histovol) )
   histovolfaces = 0
   DO IFRAME = 1, nframes
      DO I = 1, nmolecules
         bin = faces(IFRAME, I) - minval(faces) + 1
	 bin2 = INT( ( volume(IFRAME, I) - minval(volume) ) / dv ) + 1
	 histovolfaces(bin,bin2) = histovolfaces(bin,bin2) + 1
      END DO
   END DO
   
   UNIT = NEXT_UNIT ( )
   OPEN ( UNIT, FILE = out3_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT,'(''# volume - distribution of faces'')') 
   WRITE (UNIT,'(''#   No. faces '',20(I5,1x))') (minval(faces)+I-1, I = 1, n_histofaces) 
   DO I = 1, n_histovol
      WRITE (UNIT,"(1X,F9.3,4X,20(I5,1x))")   minval(volume) + 0.5*dv + REAL(I-1)*dv, (histovolfaces(J,I), J = 1,n_histofaces)
   END DO
   CLOSE (UNIT)
   print *, 'sum histovolfaces = ',SUM(histovolfaces)   

   
END PROGRAM VORONOI_OUTPUT_ANALYSIS
